@@ -1,9 +1,16 @@
 #! /bin/sh
 
+TESTSUITE="@Production"
+
+if [ $1 = 'fast' ]
+then
+	TESTSUITE="@Fast"
+fi
+
 MYDIR=$(cd `dirname $0` && pwd)
 
 # run Production without JIT
-gbx3 -j -T "@Production" $MYDIR
+gbx3 -j -T "$TESTSUITE" $MYDIR
 
 if [ $? -ne 0 ] 
 then
@@ -12,7 +19,7 @@ then
 fi
 
 # run Production with JIT
-GB_NO_JIT=01 GB_JIT_CFLAGS=-O0 gbx3 -T "@Production" $MYDIR
+GB_NO_JIT=01 GB_JIT_CFLAGS=-O0 gbx3 -T "$TESTSUITE" $MYDIR
 
 if [ $? -ne 0 ] 
 then
