@@ -2,7 +2,7 @@
 
 	gb.db.h
 
-	(c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+	(c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -133,6 +133,9 @@ typedef
 
 typedef
 	void (*DB_SUBST_CALLBACK)(int, char **, int *, char);
+	
+typedef
+	void (*DB_OPTIONS_CALLBACK)(const char *, GB_VALUE *);
 
 typedef
 	struct {
@@ -232,6 +235,7 @@ typedef
 		char *(*QuoteString)(const char *, int, char);
 		char *(*UnquoteString)(const char *, int, char);
 		DB_DATABASE *(*GetCurrentDatabase)();
+		void (*GetOptions)(DB_OPTIONS_CALLBACK);
 
 		struct {
 			void (*Init)(void);
