@@ -3,7 +3,7 @@
 	main.c
 
 	(c) 2004-2007 Andrea Bortolan <andrea_bortolan@yahoo.it>
-	(c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+	(c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 	(c) 2015-2021 zxMarce <d4t4full@gmail.com>
 
 	This program is free software; you can redistribute it and/or modify
@@ -713,16 +713,18 @@ void GetConnectedDBName(DB_DESC *desc, ODBC_CONN *odbc)
 }
 
 /*****************************************************************************
- * 
+
 	open_database()
-	
+
 	Connect to a database.
 
 	<desc> points at a structure describing each connection parameter.
+	<db> points at the DB_DATABASE structure that must be initialized.
+
+	This function must return TRUE if the connection has failed.
+
+	The name of the database can be NULL, meaning a default database.
 
-	This function must return a database handle, or NULL if the connection
-	has failed.
-	
 *****************************************************************************/
 
 static int open_database(DB_DESC *desc, DB_DATABASE *db)
