@@ -2,7 +2,7 @@
 
   main.c
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
   (c) 2011-2012 Bruce Bruen <bbruen@paddys-hill.net>
 
   This program is free software; you can redistribute it and/or modify
@@ -703,6 +703,23 @@ static const char *get_quote(void)
 
 *****************************************************************************/
 
+static const char **_options_keys;
+static const char **_options_values;
+
+static void add_option(const char *key, const char *value)
+{
+	*(const char **)GB.Add(&_options_keys) = key;
+	*(const char **)GB.Add(&_options_values) = value;
+}
+
+static void add_option_value(const char *key, GB_VALUE *value)
+{
+	if (GB.Conv(value, GB_T_STRING))
+		return;
+	
+	add_option(key, value->_string.value.addr);
+}
+
 static int open_database(DB_DESC *desc, DB_DATABASE *db)
 {
 	const char *query =
@@ -712,22 +729,34 @@ static int open_database(DB_DESC *desc, DB_DATABASE *db)
 	PGresult *res;
 	int status;
 	char *name;
-	char dbname[512];
+	char buffer[16];
 
 	if (desc->name)
 		name = desc->name;
 	else
 		name = "template1";
 
-	if (snprintf(dbname, sizeof(dbname), "dbname='%s' connect_timeout=%d", get_quote_string(name, strlen(name), '\''), db->timeout) >= sizeof(dbname))
-	{
-		GB.Error("Cannot open database: database name too long");
-		return TRUE;
-	}
-
 	//fprintf(stderr, "gb.db.postgresql: host = `%s` port = `%s` dbnname = `%s` user = `%s` password = `%s`\n", desc->host, desc->port, dbname, desc->user, desc->password);
 
-	conn = PQsetdbLogin(desc->host, desc->port, NULL, NULL, dbname, desc->user, desc->password);
+	GB.NewArray(&_options_keys, sizeof(char *), 0);
+	GB.NewArray(&_options_values, sizeof(char *), 0);
+	
+	add_option("host", desc->host);
+	add_option("port", desc->port);
+	add_option("dbname", name);
+	add_option("user", desc->user);
+	add_option("password", desc->password);
+	sprintf(buffer, "%d", db->timeout);
+	add_option("connect_timeout", buffer);
+	
+	DB.GetOptions(add_option_value);
+	
+	add_option(NULL, NULL);
+	
+	conn = PQconnectdbParams((const char *const *)_options_keys, (const char *const *)_options_values, FALSE);
+	
+	GB.FreeArray(&_options_keys);
+	GB.FreeArray(&_options_values);
 
 	if (!conn)
 	{
@@ -1318,7 +1347,7 @@ static int begin_transaction(DB_DATABASE *db)
 	}
 	else
 	{
-		char buffer[8];
+		char buffer[16];
 		sprintf(buffer, "%d", trans - 1);
 		return do_query(db, "Unable to begin transaction: Unable to define savepoint: &1", NULL, "SAVEPOINT t&1", 1, buffer);
 	}
@@ -1653,16 +1682,16 @@ static void table_release(DB_DATABASE *db, DB_INFO *info)
 static int table_exist(DB_DATABASE *db, const char *table)
 {
 	const char *query =
-		"select relname from pg_class where (relkind = 'r' or relkind = 'v'or relkind = 'm') "
+		"select relname from pg_class where (relkind in ('r', 'v', 'm', 'p')) "
 		"and (relname = '&1') "
 		"and (relnamespace not in (select oid from pg_namespace where nspname = 'information_schema'))";
 
 	const char *query_schema =
-		"select relname from pg_class where (relkind = 'r' or relkind = 'v' or relkind = 'm') "
+		"select relname from pg_class where (relkind in ('r', 'v', 'm', 'p')) "
 		"and (relname = '&1') "
 		"and (relnamespace in (select oid from pg_namespace where nspname = '&2'))";
 
-		/*"select pg_class.relname,pg_namespace.nspname from pg_class,pg_namespace where (pg_class.relkind = 'r' or pg_class.relkind = 'v') "
+/*"select pg_class.relname,pg_namespace.nspname from pg_class,pg_namespace where (pg_class.relkind = 'r' or pg_class.relkind = 'v') "
 		"and (pg_namespace.oid = pg_class.relnamespace) "
 		"and (pg_class.relname = '&1') "
 		"and (pg_namespace.nspname = '&2') "
@@ -1711,7 +1740,7 @@ static int table_exist(DB_DATABASE *db, const char *table)
 static int table_list_73(DB_DATABASE *db, char ***tables)
 {
 	const char *query =
-		"select pg_class.relname,pg_namespace.nspname from pg_class,pg_namespace where (pg_class.relkind = 'r' or pg_class.relkind = 'v' or pg_class.relkind = 'm') "
+		"select pg_class.relname,pg_namespace.nspname from pg_class,pg_namespace where (pg_class.relkind in ('r', 'v', 'm', 'p')) "
 		"and (pg_namespace.oid = pg_class.relnamespace) "
 		"and (pg_namespace.oid not in (select oid from pg_namespace where nspname = 'information_schema'))";
 
@@ -1903,7 +1932,7 @@ static int table_primary_key(DB_DATABASE *db, const char *table, char ***primary
 static int table_is_system(DB_DATABASE *db, const char *table)
 {
 	const char *query =
-		"select 1 from pg_class where (relkind = 'r' or relkind = 'v'or relkind = 'm') "
+		"select 1 from pg_class where (relkind in ('r', 'v', 'm', 'p')) "
 		"and (relname = '&1') "
 		"and (relnamespace in (select oid from pg_namespace where nspname = 'pg_catalog'))";
 
