@@ -2,7 +2,7 @@
 
 	main.c
 
-	(c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+	(c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -547,13 +547,15 @@ static const char *get_quote(void)
 	Connect to a database.
 
 	<desc> points at a structure describing each connection parameter.
+	<db> points at the DB_DATABASE structure that must be initialized.
+
+	This function must return TRUE if the connection has failed.
+
+	The name of the database can be NULL, meaning a default database.
 
 	In Sqlite, there is no such thing as a host.  If this is set then check
 	to see whether this is actually a path to a home area. NG 01/04/04
 
-	This function must return a database handle, or NULL if the connection
-	has failed.
-
 *****************************************************************************/
 
 static int open_database(DB_DESC *desc, DB_DATABASE *db)
