@@ -2,17 +2,11 @@
 
   CImage.h
 
-  (c) 2004-2006 - Daniel Campos Fernández <dcamposf@gmail.com>
-  
-  GTK+ component
-  
-  Realizado para la Junta de Extremadura. 
-  Consejería de Educación Ciencia y Tecnología. 
-  Proyecto gnuLinEx
-  
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -22,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -30,22 +25,31 @@
 #define __CIMAGE_H
 
 #include "gambas.h"
+#include "gb.image.h"
+#include "widgets.h"
 
-typedef  struct 
-{
-    GB_BASE ob;
-	gImage *image;
-
-}  CIMAGE;
+typedef 
+  struct 
+  {
+    GB_IMG img;
+  }
+  CIMAGE;
 
 #ifndef __CIMAGE_CPP
 
 extern GB_DESC CImageDesc[];
 
-#endif
+#else
 
-#define IMAGE (((CIMAGE*)(_object))->image)
+#define THIS ((CIMAGE *)_object)
+#define PICTURE ((gPicture *)THIS->img.temp_handle)
+#define GET_PICTURE(_image) ((gPicture *)(_image->img.temp_handle))
 
-void* GTK_GetImage(GdkPixbuf *buf);
+#endif
+
+void *GTK_GetImage(GdkPixbuf *buf);
+
+CIMAGE *CIMAGE_create(gPicture *picture);
+gPicture *CIMAGE_get(CIMAGE *);
 
 #endif
