@@ -3,16 +3,10 @@
   CWidget.h
 
   (c) 2004-2006 - Daniel Campos Fernández <dcamposf@gmail.com>
-  
-  GTK+ component
-  
-  Realizado para la Junta de Extremadura. 
-  Consejería de Educación Ciencia y Tecnología. 
-  Proyecto gnuLinEx
-  
+
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -22,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -31,7 +26,11 @@
 
 #include "main.h"
 #include "gcontrol.h"
+
+#ifdef GTK3
+#else
 #include "gplugin.h"
+#endif
 
 #ifndef __CWIDGET_CPP
 
@@ -54,17 +53,25 @@ typedef
 	  gControl *widget;
 		GB_VARIANT_VALUE tag;
 		void *font;
+		void *cursor;
+		char *popup;
+		char *action;
 	}  
 	CWIDGET;
 
-typedef  struct
-{
-  GB_BASE ob;
-  gPlugin *widget;
-	GB_VARIANT_VALUE tag;
-	void *font;
-}  CPLUGIN;
-
+#ifdef GTK3
+#else
+typedef
+	struct
+	{
+		GB_BASE ob;
+		gPlugin *widget;
+		GB_VARIANT_VALUE tag;
+		void *font;
+		char *popup;
+	}
+	CPLUGIN;
+#endif
 
 void InitControl(gControl *control, CWIDGET *widget);
 void DeleteControl(gControl *control);
@@ -77,9 +84,11 @@ CWIDGET *GetContainer(CWIDGET *control);
 
 DECLARE_PROPERTY(CCONTROL_action);
 
-void CACTION_register(void *control, const char *key);
+int CWIDGET_check(void *_object);
+int CWIDGET_get_handle(void *_object);
+
+void CACTION_register(void *control, const char *old, const char *key);
 void CACTION_raise(void *control);
-void CACTION_get(void *control);
 
 #endif
 
