@@ -48,13 +48,13 @@ static void cb_click(GtkButton *object, gButton *data)
 		}
 	}
 
-	data->emit(SIGNAL(data->onClick));
+	CB_button_click(data);
 }
 
-static void cb_click_radio(GtkButton *object,gControl *data)
+static void cb_click_radio(GtkButton *object, gControl *data)
 {
-	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(object)))
-		if (((gButton*)data)->onClick) ((gButton*)data)->onClick((gControl*)data);
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(object)))
+		CB_button_click(data);
 	return;
 }
 
@@ -73,7 +73,7 @@ static void cb_click_check(GtkButton *object, gButton *data)
 		data->unlock();
 	}
 
-	data->emit(SIGNAL(data->onClick));
+	CB_button_click(data);
 }
 
 #ifdef GTK3
@@ -89,7 +89,7 @@ static gboolean button_expose(GtkWidget *wid, GdkEventExpose *e, gButton *data)
 	int x, w, wt, wp, hp;
 	int d = gDesktop::scale() / 2;
 	
-	rtl = gtk_widget_get_default_direction() == GTK_TEXT_DIR_RTL;
+	rtl = gtk_widget_get_direction(wid) == GTK_TEXT_DIR_RTL;
 	
 	#ifdef GTK3
 		GtkStateFlags f = gtk_widget_get_state_flags(data->widget);
@@ -304,7 +304,6 @@ gButton::gButton(gContainer *par, Type typ) : gControl(par)
 	realize();
 	
 	gtk_widget_add_events(widget, GDK_POINTER_MOTION_MASK);
-	onClick = NULL;
 	
 	if (type == Radio)
 		g_signal_connect(G_OBJECT(widget),"clicked",G_CALLBACK(cb_click_radio),(gpointer)this);
@@ -402,7 +401,7 @@ void gButton::setText(const char *st)
 			gtk_button_set_label(GTK_BUTTON(widget), "");
 
 		_label = gtk_bin_get_child(GTK_BIN(widget));
-		gt_widget_set_inverted(_label, false);
+		updateDirection();
 		#ifndef GTK3
 		set_gdk_fg_color(_label, foreground());
 		#endif
@@ -739,14 +738,11 @@ void gButton::updateSize()
 	resize(mw, mh);
 }
 
-bool gButton::setInverted(bool v)
+void gButton::updateDirection()
 {
-	if (gControl::setInverted(v))
-		return true;
-	
+	gControl::updateDirection();
 	if (_label)
-		gt_widget_set_inverted(_label, false);
-	return false;
+		gtk_widget_set_direction(_label, gtk_widget_get_direction(widget));
 }
 
 gColor gButton::defaultBackground() const
