@@ -451,7 +451,7 @@ void gContainer::setUser()
 void gContainer::setPaint()
 {
 	arrangement.paint = true;
-	ON_DRAW(border, this, cb_expose, cb_draw);
+	ON_DRAW_BEFORE(border, this, cb_expose, cb_draw);
 }
 
 void gContainer::setInvert(bool vl)
@@ -781,6 +781,9 @@ bool gContainer::resize(int w, int h, bool no_decide)
 	_client_w = 0;
 	_client_h = 0;
 	
+	if (arrangement.paint)
+		CUSERCONTROL_cb_resize(this);
+
 	performArrange();
 	return false;
 }
