@@ -2,7 +2,7 @@
 
   gdrag.h
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -24,6 +24,8 @@
 #ifndef __GDRAG_H
 #define __GDRAG_H
 
+#include "gb.form.const.h"
+
 class gPicture;
 class gControl;
 
@@ -36,11 +38,6 @@ public:
 		Text = 1,
 		Image = 2
 	};
-	enum {
-		Copy = 0,
-		Move = 1,
-		Link = 2
-	};
 	
 	static void exit();
 	
