@@ -159,7 +159,7 @@ static gboolean cb_unmap(GtkWidget *widget, GdkEvent *event, gMainWindow *data)
 
 static gboolean cb_close(GtkWidget *widget,GdkEvent *event, gMainWindow *data)
 {
-	if (!gMainWindow::_current || data == gMainWindow::_current)
+	if ((!gMainWindow::_current || data == gMainWindow::_current) && gApplication::areInputEventsEnabled())
 		data->doClose();
 
 	return true;
@@ -258,12 +258,9 @@ static gboolean cb_configure(GtkWidget *widget, GdkEventConfigure *event, gMainW
 #ifdef GTK3
 static gboolean cb_draw(GtkWidget *wid, cairo_t *cr, gMainWindow *data)
 {
-	if (data->isTransparent())
+	if (data->background() != COLOR_DEFAULT)
 	{
-		if (data->background() == COLOR_DEFAULT)
-			gt_cairo_set_source_color(cr, 0XFF000000);
-		else
-			gt_cairo_set_source_color(cr, data->background());
+		gt_cairo_set_source_color(cr, data->background());
 		cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
 		cairo_paint(cr);
 	}
@@ -1461,6 +1458,7 @@ void gMainWindow::reparent(gContainer *newpr, int x, int y)
 
 		setParent(newpr);
 		connectParent();
+		//newpr->insert(this);
 		borderSignals();
 
 		initWindow();
