@@ -231,8 +231,6 @@ GB_DESC *GB_CLASSES[] EXPORT =
 	CTextBoxDesc,
 	CTextAreaDesc,
 	CTextAreaSelectionDesc,
-	CComboBoxDesc,
-	CComboBoxItemDesc,
 	CTabStripDesc,
 	CTabStripContainerDesc,
 	CTabStripContainerChildrenDesc,
@@ -404,7 +402,8 @@ void hook_quit (void)
 {
 	GB_FUNCTION func;
 
-	while (gtk_events_pending())
+	while (!gApplication::processInputEvent());
+	while (gApplication::eventsPending())
 		gtk_main_iteration();
 
 	if (GB.ExistClass("TrayIcons"))
@@ -449,6 +448,7 @@ static void hook_main(int *argc, char ***argv)
 	gtk_init(argc, argv);
 	gApplication::init(argc, argv);
 	gApplication::setDefaultTitle(GB.Application.Title());
+	CAPPLICATION_MiddleClickPaste = gApplication::hasMiddleClickPaste();
 
 	gApplication::onEnterEventLoop = GB.Debug.EnterEventLoop;
 	gApplication::onLeaveEventLoop = GB.Debug.LeaveEventLoop;
@@ -578,23 +578,30 @@ static void hook_wait(int duration)
 		return;
 	}
 
-	if (duration && gKey::isValid())
+	if (duration != -1 && gKey::isValid())
 	{
 		if (!_warning)
 		{
-			fprintf(stderr, "gb.gtk: warning: calling the event loop during a keyboard event handler is ignored\n");
+			fprintf(stderr, "gb.gtk3: warning: calling the event loop during a keyboard event handler is ignored\n");
 			_warning = TRUE;
 		}
 		return;
 	}
 
-	if (duration == 0)
+	if (duration >= 0)
 	{
-		while (gtk_events_pending())
-			MAIN_do_iteration(false);
+		MAIN_do_iteration(true);
 	}
-	else
-		MAIN_do_iteration(duration > 0);
+	else if (duration == -1)
+	{
+		bool d = gApplication::disableInputEvents(true);
+		MAIN_do_iteration(true); // To process deferred routines
+		while (gApplication::eventsPending())
+			MAIN_do_iteration(false);
+		gApplication::disableInputEvents(d);
+	}
+	else if (duration == -2)
+		MAIN_do_iteration(false);
 }
 
 static void hook_watch(int fd, int type, void *callback, intptr_t param)
@@ -655,17 +662,22 @@ static void hook_lang(char *lang, int rtl)
 
 void MAIN_do_iteration_just_events()
 {
-	if (gtk_events_pending())
+	if (gApplication::eventsPending())
+	{
+		while (!gApplication::processInputEvent());
 		gtk_main_iteration_do(false);
+	}
 }
 
 void MAIN_do_iteration(bool do_not_block)
 {
 	gApplication::_loopLevel++;
 
+	while (!gApplication::processInputEvent());
+	
 	if (do_not_block)
 	{
-		if (gtk_events_pending())
+		if (gApplication::eventsPending())
 			gtk_main_iteration();
 	}
 	else
@@ -673,6 +685,8 @@ void MAIN_do_iteration(bool do_not_block)
 
 	gApplication::_loopLevel--;
 
+	gContainer::postArrange();
+
 	if (_post_check)
 	{
 		_post_check = false;
@@ -680,6 +694,5 @@ void MAIN_do_iteration(bool do_not_block)
 	}
 
 	gControl::postDelete();
-	gContainer::postArrange();
 }
 
