@@ -169,6 +169,13 @@ static void get_client_area(gContainer *cont, int *x, int *y, int *w, int *h)
 	}
 }
 
+void CUSERCONTROL_cb_resize(gContainer *sender)
+{
+	CWIDGET *_object = GetObject(sender);
+	CALL_FUNCTION(THIS_USERCONTROL, resize_func);
+}
+
+//-------------------------------------------------------------------------
 
 BEGIN_PROPERTY(Container_ClientX)
 
@@ -494,6 +501,8 @@ BEGIN_METHOD(UserControl_new, GB_OBJECT parent)
 			THIS_USERCONTROL->font_func = func.index;
 		if (!GB.GetFunction(&func, THIS, "UserControl_Change", NULL, NULL))
 			THIS_USERCONTROL->change_func = func.index;
+		if (!GB.GetFunction(&func, THIS, "UserControl_Resize", NULL, NULL))
+			THIS_USERCONTROL->resize_func = func.index;
 	}
 	
 	GB.Error(NULL);
@@ -565,6 +574,16 @@ BEGIN_PROPERTY(UserControl_Container)
 END_PROPERTY
 
 
+BEGIN_PROPERTY(UserControl_Focus)
+
+	if (READ_PROPERTY)
+		GB.ReturnBoolean(WIDGET->canFocus());
+	else
+		WIDGET->setCanFocus(VPROP(GB_BOOLEAN));
+
+END_PROPERTY
+
+
 BEGIN_PROPERTY(UserContainer_Container)
 
 	if (READ_PROPERTY)
@@ -699,6 +718,7 @@ GB_DESC UserControlDesc[] =
 	GB_PROPERTY("_Indent", "b", Container_Indent),
 	GB_PROPERTY("_Invert", "b", Container_Invert),
 	GB_PROPERTY("_Centered", "b", Container_Centered),
+	GB_PROPERTY("_Focus", "b", UserControl_Focus),
 
 	USERCONTROL_DESCRIPTION,
 
