@@ -49,6 +49,7 @@
 
 #ifdef GTK3
 static GtkApplication *_app;
+GtkStyleProvider *_tooltip_css = NULL;
 #endif
 
 static bool _debug_keypress = false;
@@ -252,10 +253,30 @@ static void gambas_handle_event(GdkEvent *event)
 			fprintf(stderr, "**** GDK_GRAB_BROKEN inside popup: %s %swindow = %p grab_window = %p popup_window = %p\n", event->grab_broken.keyboard ? "keyboard" : "pointer",
 							event->grab_broken.implicit ? "implicit " : "", event->grab_broken.window, event->grab_broken.grab_window, gApplication::_popup_grab_window);
 	}*/
-
+	
 	if (!((event->type >= GDK_MOTION_NOTIFY && event->type <= GDK_FOCUS_CHANGE) || event->type == GDK_SCROLL))
 		goto __HANDLE_EVENT;
 
+	if (gApplication::_disable_input_events)
+	{
+		switch(event->type)
+		{
+			case GDK_ENTER_NOTIFY:
+			case GDK_LEAVE_NOTIFY:
+			case GDK_BUTTON_PRESS:
+			case GDK_2BUTTON_PRESS:
+			case GDK_BUTTON_RELEASE:
+			case GDK_MOTION_NOTIFY:
+			case GDK_SCROLL:
+			case GDK_KEY_PRESS:
+			case GDK_KEY_RELEASE:
+				//gApplication::pushInputEvent(event);
+				return;
+			default:
+				;
+		}
+	}
+
 	widget = gtk_get_event_widget(event);
 	
 	if (!widget)
@@ -919,6 +940,9 @@ GdkEvent *gApplication::_event = NULL;
 bool gApplication::_keep_focus = false;
 bool gApplication::_disable_mapping_events = false;
 
+bool gApplication::_disable_input_events = false;
+//GQueue *gApplication::_input_events = NULL;
+
 bool gApplication::_fix_breeze = false;
 bool gApplication::_fix_oxygen = false;
 int gApplication::_scrollbar_size = 0;
@@ -946,6 +970,26 @@ void gApplication::ungrabPopup()
 	}
 }
 
+#ifdef GTK3
+
+bool gApplication::areTooltipsEnabled()
+{
+	return _tooltip_css == NULL;
+}
+
+void gApplication::enableTooltips(bool vl)
+{
+	if (vl == areTooltipsEnabled())
+		return;
+
+	gt_define_style_sheet(&_tooltip_css, NULL);
+
+	if (!vl)
+		gt_define_style_sheet(&_tooltip_css, g_string_new("tooltip { opacity: 0; }"));
+}
+
+#else
+
 bool gApplication::areTooltipsEnabled()
 {
   gboolean enabled;
@@ -966,6 +1010,8 @@ void gApplication::enableTooltips(bool vl)
   g_object_set (settings, "gtk-enable-tooltips", enabled, (char *)NULL);
 }
 
+#endif
+
 static void do_nothing()
 {
 }
@@ -1166,16 +1212,15 @@ void gApplication::enterLoop(void *owner, bool showIt, GtkWindow *modal)
 {
 	void *old_owner = _loop_owner;
 	int l = _loopLevel;
-	//GtkWindowGroup *oldGroup;
+	bool d; 
 
 	if (showIt) ((gControl *)owner)->show();
 
-	//oldGroup = enterGroup();
-
 	_loopLevel++;
 	_loop_owner = owner;
 	setButtonGrab(NULL);
 
+	d = gApplication::disableInputEvents(false);
 	(*onEnterEventLoop)();
 	do
 	{
@@ -1183,17 +1228,16 @@ void gApplication::enterLoop(void *owner, bool showIt, GtkWindow *modal)
 	}
 	while (_loopLevel > l);
 	(*onLeaveEventLoop)();
+	gApplication::disableInputEvents(d);
 
 	_loop_owner = old_owner;
-
-	//exitGroup(oldGroup);
 }
 
 void gApplication::enterPopup(gMainWindow *owner)
 {
 	void *old_owner;
 	int l;
-	//GtkWindowGroup *oldGroup;
+	bool d;
 	GtkWindow *window = GTK_WINDOW(owner->border);
 	GtkWidget *old_popup_grab;
 
@@ -1202,8 +1246,6 @@ void gApplication::enterPopup(gMainWindow *owner)
 	// Remove possible current button grab
 	gApplication::setButtonGrab(NULL);
 //
-	//oldGroup = enterGroup();
-
 	gtk_window_set_modal(window, true);
 	owner->show();
 	gdk_window_set_override_redirect(gtk_widget_get_window(owner->border), true);
@@ -1215,7 +1257,6 @@ void gApplication::enterPopup(gMainWindow *owner)
 
 		if (_in_popup == 1)
 			owner->_grab_on_show = TRUE;
-			//gApplication::grabPopup();
 
 		l = _loopLevel;
 		old_owner = _loop_owner;
@@ -1223,6 +1264,7 @@ void gApplication::enterPopup(gMainWindow *owner)
 		_loopLevel++;
 		_loop_owner = owner;
 
+		d = gApplication::disableInputEvents(false);
 		(*onEnterEventLoop)();
 		//fprintf(stderr, "event loop <----\n");
 		do
@@ -1232,6 +1274,7 @@ void gApplication::enterPopup(gMainWindow *owner)
 		while (_loopLevel > l);
 		//fprintf(stderr, "event loop ---->\n");
 		(*onLeaveEventLoop)();
+		gApplication::disableInputEvents(d);
 
 		if (_in_popup == 1)
 		{
@@ -1250,10 +1293,7 @@ void gApplication::enterPopup(gMainWindow *owner)
 			gdk_window_set_override_redirect(gtk_widget_get_window(owner->border), false);
 			gtk_window_set_modal(window, false);
 		}
-		//exitGroup(oldGroup);
 	}
-	/*else
-		gControl::postDelete();*/
 
 	_in_popup--;
 }
@@ -1726,3 +1766,58 @@ void gApplication::forEachControl(void (*cb)(gControl *), bool (*filter)(gContro
 		}
 	}
 }
+
+#if 0
+void gApplication::pushInputEvent(GdkEvent *event)
+{
+	if (!_input_events)
+		_input_events = g_queue_new();
+	
+	g_queue_push_tail(_input_events, gdk_event_copy(event));
+}
+
+bool gApplication::processInputEvent()
+{
+	GdkEvent *event;
+	
+	if (!areInputEventsEnabled())
+		return true;
+	
+	if (!_input_events || g_queue_is_empty(_input_events))
+		return true;
+	
+	event = (GdkEvent *)g_queue_pop_head(_input_events);
+	gambas_handle_event(event);
+	gdk_event_free(event);
+	return false;
+}
+#endif
+
+bool gApplication::disableInputEvents(bool disable)
+{
+	bool d = _disable_input_events;
+	_disable_input_events = disable;
+	return d;
+}
+
+bool gApplication::eventsPending()
+{
+	/*if (areInputEventsEnabled() && _input_events && !g_queue_is_empty(_input_events))
+		return true;*/
+	
+	return gtk_events_pending();
+		
+}
+
+bool gApplication::hasMiddleClickPaste()
+{
+  gboolean enabled;
+  GtkSettings *settings;
+
+  settings = gtk_settings_get_default();
+
+  g_object_get(settings, "gtk-enable-primary-paste", &enabled, (char *)NULL);
+
+  return enabled;
+}
+
