@@ -3,6 +3,7 @@
 	gcontrol.cpp
 
 	(c) 2004-2006 - Daniel Campos Fernández <dcamposf@gmail.com>
+	(c) Benoît Minisini <benoit.minisini@gambas-basic.org>
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -308,11 +309,6 @@ void gControl::postDelete()
 	_destroy_list = NULL;
 }
 
-static bool always_can_raise(gControl *sender, int type)
-{
-	return true;
-}
-
 void gControl::initAll(gContainer *parent)
 {
 	bufW = 1;
@@ -333,7 +329,6 @@ void gControl::initAll(gContainer *parent)
 	_inverted = false;
 	_accept_drops = false;
 	_dragging = false;
-	_drag_get_data = false;
 	frame_border = 0;
 	frame_padding = 0;
 	_bg_set = false;
@@ -367,19 +362,9 @@ void gControl::initAll(gContainer *parent)
 	_is_drawingarea = false;
 	_has_native_popup = false;
 	_eat_return_key = false;
-	_hidden_temp = false;
+	//_hidden_temp = false;
 	_allow_show = false;
-
-	onFinish = NULL;
-	onFocusEvent = NULL;
-	onKeyEvent = NULL;
-	onMouseEvent = NULL;
-	onDrag = NULL;
-	onDragMove = NULL;
-	onDrop = NULL;
-	onDragLeave = NULL;
-	onEnterLeave = NULL;
-	canRaise = always_can_raise;
+	_direction = DIRECTION_DEFAULT;
 
 	frame = border = widget = NULL;
 	_scroll = NULL;
@@ -410,8 +395,16 @@ void gControl::dispose()
 	gMainWindow *win;
 	
 	win = window();
-	if (win && win->focus == this)
-		win->focus = NULL;
+	if (win && win->_initial_focus == this)
+		win->_initial_focus = NULL;
+	
+	win = gMainWindow::_current;
+	while (win)
+	{
+		if (win && win->_save_focus == this)
+			win->_save_focus = NULL;
+		win = win->_previous;
+	}
 
 	if (pr)
 	{
@@ -423,7 +416,7 @@ void gControl::dispose()
 gControl::~gControl()
 {
 	//fprintf(stderr, "~gControl: %s %p %s\n", name(), this, GB.GetClassName(hFree));
-	emit(SIGNAL(onFinish));
+	CB_control_finish(this);
 
 	dispose();
 
@@ -550,8 +543,12 @@ void gControl::setVisibility(bool vl)
 	}
 	else
 	{
-		if (parent() && hasFocus())
-			gcb_focus(widget, GTK_DIR_TAB_FORWARD, this);
+		if (hasFocus())
+		{
+			if (parent())
+				gcb_focus(widget, GTK_DIR_TAB_FORWARD, this);
+			gApplication::setActiveControl(this, false);
+		}
 		if (gtk_widget_has_grab(border))
 			gtk_grab_remove(border);
 		gtk_widget_hide(border);
@@ -644,20 +641,15 @@ int gControl::screenY()
 	return pr->screenY() + y() + pr->clientY() - pr->scrollY();
 }
 
-static void send_configure (gControl *control)
+static void send_configure(gControl *control)
 {
 	GtkWidget *widget;
 	GdkEvent *event;
 
 	widget = control->border;
 
-#if GTK_CHECK_VERSION(2, 20, 0)
 	if (!gtk_widget_get_realized(widget))
 		return;
-#else
-	if (!GTK_WIDGET_REALIZED(widget))
-		return;
-#endif
 
 // 	if (control->isWindow())
 // 	 g_debug("send configure to window: %s", control->name());
@@ -700,9 +692,6 @@ void gControl::move(int x, int y)
 	updateGeometry();
 	#endif
 	
-	/*if (name() && !::strcmp(name(), "txtTagEditor") && y == 43)
-		BREAKPOINT();*/
-
 	checkVisibility();
 	
 	send_configure(this); // needed for Watcher and Form Move events
@@ -712,10 +701,14 @@ void gControl::hideButKeepFocus()
 {
 	//fprintf(stderr, "gControl::hideButKeepFocus: %s\n", gApplication::_active_control ? gApplication::_active_control->name() : "NULL");
 
-	_hidden_temp = true;
-	gApplication::_keep_focus = true;
-	gtk_widget_hide(border);
-	gApplication::_keep_focus = false;
+	if (gtk_widget_get_visible(border))
+	{
+		gApplication::_keep_focus = true;
+		gApplication::_disable_mapping_events = true;
+		gtk_widget_hide(border);
+		gApplication::_disable_mapping_events = false;
+		gApplication::_keep_focus = false;
+	}
 }
 
 void gControl::showButKeepFocus()
@@ -724,19 +717,20 @@ void gControl::showButKeepFocus()
 
 	//fprintf(stderr, "gControl::showButKeepFocus: %s\n", gApplication::_active_control ? gApplication::_active_control->name() : "NULL");
 
-	if (_allow_show)
+	if (_allow_show && !gtk_widget_get_visible(border))
+	{
+		gApplication::_disable_mapping_events = true;
 		gtk_widget_show(border);
+		gApplication::_disable_mapping_events = false;
+	}
 	
 	focus = gApplication::_active_control;
-	if (focus)
-	{
-		gApplication::_active_control = NULL;
-		if (!focus->hasFocus())
-			focus->setFocus();
-		gApplication::_active_control = focus;
-	}
+	gApplication::_active_control = NULL;
+	if (focus && !focus->hasFocus())
+		focus->setFocus();
+	gApplication::_active_control = focus;
 
-	_hidden_temp = false;
+	//_hidden_temp = false;
 }
 
 bool gControl::resize(int w, int h, bool no_decide)
@@ -757,8 +751,8 @@ bool gControl::resize(int w, int h, bool no_decide)
 			h = height();
 	}
 
-	if (w <= 0) w = 1;
-	if (h <= 0) h = 1;
+	if (w < 0) w = 0;
+	if (h < 0) h = 0;
 
 	if (width() == w && height() == h)
 		return true;
@@ -772,15 +766,6 @@ bool gControl::resize(int w, int h, bool no_decide)
 	}
 	else
 	{
-		/*if (frame && widget != border)
-		{
-			int fw = getFrameWidth() * 2;
-			if (w < fw || h < fw)
-				gtk_widget_hide(widget);
-			else
-				gtk_widget_show(widget);
-		}*/
-
 		//g_debug("resize: %p %s: %d %d", this, name(), w, h);
 		_dirty_size = true;
 
@@ -949,10 +934,21 @@ void gControl::resolveFont()
 void gControl::setFont(gFont *ft)
 {
 	//fprintf(stderr, "setFont: %s: %s\n", name(), ft->toFullString());
+	
 	if (ft)
+	{
+		if (_font && _font->equals(ft))
+			return;
+		
 		gFont::assign(&_font, ft);
-	else if (_font)
+	}
+	else
+	{
+		if (!_font)
+			return;
+		
 		gFont::assign(&_font);
+	}
 
 	gFont::assign(&_resolved_font);
 
@@ -1035,18 +1031,11 @@ void gControl::updateCursor(GdkCursor *cursor)
 {
 	if (GDK_IS_WINDOW(gtk_widget_get_window(border)) && _inside)
 	{
-		#if DEBUG_ENTER_LEAVE
-		fprintf(stderr, "updateCursor: %s %p\n", name(), cursor);
-		#endif
-		if (!cursor && parent() && gtk_widget_get_window(parent()->border) == gtk_widget_get_window(border))
-			parent()->updateCursor(parent()->getGdkCursor());
-		else
-		{
-			#if DEBUG_ENTER_LEAVE
-			fprintf(stderr, "updateCursor: gdk_window_set_cursor: window = %p\n", gtk_widget_get_window(border));
-			#endif
+		if (cursor || isWindow())
 			gdk_window_set_cursor(gtk_widget_get_window(border), cursor);
-		}
+		
+		if (!cursor && parent())
+			parent()->updateCursor(parent()->getGdkCursor());
 	}
 }
 
@@ -1057,7 +1046,7 @@ GdkCursor *gControl::getGdkCursor()
 	int m = _mouse;
 
 	if (gApplication::isBusy())
-		m = GDK_WATCH;
+		m = CURSOR_WAIT;
 
 	if (m == CURSOR_CUSTOM)
 	{
@@ -1069,52 +1058,47 @@ GdkCursor *gControl::getGdkCursor()
 	{
 		switch(m)
 		{
-			case GDK_BLANK_CURSOR: name = "none"; break;
-			case GDK_LEFT_PTR: name = "default"; break;
-			case GDK_CROSSHAIR: name = "crosshair"; break;
-			case GDK_WATCH: name = "wait"; break;
-			case GDK_XTERM: name = "text"; break;
-			case GDK_FLEUR: name = "move"; break;
-			case GDK_SB_H_DOUBLE_ARROW: name = "ew-resize"; break;
-			case GDK_SB_V_DOUBLE_ARROW: name = "ns-resize"; break;
-			case GDK_TOP_SIDE: name = "n-resize"; break;
-			case GDK_BOTTOM_SIDE: name = "s-resize"; break;
-			case GDK_LEFT_SIDE: name = "w-resize"; break;
-			case GDK_RIGHT_SIDE: name = "e-resize"; break;
-			case GDK_TOP_LEFT_CORNER: name = "nw-resize"; break;
-			case GDK_BOTTOM_RIGHT_CORNER: name = "se-resize"; break;
-			case GDK_TOP_RIGHT_CORNER: name = "ne-resize"; break;
-			case GDK_BOTTOM_LEFT_CORNER: name = "sw-resize"; break;
-			case GDK_LAST_CURSOR+1: name = "nwse-resize"; break;
-			case GDK_LAST_CURSOR+2: name = "nesw-resize"; break;
-			case GDK_HAND2: name = "pointer"; break;
+			case CURSOR_NONE: name = "none"; break;
+			case CURSOR_ARROW: name = "default"; break;
+			case CURSOR_HELP: name = "help"; break;
+			case CURSOR_POINTER: name = "pointer"; break;
+			case CURSOR_CONTEXT_MENU: name = "context-menu"; break;
+			case CURSOR_PROGRESS: name = "progress"; break;
+			case CURSOR_WAIT: name = "wait"; break;
+			case CURSOR_CELL: name = "cell"; break;
+			case CURSOR_CROSSHAIR: name = "crosshair"; break;
+			case CURSOR_TEXT: name = "text"; break;
+			case CURSOR_VERTICAL_TEXT: name = "vertical-text"; break;
+			case CURSOR_ALIAS: name = "alias"; break;
+			case CURSOR_COPY: name = "copy"; break;
+			case CURSOR_NO_DROP: name = "no-drop"; break;
+			case CURSOR_MOVE: name = "move"; break;
+			case CURSOR_NOT_ALLOWED: name = "not-allowed"; break;
+			case CURSOR_GRAB: name = "grab"; break;
+			case CURSOR_GRABBING: name = "grabbing"; break;
+			case CURSOR_ALL_SCROLL: name = "all-scroll"; break;
+			case CURSOR_COL_RESIZE: name = "col-resize"; break;
+			case CURSOR_ROW_RESIZE: name = "row-resize"; break;
+			case CURSOR_N_RESIZE: name = "n-resize"; break;
+			case CURSOR_E_RESIZE: name = "e-resize"; break;
+			case CURSOR_S_RESIZE: name = "s-resize"; break;
+			case CURSOR_W_RESIZE: name = "w-resize"; break;
+			case CURSOR_NE_RESIZE: name = "ne-resize"; break;
+			case CURSOR_NW_RESIZE: name = "nw-resize"; break;
+			case CURSOR_SW_RESIZE: name = "sw-resize"; break;
+			case CURSOR_SE_RESIZE: name = "se-resize"; break;
+			case CURSOR_EW_RESIZE: name = "ew-resize"; break;
+			case CURSOR_NS_RESIZE: name = "ns-resize"; break;
+			case CURSOR_NESW_RESIZE: name = "nesw-resize"; break;
+			case CURSOR_NWSE_RESIZE: name = "nwse-resize"; break;
+			case CURSOR_ZOOM_IN: name = "zoom-in"; break;
+			case CURSOR_ZOOM_OUT: name = "zoom-out"; break;
 			default: name = "default";
 		}
 
 		cr = gdk_cursor_new_from_name(gdk_display_get_default(), name);
 		if (!cr)
 			cr = gdk_cursor_new_for_display(gdk_display_get_default(), (GdkCursorType)m);
-
-		/*
-		if (m < GDK_LAST_CURSOR)
-		{
-			cr = gdk_cursor_new_for_display(gdk_display_get_default(), (GdkCursorType)m);
-		}
-		else
-		{
-			if (m == (GDK_LAST_CURSOR+1)) //FDiag
-			{
-				pix = gdk_pixbuf_new_from_xpm_data(_cursor_fdiag);
-				cr = gdk_cursor_new_from_pixbuf(gdk_display_get_default(), pix, 8, 8);
-				g_object_unref(pix);
-			}
-			else if (m == (GDK_LAST_CURSOR+2)) //BDiag
-			{
-				pix = gdk_pixbuf_new_from_xpm_data(_cursor_bdiag);
-				cr = gdk_cursor_new_from_pixbuf(gdk_display_get_default(), pix, 8, 8);
-				g_object_unref(pix);
-			}
-		}*/
 	}
 
 	return cr;
@@ -1128,14 +1112,8 @@ void gControl::setMouse(int m)
 		return;
 	}
 
-	if (m == CURSOR_CUSTOM)
-	{
-		if (!curs || !curs->cur)
-			m = CURSOR_DEFAULT;
-	}
-
+	//fprintf(stderr, "setMouse: %s\n", name());
 	_mouse = m;
-
 	updateCursor(getGdkCursor());
 }
 
@@ -1171,6 +1149,7 @@ gMainWindow* gControl::topLevel() const
 long gControl::handle()
 {
 #ifdef GTK3
+	gtk_widget_realize(border);
 	GdkWindow *window = gtk_widget_get_window(border);
 	return PLATFORM.Window.GetId(window);
 #else
@@ -1192,6 +1171,7 @@ MISC
 
 void gControl::refresh()
 {
+	//fprintf(stderr, "%s: refresh %s\n", GB.Debug.GetCurrentPosition(), name());
 	gtk_widget_queue_draw(border);
 	if (frame != border && GTK_IS_WIDGET(frame))
 		gtk_widget_queue_draw(frame);
@@ -1269,15 +1249,9 @@ gControl *gControl::ignoreDesign()
 
 bool gControl::canFocus() const
 {
-	#if DEBUG_FOCUS
-	fprintf(stderr, "canFocus: %s ?\n", name());
-	#endif
-	/*if (_proxy)
-		return _proxy->canFocus();*/
-	
-	#if DEBUG_FOCUS
+	/*#if DEBUG_FOCUS
 	fprintf(stderr, "canFocus: %s -> %d\n", name(), gtk_widget_get_can_focus(widget));
-	#endif
+	#endif*/
 	
 #if GTK_CHECK_VERSION(2, 18, 0)
 	return gtk_widget_get_can_focus(widget);
@@ -1299,9 +1273,9 @@ bool gControl::canFocusOnClick() const
 
 void gControl::setCanFocus(bool vl)
 {
-	#if DEBUG_FOCUS
+	/*#if DEBUG_FOCUS
 	fprintf(stderr, "setCanFocus: %s %d ?\n", name(), vl);
-	#endif
+	#endif*/
 	if (isDesign() || vl == canFocus())
 		return;
 
@@ -1309,9 +1283,9 @@ void gControl::setCanFocus(bool vl)
 		_proxy->setCanFocus(vl);
 	else*/
 	{
-		#if DEBUG_FOCUS
+		/*#if DEBUG_FOCUS
 		fprintf(stderr, "setCanFocus: %s %p %d\n", name(), this, vl);
-		#endif
+		#endif*/
 		gtk_widget_set_can_focus(widget, vl);
 	}
 
@@ -1341,7 +1315,7 @@ void gControl::setFocus()
 		return;
 	}
 
-	if (hasFocus())
+	if (!canFocus() || hasFocus())
 		return;
 	
 	gMainWindow *win = window();
@@ -1351,20 +1325,18 @@ void gControl::setFocus()
 
 	if (win->isVisible())
 	{
-		//if (isVisible() && bufW > 0 && bufH > 0)
-		#if DEBUG_FOCUS
-		fprintf(stderr, "setFocus now %s\n", name());
-		#endif
-		//win->activate();
-		
 		gtk_widget_grab_focus(widget);
+		
+		#if DEBUG_FOCUS
+		fprintf(stderr, "setFocus NOW: %s %s %p -> %d\n", GB.GetClassName(hFree), name(), hFree, hasFocus());
+		#endif
 	}
 	else
 	{
 		#if DEBUG_FOCUS
-		fprintf(stderr, "setFocus later %s\n", name());
+		fprintf(stderr, "setFocus later: %s %s %p\n", GB.GetClassName(hFree), name(), hFree);
 		#endif
-		win->focus = this;
+		win->_initial_focus = this;
 	}
 }
 
@@ -1553,33 +1525,57 @@ Drag & Drop
 
 **********************************************************************/
 
+/*static void print_full_name(gControl *ctrl)
+{
+	if (ctrl->parent())
+	{
+		print_full_name(ctrl->parent());
+		fprintf(stderr, ">");
+	}
+
+	fprintf(stderr, "%s", ctrl->name() ? ctrl->name() : "?");
+}*/
+
+void gControl::updateAcceptDrops()
+{
+	GtkWidget *w = _scroll ? widget : border;
+
+	if (_accept_drops)
+	{
+		gtk_drag_dest_set(w, (GtkDestDefaults)0, NULL, 0, (GdkDragAction)(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
+		gtk_drag_dest_set_track_motion(w, TRUE);
+	}
+	else
+	{
+		gtk_drag_dest_unset(w);
+	}
+
+	/*fprintf(stderr, "updateAcceptDrops: ");
+	print_full_name(this);
+	fprintf(stderr, " -> %d\n", _accept_drops);*/
+}
+
+bool gControl::acceptDrops() const
+{
+	if (_proxy)
+		return _proxy->acceptDrops();
+	else
+		return _accept_drops;
+}
+
 void gControl::setAcceptDrops(bool vl)
 {
-	//GtkWidget *w;
-	//GtkTargetEntry entry[7];
-
-	/* BM: ??
-	if (!pr) w=frame;
-	else w=widget;
-	*/
-
+	if (_proxy)
+	{
+		_proxy->setAcceptDrops(vl);
+		return;
+	}
+	
 	if (vl == _accept_drops)
 		return;
 
 	_accept_drops = vl;
-
-	if (vl)
-	{
-		gtk_drag_dest_set(border, (GtkDestDefaults)0, NULL, 0, (GdkDragAction)(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
-		//if (widget != border)
-		//	gtk_drag_dest_set(widget, (GtkDestDefaults)0, NULL, 0, (GdkDragAction)(GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK));
-	}
-	else
-	{
-		gtk_drag_dest_unset(border);
-		//if (widget != border)
-		//	gtk_drag_dest_unset(widget);
-	}
+	updateAcceptDrops();
 }
 
 /*********************************************************************
@@ -1713,25 +1709,8 @@ void gControl::registerControl()
 	gt_register_control(border, this);
 }
 
-#ifdef GTK3
-/*static gboolean cb_clip_children(GtkWidget *wid, GdkEventExpose *e, gContainer *d)
-{
-	cairo_region_t *me;
-	GtkAllocation a;
+#ifndef GTK3
 
-	gtk_widget_get_allocation(wid, &a);
-	me = cairo_region_create_rectangle((cairo_rectangle_int_t *)&a);
-
-	cairo_region_intersect(e->region, me);
-
-	cairo_region_destroy(me);
-
-	if (cairo_region_is_empty(e->region))
-		return TRUE;
-
-	return FALSE;
-}*/
-#else
 static gboolean cb_clip_children(GtkWidget *wid, GdkEventExpose *e, gContainer *d)
 {
 	GdkRegion *me;
@@ -1749,6 +1728,7 @@ static gboolean cb_clip_children(GtkWidget *wid, GdkEventExpose *e, gContainer *
 
 	return FALSE;
 }
+
 #endif
 
 #if 0
@@ -1832,6 +1812,30 @@ static bool must_patch(GtkWidget *widget)
 	return (parent_control->widget == widget || (GtkWidget *)parent_control->_scroll == widget);
 }
 
+static gboolean draw_container(GtkWidget *widget, cairo_t *cr)
+{
+	GList *children;
+	GtkFixedChild *child;
+	GtkAllocation a;
+
+	gtk_widget_get_allocation(widget, &a);
+	
+	cairo_save(cr);
+	cairo_rectangle(cr, 0, 0, a.width, a.height);
+	cairo_clip(cr);	
+	
+  for (children = *get_children_list(GTK_CONTAINER(widget)); children; children = children->next)
+	{
+		child = (GtkFixedChild *)children->data;
+		cairo_save(cr);
+		gtk_container_propagate_draw (GTK_CONTAINER(widget), child->widget, cr);
+		cairo_restore(cr);
+	}
+  
+  cairo_restore(cr);
+  return FALSE;
+}
+
 #include "gb.gtk.patch.h"
 
 PATCH_DECLARE(GTK_TYPE_WINDOW)
@@ -1841,7 +1845,6 @@ PATCH_DECLARE(GTK_TYPE_SPIN_BUTTON)
 PATCH_DECLARE(GTK_TYPE_BUTTON)
 PATCH_DECLARE(GTK_TYPE_FIXED)
 PATCH_DECLARE(GTK_TYPE_EVENT_BOX)
-//PATCH_DECLARE(GTK_TYPE_ALIGNMENT)
 PATCH_DECLARE(GTK_TYPE_BOX)
 PATCH_DECLARE(GTK_TYPE_TOGGLE_BUTTON)
 PATCH_DECLARE(GTK_TYPE_SCROLLED_WINDOW)
@@ -1867,7 +1870,6 @@ void gt_patch_control(GtkWidget *widget)
 	else PATCH_CLASS_BASELINE(widget, GTK_TYPE_BUTTON)
 	else PATCH_CLASS(widget, GTK_TYPE_FIXED)
 	else PATCH_CLASS(widget, GTK_TYPE_EVENT_BOX)
-	//else PATCH_CLASS(widget, GTK_TYPE_ALIGNMENT)
 	else PATCH_CLASS(widget, GTK_TYPE_BOX)
 	else PATCH_CLASS(widget, GTK_TYPE_TOGGLE_BUTTON)
 	else PATCH_CLASS(widget, GTK_TYPE_SCROLLED_WINDOW)
@@ -1879,6 +1881,8 @@ void gt_patch_control(GtkWidget *widget)
 	else PATCH_CLASS(widget, GTK_TYPE_SCALE)
 	else PATCH_CLASS_BASELINE(widget, GTK_TYPE_COMBO_BOX)
 	else PATCH_CLASS(widget, GTK_TYPE_TEXT_VIEW)
+		
+	PATCH_CLASS_FIXED(widget, GTK_TYPE_FIXED)
 }
 
 #endif
@@ -1897,14 +1901,20 @@ void gControl::setMinimumSize()
 		bool mapped = gtk_widget_get_mapped(border);
 		
 		if (!mapped)
+		{
+			gApplication::_disable_mapping_events = true;
 			gtk_widget_show(border);
+		}
 			
 		_do_not_patch = true;
 		gtk_widget_get_preferred_size(widget, &minimum_size, &natural_size);
 		_do_not_patch = false;
 		
 		if (!mapped)
+		{
 			gtk_widget_hide(border);
+			gApplication::_disable_mapping_events = false;
+		}
 		
 		//fprintf(stderr, "gtk_widget_get_preferred_size: %s: min = %d %d / nat = %d %d\n", GB.GetClassName(hFree), minimum_size.width, minimum_size.height, natural_size.width, natural_size.height);
 
@@ -1920,8 +1930,18 @@ void gControl::setMinimumSize()
 }
 
 
-void gControl::connectBorder()
+void gControl::updateEventMask()
 {
+	gtk_widget_add_events(widget, GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK
+		| GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK | GDK_SCROLL_MASK
+		| GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK);
+
+	if (widget != border && (GTK_IS_WINDOW(border) || (GTK_IS_EVENT_BOX(border) && !gtk_event_box_get_visible_window(GTK_EVENT_BOX(border)))))
+	{
+		gtk_widget_add_events(border, GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK
+			| GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK | GDK_SCROLL_MASK
+			| GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK);
+	}
 }
 
 void gControl::realize(bool draw_frame)
@@ -1948,13 +1968,12 @@ void gControl::realize(bool draw_frame)
 		gt_patch_control(widget);
 #endif
 
+	initSignals();
 	connectParent();
-	connectBorder();
 	
 	setMinimumSize();
 	resize(Max(8, _min_w), Max(8, _min_h), true);
-	initSignals();
-
+	
 	if (!_no_background && !gtk_widget_get_has_window(border))
 		ON_DRAW_BEFORE(border, this, cb_background_expose, cb_background_draw);
 
@@ -1966,20 +1985,7 @@ void gControl::realize(bool draw_frame)
 		g_signal_connect(G_OBJECT(widget), "expose-event", G_CALLBACK(cb_clip_children), (gpointer)this);
 #endif
 
-	//if (isContainer() && widget != border)
-	//	g_signal_connect(G_OBJECT(widget), "size-allocate", G_CALLBACK(cb_size_allocate), (gpointer)this);
-
-	gtk_widget_add_events(widget, GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK
-		| GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK | GDK_SCROLL_MASK
-		| GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK);
-
-	if (widget != border && (GTK_IS_WINDOW(border) || (GTK_IS_EVENT_BOX(border) && !gtk_event_box_get_visible_window(GTK_EVENT_BOX(border)))))
-	{
-		gtk_widget_add_events(border, GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK
-			| GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK | GDK_SCROLL_MASK
-			| GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK);
-	}
-
+	updateEventMask();
 	registerControl();
 	updateFont();
 }
@@ -2219,10 +2225,10 @@ void gControl::updateStyleSheet(bool dirty)
 		gt_css_add_color(css, bg, fg);
 	}
 	
-	if (_font)
+	if (_resolved_font)
 	{
 		setStyleSheetNode(css, getStyleSheetFontNode());
-		gt_css_add_font(css, _font);
+		gt_css_add_font(css, _resolved_font);
 	}
 
 	customStyleSheet(css);
@@ -2245,8 +2251,14 @@ gColor gControl::realBackground(bool no_default)
 {
 	if (_bg != COLOR_DEFAULT)
 		return _bg;
-	else
-		return no_default ? defaultBackground() : COLOR_DEFAULT;
+	
+	if (!no_default)
+		return COLOR_DEFAULT;
+	
+	if (!use_base && parent())
+		return parent()->realBackground(true);
+	
+	return defaultBackground();
 }
 
 void gControl::setRealBackground(gColor color)
@@ -2641,6 +2653,14 @@ bool gControl::setProxy(gControl *proxy)
 
 		check = check->_proxy;
 	}
+	
+	if (proxy == _proxy)
+		return false;
+
+	//fprintf(stderr, "proxy: (%p %s) -> (%p %s)\n", this, name(), proxy, proxy ? proxy->name() : "NULL");
+	
+	if (proxy && proxy->_proxy_for)
+		proxy->_proxy_for->_proxy = NULL;
 
 	if (_proxy)
 		_proxy->_proxy_for = NULL;
@@ -2661,9 +2681,6 @@ void gControl::setNoTabFocus(bool v)
 		return;
 	}
 	
-	if (_no_tab_focus == v)
-		return;
-
 	_no_tab_focus = v;
 }
 
@@ -2675,6 +2692,11 @@ bool gControl::isNoTabFocus() const
 		return _no_tab_focus;
 }
 
+bool gControl::isNoTabFocusRec() const
+{
+	return isNoTabFocus() || (parent() && parent()->isNoTabFocusRec());
+}
+
 #ifdef GTK3
 void gControl::onEnterEvent()
 {
@@ -2740,7 +2762,7 @@ void gControl::emitEnterEvent(bool no_leave)
 	}
 
 	//fprintf(stderr, "RAISE ENTER: %s\n", name());
-	emit(SIGNAL(onEnterLeave), gEvent_Enter);
+	CB_control_enter_leave(this, gEvent_Enter);
 }
 
 void gControl::emitLeaveEvent()
@@ -2785,7 +2807,7 @@ void gControl::emitLeaveEvent()
 	}
 
 	//fprintf(stderr, "RAISE LEAVE: %s\n", name());
-	emit(SIGNAL(onEnterLeave), gEvent_Leave);
+	CB_control_enter_leave(this, gEvent_Leave);
 }
 
 bool gControl::isAncestorOf(gControl *child)
@@ -2806,10 +2828,16 @@ bool gControl::isAncestorOf(gControl *child)
 #ifdef GTK3
 void gControl::drawBackground(cairo_t *cr)
 {
-	if (background() == COLOR_DEFAULT)
-		return;
+	gColor col= background();
+	
+	if (col == COLOR_DEFAULT)
+	{
+		if (!gtk_widget_get_has_window(border))
+			return;
+		col = realBackground(true);
+	}
 
-	gt_cairo_set_source_color(cr, background());
+	gt_cairo_set_source_color(cr, col);
 	cairo_rectangle(cr, 0, 0, width(), height());
 	cairo_fill(cr);
 }
@@ -2884,50 +2912,41 @@ gControl *gControl::nextFocus()
 	gControl *ctrl;
 	gControl *next_ctrl;
 	
-	//fprintf(stderr, "next: %s\n", name());
-	
-	if (isContainer())
-	{
-		ctrl = ((gContainer *)this)->firstChild();
-		if (ctrl)
-		{
-			//fprintf(stderr, "==> %s\n", ctrl->name());
-			return ctrl;
-		}
-	}
-	
 	ctrl = this;
 	
+	if (ctrl->isContainer() && ((gContainer *)ctrl)->childCount())
+		return ((gContainer *)ctrl)->firstChild();
+	
 	for(;;)
 	{
 		next_ctrl = ctrl->next();
 		if (next_ctrl)
 			return next_ctrl;
-		
+		if (ctrl->isTopLevel())
+			return ctrl;
 		ctrl = ctrl->parent();
-		if (!ctrl)
-			return NULL;
-		/*if (!ctrl->parent())
-			return ctrl->nextFocus();*/
 	}
 }
 
 gControl *gControl::previousFocus()
 {
-	gControl *ctrl = previous();
+	gControl *ctrl;
+	gControl *next_ctrl;
 	
-	if (!ctrl)
+	ctrl = this;
+	
+	if (ctrl->isContainer() && ((gContainer *)ctrl)->childCount())
+		return ((gContainer *)ctrl)->lastChild();
+	
+	for(;;)
 	{
-		if (!isTopLevel())
-			return parent()->previousFocus();
-		
-		ctrl = this;
+		next_ctrl = ctrl->previous();
+		if (next_ctrl)
+			return next_ctrl;
+		if (ctrl->isTopLevel())
+			return ctrl;
+		ctrl = ctrl->parent();
 	}
-	
-	while (ctrl->isContainer() && ((gContainer *)ctrl)->childCount())
-		ctrl = ((gContainer *)ctrl)->lastChild();
-
-	return ctrl;
 }
 
 void gControl::createWidget()
@@ -2946,7 +2965,6 @@ void gControl::createBorder(GtkWidget *new_border, bool keep_widget)
 	GtkWidget *old = border;
 	
 	border = new_border;
-	connectBorder();
 	
 	if (keep_widget && widget)
 		gt_widget_reparent(widget, border);
@@ -2958,16 +2976,63 @@ void gControl::createBorder(GtkWidget *new_border, bool keep_widget)
 		_no_delete = false;
 		createWidget();
 	}
+
+	updateAcceptDrops();
 }
 
-bool gControl::setInverted(bool v)
+int gControl::actualDirection() const
+{
+	const gControl *ctrl = this;
+	
+	for(;;)
+	{
+		if (ctrl->direction() != DIRECTION_DEFAULT)
+			return ctrl->direction();
+		if (ctrl->isWindow())
+			return DIRECTION_DEFAULT;
+		ctrl = ctrl->parent();
+	}
+}
+
+void gControl::updateDirection()
+{
+	GtkTextDirection dir;
+	
+	switch(actualDirection())
+	{
+		case DIRECTION_LTR: dir = GTK_TEXT_DIR_LTR; break;
+		case DIRECTION_RTL: dir = GTK_TEXT_DIR_RTL; break;
+		default: dir = MAIN_rtl ? GTK_TEXT_DIR_RTL : GTK_TEXT_DIR_LTR;
+	}
+	
+	if (_inverted)
+		dir = dir == GTK_TEXT_DIR_LTR ? GTK_TEXT_DIR_RTL : GTK_TEXT_DIR_LTR;
+	
+	gtk_widget_set_direction(widget, dir);
+}
+
+bool gControl::isRightToLeft() const
+{
+	return gtk_widget_get_direction(widget) == GTK_TEXT_DIR_RTL;
+}
+
+
+void gControl::setInverted(bool v)
 {
 	if (v == _inverted)
-		return true;
+		return;
 	
 	_inverted = v;
-	gt_widget_set_inverted(widget, v);
-	return false;
+	updateDirection();
+}
+
+void gControl::setDirection(int v)
+{
+	if (v == _direction)
+		return;
+	
+	_direction = v;
+	updateDirection();
 }
 
 void gControl::checkVisibility()
