@@ -54,7 +54,7 @@ typedef
 static int _current_clipboard = gClipboard::Clipboard;
 static GtkClipboard *_clipboard = NULL;
 static GtkClipboard *_selection = NULL;
-static bool _clipboard_has_changed[2] = { FALSE };
+static bool _clipboard_has_changed[2] = { TRUE };
 
 static char *convert_format(char *fmt)
 {
@@ -787,8 +787,10 @@ bool gDrag::setCurrent(gControl *control)
 {
 	gControl *current;
 	
+	//fprintf(stderr, "gDrag::setCurrent: %s -> %s\n", _current ? _current->name() : "NULL", control ? control->name() : "NULL");
+
 	if (_current == control)
-		return true;
+		return false;
 	
 	if (_current)
 	{
@@ -796,14 +798,18 @@ bool gDrag::setCurrent(gControl *control)
 		while (current)
 		{
 			CB_control_drag_leave(current);
-			current = current->_proxy;
+			current = current->_proxy_for;
 		}
 	}
 	
 	_current = control;
 	
-	if (control)
-		return !CB_control_drag(control);
-	else
-		return false;
+	while (control)
+	{
+		if (CB_control_drag(control))
+			return true;
+		control = control->_proxy_for;
+	}
+	
+	return false;
 }
