@@ -43,10 +43,6 @@ int gDrawingArea::_in_any_draw_event = 0;
 #ifdef GTK3
 static gboolean cb_draw(GtkWidget *wid, cairo_t *cr, gDrawingArea *data)
 {
-	/*cairo_rectangle_list_t *list;
-	cairo_rectangle_t *r;
-	int i;*/
-
 	if (data->cached())
 	{
 		cairo_set_source_surface(cr, data->buffer, 0, 0);
@@ -55,39 +51,13 @@ static gboolean cb_draw(GtkWidget *wid, cairo_t *cr, gDrawingArea *data)
 	}
 	else
 	{
-		//data->drawBackground();
+		data->drawBackground(cr);
 
-		if (data->onExpose)
-		{
-			gDrawingArea::_in_any_draw_event++;
-			data->_in_draw_event = true;
-			data->onExpose(data, cr);
-			gDrawingArea::_in_any_draw_event--;
-			data->_in_draw_event = false;
-			/*
-			list = cairo_copy_clip_rectangle_list(cr);
-
-			fprintf(stderr, "%d %d\n", list->status, list->num_rectangles);
-			if (list->status != CAIRO_STATUS_SUCCESS)
-			{
-				data->onExpose(data, cr);
-			}
-			else
-			{
-				for (i = 0; i < list->num_rectangles; i++)
-				{
-					r = &list->rectangles[i];
-					cairo_save(cr);
-					cairo_rectangle(cr, r->x, r->y, r->width, r->height);
-					cairo_clip(cr);
-					data->onExpose(data, cr);
-					cairo_restore(cr);
-				}
-			}
-
-			cairo_rectangle_list_destroy(list);
-			*/
-		}
+		gDrawingArea::_in_any_draw_event++;
+		data->_in_draw_event = true;
+		CB_drawingarea_expose(data, cr);
+		gDrawingArea::_in_any_draw_event--;
+		data->_in_draw_event = false;
 		data->drawBorder(cr);
 	}
 
@@ -104,19 +74,16 @@ static gboolean cb_expose(GtkWidget *wid, GdkEventExpose *e, gDrawingArea *data)
 	{
 		//data->drawBackground();
 
-		if (data->onExpose)
-		{
-			gDrawingArea::_in_any_draw_event++;
-			data->_in_draw_event = true;
+		gDrawingArea::_in_any_draw_event++;
+		data->_in_draw_event = true;
 
-			GtkAllocation a;
-			gtk_widget_get_allocation(wid, &a);
-			//fprintf(stderr, "%s: %d %d\n", data->parent()->name(), a.x, a.y);
-			
-			data->onExpose(data, e->region, a.x, a.y);
-			gDrawingArea::_in_any_draw_event--;
-			data->_in_draw_event = false;
-		}
+		GtkAllocation a;
+		gtk_widget_get_allocation(wid, &a);
+		//fprintf(stderr, "%s: %d %d\n", data->parent()->name(), a.x, a.y);
+		
+		CB_drawingarea_expose(data, e->region, a.x, a.y);
+		gDrawingArea::_in_any_draw_event--;
+		data->_in_draw_event = false;
 		data->drawBorder(e);
 	}
 
@@ -135,6 +102,7 @@ void gDrawingArea::create(void)
 	GtkWidget *ch;
 	bool doReparent = false;
 	bool was_visible = isVisible();
+	bool can_focus = widget ? canFocus() : false;
 	GdkRectangle rect;
 	int bg, fg;
 
@@ -156,16 +124,18 @@ void gDrawingArea::create(void)
 	}
 
 #ifdef GTK3
-	if (_cached || _use_tablet)
+	if (_cached || _use_tablet || _own_window)
 #else
-	if (_cached || _use_tablet || background() != COLOR_DEFAULT)
+	if (_cached || _use_tablet || _own_window || background() != COLOR_DEFAULT)
 #endif
 	{
 		createBorder(gtk_event_box_new());
 		widget = gtk_fixed_new();
 		box = widget;
+#ifndef GTK3
 		gtk_widget_set_app_paintable(border, TRUE);
 		gtk_widget_set_app_paintable(box, TRUE);
+#endif
 	}
 	else
 	{
@@ -183,6 +153,8 @@ void gDrawingArea::create(void)
 
 	updateUseTablet();
 
+	gtk_widget_set_can_focus(widget, can_focus);
+	
 	if (doReparent)
 	{
 		if (box)
@@ -212,10 +184,10 @@ void gDrawingArea::create(void)
 gDrawingArea::gDrawingArea(gContainer *parent) : gContainer(parent)
 {
 	_is_drawingarea = true;
+	_own_window = false;
 	_cached = false;
 	buffer = NULL;
 	box = NULL;
-	_old_bg_id = 0;
 	_resize_cache = false;
 	_no_background = false;
 	_use_tablet = false;
@@ -223,9 +195,6 @@ gDrawingArea::gDrawingArea(gContainer *parent) : gContainer(parent)
 	_no_style_without_child = true;
 #endif
 
-	onExpose = NULL;
-	onFontChange = NULL;
-
 	create();
 }
 
@@ -235,40 +204,6 @@ gDrawingArea::~gDrawingArea()
 		UNREF_BUFFER();
 }
 
-/*void gDrawingArea::resize(int w, int h)
-{
-	// TODO Do not resize cache if the DrawingArea is being painted
-	gContainer::resize(w,h);
-	//updateCache();
-}*/
-
-void gDrawingArea::updateEventMask()
-{
-	/*
-	static int event_mask;
-	XWindowAttributes attr;
-
-	gtk_widget_realize(border);
-
-	if (!enabled())
-	{
-		XGetWindowAttributes(gdk_display, GDK_WINDOW_XID(border->window), &attr);
-		event_mask = attr.your_event_mask;
-		XSelectInput(gdk_display, GDK_WINDOW_XID(border->window), ExposureMask);
-	}
-	else
-	{
-		XSelectInput(gdk_display, GDK_WINDOW_XID(border->window), event_mask);
-	}
-	*/
-}
-
-void gDrawingArea::setEnabled(bool vl)
-{
-	gContainer::setEnabled(vl);
-	updateEventMask();
-}
-
 void gDrawingArea::setCached(bool vl)
 {
 	if (vl == _cached) return;
@@ -414,17 +349,6 @@ void gDrawingArea::refreshCache()
 
 void gDrawingArea::setNoBackground(bool vl)
 {
-	/*
-	GdkWindow *win;
-
-	gtk_widget_realize(widget);
-	win = widget->window;
-
-	if (vl)
-		gdk_window_set_back_pixmap(win, NULL, FALSE);
-	else
-		setBackground(background());
-	*/
 	if (vl != _no_background)
 	{
 		_no_background = vl;
@@ -459,7 +383,7 @@ void gDrawingArea::setUseTablet(bool vl)
 void gDrawingArea::updateFont()
 {
 	gContainer::updateFont();
-	emit(SIGNAL(onFontChange));
+	CB_drawingarea_font(this);
 }
 
 #ifdef GTK3
@@ -474,3 +398,14 @@ void gDrawingArea::setBackground(gColor color)
 		create();
 }
 #endif
+
+long gDrawingArea::handle()
+{
+	if (!_own_window)
+	{
+		_own_window = true;
+		create();
+	}
+	
+	return gControl::handle();
+}
