@@ -120,22 +120,17 @@ static void cb_activate(GtkMenuItem *menuitem, gMenu *data)
 		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem), true);
 	}
 	
-	if (data->onClick)
-	{
-		//fprintf(stderr, "cb_activate: %s\n", data->name());
-		data->onClick(data);
-	}
+	//fprintf(stderr, "cb_activate: %s\n", data->name());
+	CB_menu_click(data);
 }
 
-static void cb_size_allocate(GtkWidget *menu, GdkRectangle *allocation, gMenu *data)
+static void cb_show(GtkWidget *menu, gMenu *data)
 {
-	//fprintf(stderr, "cb_size_allocate: %s %d x %d (%d)\n", data->name(), allocation->width, allocation->height, gtk_widget_get_mapped(menu));
-	
 	if (!data->_opened)
 	{
 		data->_opened = true;
-		if (data->onShow) (*data->onShow)(data);
-		//data->hideSeparators();
+		CB_menu_show(data);
+		//gtk_menu_reposition(GTK_MENU(menu));
 	}
 }
 
@@ -148,11 +143,11 @@ static gboolean cb_map(GtkWidget *menu, gMenu *data)
 
 	data->_mapping = true;
 	
+	gtk_widget_hide(gtk_widget_get_parent(menu));
 	data->hideSeparators();
-	gtk_widget_hide(menu);
-	gtk_widget_show(menu);
-	//gtk_menu_reposition(GTK_MENU(menu));
-	
+	gtk_widget_show(gtk_widget_get_parent(menu));
+	gtk_menu_reposition(GTK_MENU(menu));
+
 	data->_mapping = false;
 
 	//fprintf(stderr, "cb_map: <<<\n");
@@ -167,7 +162,8 @@ static gboolean cb_unmap(GtkWidget *menu, gMenu *data)
 		return false;
 
 	data->_opened = false;
-	if (data->onHide) (*data->onHide)(data);
+	CB_menu_hide(data);
+	//gtk_widget_set_size_request(menu, -1, -1);
 
 	//fprintf(stderr, "cb_unmap: <<<\n");
 	return false;
@@ -364,7 +360,8 @@ void gMenu::update()
 					
 					//fprintf(stderr, "creates a new child menu container in parent %s\n", parent->name());
 					
-					g_signal_connect(G_OBJECT(parent->_popup), "size-allocate", G_CALLBACK(cb_size_allocate), (gpointer)parent);
+					//g_signal_connect(G_OBJECT(parent->_popup), "size-allocate", G_CALLBACK(cb_size_allocate), (gpointer)parent);
+					g_signal_connect(G_OBJECT(parent->_popup), "show", G_CALLBACK(cb_show), (gpointer)parent);
 					g_signal_connect(G_OBJECT(parent->_popup), "map", G_CALLBACK(cb_map), (gpointer)parent);
 					g_signal_connect(G_OBJECT(parent->_popup), "unmap", G_CALLBACK(cb_unmap), (gpointer)parent);
 					gtk_widget_show_all(GTK_WIDGET(parent->_popup));
@@ -462,11 +459,6 @@ void gMenu::initialize()
 {
 	//fprintf(stderr, "gMenu::gMenu: %p (%p)\n", this, pr);
 	
-	onFinish = NULL;
-	onClick = NULL;
-	onShow = NULL;
-	onHide = NULL;
-	
 	hFree = NULL;
 	_popup = NULL;
 	image = NULL;
@@ -649,7 +641,7 @@ gMenu::~gMenu()
 	#if DEBUG_DELETE
 	fprintf(stderr, "~gMenu: >>> %s\n", name());
 	#endif
-	if (onFinish) onFinish(this);
+	CB_menu_finish(this);
 }
 
 void gMenu::setEnabled(bool vl)
@@ -829,6 +821,7 @@ void gMenu::destroy()
 	if (_destroyed)
 		return;
 	
+	hide();
 	dispose();
 	menus_destroyed = g_list_prepend(menus_destroyed, (gpointer)this);
 	_destroyed = true;
@@ -850,7 +843,10 @@ void gMenu::doPopup(bool move, int x, int y)
 		return;
 	
 	gMenu *save_current_popup = _current_popup;
+	GtkWidget *save_grab = gApplication::_popup_grab;
+
 	_current_popup = this;
+	gApplication::_popup_grab = GTK_WIDGET(_popup);
 	
 	_in_popup++;
 	_popup_count++;
@@ -861,13 +857,25 @@ void gMenu::doPopup(bool move, int x, int y)
 	GdkWindow *win;
 	GdkRectangle rect;
 	GdkEvent *event;
+	GdkEvent *last_event;
 	
 	gt_disable_warnings(true);
 	
 	event = gdk_event_new(GDK_BUTTON_PRESS);
 	event->button.time = gApplication::lastEventTime(); //GDK_CURRENT_TIME;
-	event->button.button = 1;
-	event->button.window = gtk_widget_get_window(window()->border);
+	
+	last_event = gApplication::lastEvent();
+	if (last_event && last_event->type == GDK_BUTTON_PRESS)
+	{
+		event->button.button = last_event->button.button;
+		event->button.window = last_event->button.window;
+	}
+	else
+	{
+		event->button.button = 1;
+		event->button.window = gtk_widget_get_window(window()->border);
+	}
+	
 	gdk_event_set_device(event, gMouse::getPointer());
 	
 	if (move)
@@ -912,7 +920,9 @@ void gMenu::doPopup(bool move, int x, int y)
 		MAIN_do_iteration(false);
 
 	_exec = false;
+
 	_current_popup = save_current_popup;
+	gApplication::_popup_grab = save_grab;
 
 	_in_popup--;
 
@@ -924,7 +934,7 @@ void gMenu::doPopup(bool move, int x, int y)
 	
 	// flush the event loop so that the main window is reactivated before the click menu event is raised
 
-	while (gtk_events_pending())
+	while (gApplication::eventsPending())
 		MAIN_do_iteration(false);
 }
 
