@@ -2,7 +2,7 @@
 
   kentities.h
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -96,7 +96,7 @@ inline
 #endif
 #endif
 static unsigned int
-hash_Entity (register const char *str, register unsigned int len)
+hash_Entity (const char *str, unsigned int len)
 {
   static const unsigned short asso_values[] =
     {
@@ -127,7 +127,7 @@ hash_Entity (register const char *str, register unsigned int len)
       932, 932, 932, 932, 932, 932, 932, 932, 932, 932,
       932, 932, 932, 932, 932, 932, 932
     };
-  register int hval = len;
+  int hval = len;
 
   switch (hval)
     {
@@ -163,7 +163,7 @@ hash_Entity (register const char *str, register unsigned int len)
 __inline
 #endif
 const struct entity *
-kde_findEntity (register const char *str, register unsigned int len)
+kde_findEntity (const char *str, unsigned int len)
 {
   enum
     {
@@ -838,15 +838,15 @@ kde_findEntity (register const char *str, register unsigned int len)
 
   if (len <= MAX_WORD_LENGTH && len >= MIN_WORD_LENGTH)
     {
-      register int key = hash_Entity (str, len);
+      int key = hash_Entity (str, len);
 
       if (key <= MAX_HASH_VALUE && key >= 0)
         {
-          register int index = lookup[key];
+          int index = lookup[key];
 
           if (index >= 0)
             {
-              register const char *s = wordlist_Entity[index].name;
+              const char *s = wordlist_Entity[index].name;
 
               if (*str == *s && !strncmp (str + 1, s + 1, len - 1) && s[len] == '\0')
                 return &wordlist_Entity[index];
