@@ -3,7 +3,7 @@
   main.cpp
 
   (c) Daniel Campos Fernández <dcamposf@gmail.com>
-  (c) Benoît Minisini <g4mba5@gmail.com>
+  (c) Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -232,8 +232,6 @@ GB_DESC *GB_CLASSES[] EXPORT =
 	CTextBoxDesc,
 	CTextAreaDesc,
 	CTextAreaSelectionDesc,
-	CComboBoxDesc,
-	CComboBoxItemDesc,
 	CTabStripDesc,
 	CTabStripContainerDesc,
 	CTabStripContainerChildrenDesc,
@@ -355,10 +353,8 @@ void EXPORT GB_SIGNAL(int signal, void *param)
 			break;
 
 		case GB_SIGNAL_DEBUG_FORWARD:
-			//while (qApp->activePopupWidget())
-			//	delete qApp->activePopupWidget();
 			if (gdk_display_get_default())
-			gdk_display_sync(gdk_display_get_default());
+				gdk_display_sync(gdk_display_get_default());
 			break;
 
 		case GB_SIGNAL_DEBUG_CONTINUE:
@@ -415,7 +411,8 @@ void hook_quit(void)
 {
 	GB_FUNCTION func;
 
-	while (gtk_events_pending())
+	while (!gApplication::processInputEvent());
+	while (gApplication::eventsPending())
 		gtk_main_iteration();
 
 	if (GB.ExistClass("TrayIcons"))
@@ -472,6 +469,7 @@ static void hook_main(int *argc, char ***argv)
 	gApplication::init(argc, argv);
 
 	gApplication::setDefaultTitle(GB.Application.Title());
+	CAPPLICATION_MiddleClickPaste = gApplication::hasMiddleClickPaste();
 
 	gApplication::onEnterEventLoop = GB.Debug.EnterEventLoop;
 	gApplication::onLeaveEventLoop = GB.Debug.LeaveEventLoop;
@@ -592,7 +590,7 @@ static void hook_wait(int duration)
 		return;
 	}
 
-	if (duration && gKey::isValid())
+	if (duration != -1 && gKey::isValid())
 	{
 		if (!_warning)
 		{
@@ -602,13 +600,20 @@ static void hook_wait(int duration)
 		return;
 	}
 
-	if (duration == 0)
+	if (duration >= 0)
 	{
-		while (gtk_events_pending())
-			MAIN_do_iteration(false);
+		MAIN_do_iteration(true);
 	}
-	else
-		MAIN_do_iteration(duration > 0);
+	else if (duration == -1)
+	{
+		bool d = gApplication::disableInputEvents(true);
+		MAIN_do_iteration(true); // To process deferred routines
+		while (gApplication::eventsPending())
+			MAIN_do_iteration(false);
+		gApplication::disableInputEvents(d);
+	}
+	else if (duration == -2)
+		MAIN_do_iteration(false);
 }
 
 static void hook_watch(int fd, int type, void *callback, intptr_t param)
@@ -669,24 +674,33 @@ static void hook_lang(char *lang, int rtl)
 
 void MAIN_do_iteration_just_events()
 {
-	if (gtk_events_pending())
+	if (gApplication::eventsPending())
+	{
+		while (!gApplication::processInputEvent());
 		gtk_main_iteration_do(false);
+	}
 }
 
 void MAIN_do_iteration(bool do_not_block)
 {
 	gApplication::_loopLevel++;
 
+	while (!gApplication::processInputEvent());
+	
 	if (do_not_block)
 	{
-		if (gtk_events_pending())
+		if (gApplication::eventsPending())
 			gtk_main_iteration();
 	}
 	else
+	{
 		gtk_main_iteration_do(true);
+	}
 
 	gApplication::_loopLevel--;
 
+	gContainer::postArrange();
+
 	if (_post_check)
 	{
 		_post_check = false;
@@ -694,6 +708,5 @@ void MAIN_do_iteration(bool do_not_block)
 	}
 
 	gControl::postDelete();
-	gContainer::postArrange();
 }
 
