@@ -4,7 +4,7 @@
 
   gb.media component
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -326,38 +326,63 @@ static void return_value(const GValue *value)
 			}
 			else if(GST_VALUE_HOLDS_LIST(value))
 			{
-				guint listSize = gst_value_list_get_size(value);
+				guint size = gst_value_list_get_size(value);
 				GB_ARRAY array;
-				
-				if(listSize <= 0)
+
+				if(size <= 0)
 				{
 					GB.Array.New(&array, GB_T_VARIANT, 0);
 				}
 				else
 				{
-					GB.Array.New(&array, GB_T_VARIANT, listSize);
-					
+					GB.Array.New(&array, GB_T_VARIANT, size);
+
 					GB_VALUE val;
 					int i;
-					for (i = 0; i < listSize; i++)
+					for (i = 0; i < size; i++)
 					{
 						to_gambas_value(gst_value_list_get_value(value, i), &val);
 						GB.Store(GB_T_VARIANT, &val, GB.Array.Get(array, i));
 						GB.ReleaseValue(&val);
 					}
 				}
-				
+
 				GB.ReturnObject(array);
 			}
-#ifdef G_TYPE_VALUE_ARRAY
+			else if(GST_VALUE_HOLDS_ARRAY(value))
+			{
+				guint size = gst_value_array_get_size(value);
+				GB_ARRAY array;
+
+				if(size <= 0)
+				{
+					GB.Array.New(&array, GB_T_VARIANT, 0);
+				}
+				else
+				{
+					GB.Array.New(&array, GB_T_VARIANT, size);
+
+					GB_VALUE val;
+					int i;
+					for (i = 0; i < size; i++)
+					{
+						to_gambas_value(gst_value_array_get_value(value, i), &val);
+						GB.Store(GB_T_VARIANT, &val, GB.Array.Get(array, i));
+						GB.ReleaseValue(&val);
+					}
+				}
+
+				GB.ReturnObject(array);
+			}
+#if defined(G_TYPE_VALUE_ARRAY) && !GLIB_CHECK_VERSION(2, 32, 0)
 			else if (G_VALUE_HOLDS(value, G_TYPE_VALUE_ARRAY))
 			{
 				GValueArray *garray = (GValueArray *)g_value_get_boxed(value);
 				guint len = garray->n_values;
 				GB_ARRAY array;
-				
+
 				GB.Array.New(&array, GB_T_VARIANT, len);
-				
+
 				if (len <= 0)
 				{
 					GB.Array.New(&array, GB_T_VARIANT, 0);
@@ -373,7 +398,6 @@ static void return_value(const GValue *value)
 						GB.ReleaseValue(&val);
 					}
 				}
-				
 				GB.ReturnObject(array);
 			}
 #endif
@@ -1861,12 +1885,10 @@ END_METHOD
 
 BEGIN_METHOD_VOID(MediaPipeline_free)
 
+	//fprintf(stderr, "MediaPipeline_free\n");
 	MEDIA_stop_pipeline(THIS);
 	if (THIS_PIPELINE->watch)
-	{
-		//GB.Wait(THIS_PIPELINE->polling);
 		GB.Unref(POINTER(&THIS_PIPELINE->watch));
-	}
 
 END_METHOD
 
@@ -2037,14 +2059,12 @@ GB_DESC MediaMessageDesc[] =
 	GB_PROPERTY_READ("Type", "i", MediaMessage_Type),
 	GB_PROPERTY_READ("Name", "s", MediaMessage_Name),
 	
-	
 	GB_METHOD("_get", "v", MediaMessage_get, "(Name)s"),
 	GB_METHOD("_next", "v", MediaMessage_next, NULL),
 	GB_PROPERTY_READ("Key", "s", MediaMessage_Key),
 	GB_PROPERTY_READ("Keys", "String[]", MediaMessage_Keys),
 	GB_PROPERTY_READ("Count", "i", MediaMessage_Count),
 	
-	
 	//Constants
 	GB_CONSTANT("Eos", "i", GST_MESSAGE_EOS),
 	GB_CONSTANT("Error", "i", GST_MESSAGE_ERROR),
