@@ -2,14 +2,11 @@
 
   main.h
 
-  The Gambas openGL component
-
   (c) 2005-2007 Laurent Carlier <lordheavy@users.sourceforge.net>
-                Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -19,21 +16,32 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
 #ifndef __MAIN_H
 #define __MAIN_H
 
+#include "gb_common.h"
 #include "gambas.h"
+#include "gb.image.h"
 
 #ifndef __MAIN_C
 extern GB_INTERFACE GB;
+extern IMAGE_INTERFACE IMAGE;
 #endif
 
 #ifndef WARNING
-#define WARNING(c) printf("WARNING: " c)
+#define WARNING(_c) printf("warning: " _c)
 #endif /* WARNING */
 
+int IMAGE_get_pixel_format(GB_IMG *image);
+bool IMAGE_get(GB_OBJECT *arg, GB_IMG **img, int *format);
+
+#define IMAGE_get_ncolors(_format) (GB_IMAGE_FMT_IS_24_BITS(_format) ? 3 : 4)
+
+
 #endif
+
