@@ -3,7 +3,7 @@
   main.c
 
   (c) 2004 Rob Kudla <pcre-component@kudla.org>
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -29,7 +29,13 @@
 #include <stdarg.h>
 #include <string.h>
 
+#include "config.h"
+
+#if PCRE2
+#include "regexp2.h"
+#else
 #include "regexp.h"
+#endif
 
 #include "main.h"
 
