@@ -2,7 +2,7 @@
 
   CStyle.cpp
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -108,7 +108,7 @@ static void init_option(QStyleOption &opt, int x, int y, int w, int h, int state
 	if (!(state & GB_DRAW_STATE_DISABLED))
 		opt.state |= QStyle::State_Enabled;
 	if (state & GB_DRAW_STATE_FOCUS)
-		opt.state |= QStyle::State_HasFocus;
+		opt.state |= QStyle::State_HasFocus | QStyle::State_KeyboardFocusChange;
 	if (state & GB_DRAW_STATE_HOVER)
 		opt.state |= QStyle::State_MouseOver;
 	if (state & GB_DRAW_STATE_ACTIVE)
