@@ -2,7 +2,7 @@
 
   cpaint_impl.cpp
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -283,11 +283,7 @@ static int Begin(GB_PAINT *d)
 		if (init_painting(d, target))
 			return TRUE;
 		// Paint.W / $hPrinter.PaperWidth * 25.4  / $hPrinter.Resolution
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
-		QSizeF size = printer->printer->pageLayout().pageSize().size(QPageSize::Millimeter);
-#else		
-		QSizeF size = printer->printer->paperSize(QPrinter::Millimeter);
-#endif
+		QSizeF size = CPRINTER_get_page_size(printer);
 		//qDebug("d->area.width = %g / paper width = %g / resolution = %d", d->area.width, (floor((double)size.width() * 1E6) / 1E6), printer->printer->resolution());
 		d->fontScale = 25.4 * d->area.width / (floor((double)size.width() * 1E6) / 1E6) / printer->printer->resolution();
 		
@@ -402,10 +398,20 @@ static void Antialias(GB_PAINT *d, int set, int *antialias)
 
 static void set_painter_font(QPainter *p, const QFont &f)
 {
-	p->setFont(f);	
+	QFont pf;
+
+	pf.setFamily(f.family());
+	pf.setPointSizeF(f.pointSizeF());
+	pf.setWeight(f.weight());
+	pf.setStyle(f.style());
+	pf.setUnderline(f.underline());
+	pf.setStrikeOut(f.strikeOut());
+
+	p->setFont(pf);
+
 	// Strange bug of QT. Sometimes the font does not apply (cf. DrawTextShadow)
 		
-	/*if (f != p->font())
+	/*if (pf != p->font())
 	{
 		fprintf(stderr, "set_painter_font: %s / %s\n", TO_UTF8(f.toString()), TO_UTF8(p->font().toString()));
 		QFont f2;
