@@ -2,7 +2,7 @@
 
   main.cpp
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -143,7 +143,6 @@ GB_CLASS CLASS_Printer;
 GB_CLASS CLASS_Image;
 GB_CLASS CLASS_SvgImage;
 GB_CLASS CLASS_TextArea;
-GB_CLASS CLASS_ComboBox;
 
 static bool in_event_loop = false;
 static int _no_destroy = 0;
@@ -1045,25 +1044,24 @@ static void hook_wait(int duration)
 		return;
 	}
 
+	if (CKEY_is_valid() && duration != -1)
+	{
+		if (!_warning)
+		{
+			fprintf(stderr, QT_NAME ": warning: calling the event loop during a keyboard event handler is ignored\n");
+			_warning = TRUE;
+		}
+		return;
+	}
+	
 	MAIN_in_wait++;
 	
-	if (duration > 0)
-	{
-		if (CKEY_is_valid())
-		{
-			if (!_warning)
-			{
-				fprintf(stderr, QT_NAME ": warning: calling the event loop during a keyboard event handler is ignored\n");
-				_warning = TRUE;
-			}
-		}
-		else
-			qApp->processEvents(QEventLoop::AllEvents, duration);
-	}
-	else if (duration < 0)
+	if (duration >= 0)
+		qApp->processEvents(QEventLoop::AllEvents, duration);
+	else if (duration == -1)
+		qApp->processEvents(QEventLoop::ExcludeUserInputEvents, 0);
+	else if (duration == -2)
 		qApp->processEvents(QEventLoop::AllEvents | QEventLoop::WaitForMoreEvents);
-	else
-		qApp->processEvents(QEventLoop::ExcludeUserInputEvents, duration);
 	
 	MAIN_in_wait--;
 }
@@ -1369,7 +1367,7 @@ GB_DESC *GB_CLASSES[] EXPORT =
 	CMenuChildrenDesc, CMenuDesc,
 	CButtonDesc, CToggleButtonDesc, CToolButtonDesc,
 	CCheckBoxDesc, CRadioButtonDesc,
-	CTextBoxSelectionDesc, CTextBoxDesc, CComboBoxItemDesc, CComboBoxDesc,
+	CTextBoxSelectionDesc, CTextBoxDesc,
 	CTextAreaSelectionDesc, CTextAreaDesc,
 	CPanelDesc, CHBoxDesc, CVBoxDesc, CHPanelDesc, CVPanelDesc,
 	CTabStripContainerChildrenDesc, CTabStripContainerDesc, CTabStripDesc,
@@ -1476,7 +1474,6 @@ int EXPORT GB_INIT(void)
 	CLASS_Image = GB.FindClass("Image");
 	CLASS_SvgImage = GB.FindClass("SvgImage");
 	CLASS_TextArea = GB.FindClass("TextArea");
-	CLASS_ComboBox = GB.FindClass("ComboBox");
 
 	QT_InitEventLoop();
 
