@@ -2,7 +2,7 @@
 
   c_webview.cpp
 
-  (c) Benoît Minisini <g4mba5@gmail.com>
+  (c) Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -128,6 +128,11 @@ static void set_link(void *_object, const QString &link)
 	THIS->link = QT.NewString(link);
 }
 
+static void update_language(void *_object)
+{
+	// TODO: it seems that Qt4 QWebView does not support setting the language.
+}
+
 //-------------------------------------------------------------------------
 
 BEGIN_METHOD(WebView_new, GB_OBJECT parent)
@@ -180,6 +185,9 @@ BEGIN_METHOD(WebView_new, GB_OBJECT parent)
 
 	/*QObject::connect(wid->page()->networkAccessManager(), SIGNAL(authenticationRequired(QNetworkReply *, QAuthenticator *)), &CWebView::manager,
 										SLOT(authenticationRequired(QNetworkReply *, QAuthenticator *)));*/
+
+	update_language(THIS);
+
 END_METHOD
 
 BEGIN_METHOD_VOID(WebView_free)
@@ -190,6 +198,7 @@ BEGIN_METHOD_VOID(WebView_free)
 	//GB.FreeString(&THIS->status);
 	//GB.FreeString(&THIS->userAgent);
 	GB.FreeString(&THIS->link);
+	GB.FreeString(&THIS->language);
 	GB.Unref(POINTER(&THIS->icon));
 	GB.Unref(POINTER(&THIS->new_view));
 
@@ -336,6 +345,24 @@ BEGIN_METHOD(WebView_ExecJavascript, GB_STRING script)
 	
 END_METHOD
 
+BEGIN_METHOD_VOID(WebView_GetHtml)
+
+	RETURN_NEW_STRING(WIDGET->page()->mainFrame()->toHtml());
+
+END_METHOD
+
+BEGIN_PROPERTY(WebView_Language)
+
+	if (READ_PROPERTY)
+		GB.ReturnString(THIS->language);
+	else
+	{
+		GB.StoreString(PROP(GB_STRING), &THIS->language);
+		update_language(THIS);
+	}
+
+END_PROPERTY
+
 //-------------------------------------------------------------------------
 
 static QWebHistoryItem get_item(QWebHistory *history, int index)
@@ -423,6 +450,30 @@ END_PROPERTY
 
 //-------------------------------------------------------------------------
 
+class MyCookieJar : public QNetworkCookieJar
+{
+	Q_OBJECT
+
+public:
+
+	MyCookieJar(QObject *parent = 0);
+
+	QList<QNetworkCookie> allCookies () const { return QNetworkCookieJar::allCookies(); }
+	void setAllCookies(const QList<QNetworkCookie> &cookieList) { QNetworkCookieJar::setAllCookies(cookieList); }
+
+	//virtual QList<QNetworkCookie> cookiesForUrl(const QUrl & url) const;
+	//virtual bool setCookiesFromUrl(const QList<QNetworkCookie> &cookieList, const QUrl &url);
+};
+
+BEGIN_METHOD_VOID(WebView_Cookies_Clear)
+
+	QList<QNetworkCookie> list;
+	static_cast<MyCookieJar *>(WIDGET->page()->networkAccessManager()->cookieJar())->setAllCookies(list);
+
+END_METHOD
+
+//-------------------------------------------------------------------------
+
 GB_DESC WebViewHistoryItemDesc[] = 
 {
 	GB_DECLARE_VIRTUAL(".WebView.History.Item"),
@@ -446,6 +497,15 @@ GB_DESC WebViewHistoryDesc[] =
 	GB_END_DECLARE
 };
 
+GB_DESC WebViewCookiesDesc[] =
+{
+	GB_DECLARE_VIRTUAL(".WebView.Cookies"),
+
+	GB_METHOD("Clear", NULL, WebView_Cookies_Clear, NULL),
+
+	GB_END_DECLARE
+};
+
 GB_DESC WebViewDesc[] =
 {
   GB_DECLARE("WebView", sizeof(CWEBVIEW)), GB_INHERITS("Control"),
@@ -460,8 +520,11 @@ GB_DESC WebViewDesc[] =
 	GB_PROPERTY_READ("Progress", "f", WebView_Progress),
 	GB_PROPERTY("NewView", "WebView", WebView_NewView),
 	GB_PROPERTY_READ("Link", "s", WebView_Link),
+	GB_PROPERTY("Language", "s", WebView_Language),
 
 	GB_METHOD("SetHtml", NULL, WebView_SetHtml, "(Html)s[(Root)s]"),
+	GB_METHOD("GetHtml", "s", WebView_GetHtml, NULL),
+
 	GB_METHOD("Clear", NULL, WebView_Clear, NULL),
 	
 	GB_METHOD("Back", NULL, WebView_Back, NULL),
@@ -473,7 +536,8 @@ GB_DESC WebViewDesc[] =
 
 	GB_PROPERTY_SELF("History", ".WebView.History"),
 	GB_PROPERTY_SELF("Settings", ".WebView.Settings"),
-	
+	GB_PROPERTY_SELF("Cookies", ".WebView.Cookies"),
+
 	GB_CONSTANT("_Properties", "s", "*,Url,Zoom=1"),
 	GB_CONSTANT("_Group", "s", "View"),
 	
