@@ -2,7 +2,7 @@
 
   CContainer.cpp
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -906,53 +906,20 @@ void MyContainer::changeEvent(QEvent *e)
 	}
 	
 	if (e->type() == QEvent::FontChange)
+	{
 		CALL_FUNCTION(THIS_USERCONTROL, font_func);
+	}
+	else if (e->type() == QEvent::EnabledChange)
+	{
+		update();
+	}
 }
 
-
-/*void MyContainer::childEvent(QChildEvent *e)
+void MyContainer::resizeEvent(QResizeEvent *e)
 {
-	//void *_object = CWidget::get(this);
-	void *child;
-	//qDebug("MyContainer::childEvent %p", CWidget::get(this));
-	
-	QFrame::childEvent(e);
-
-	if (!e->child()->isWidgetType())
-		return;
-
-	child = CWidget::get((QWidget *)e->child());
-
-	if (e->added())
-	{
-		//e->child()->installEventFilter(this);
-		//qApp->sendEvent(WIDGET, new QEvent(EVENT_INSERT));
-		//if (THIS_ARRANGEMENT->user)
-		//	GB.Raise(THIS, EVENT_Insert, 1, GB_T_OBJECT, child);    
-	}
-	else if (e->removed())
-	{
-		//e->child()->removeEventFilter(this);
-		//if (THIS_ARRANGEMENT->user)
-		//	GB.Raise(THIS, EVENT_Remove, 1, GB_T_OBJECT, child);
-	}
-
-	arrange_later(this);
-}*/
-
-/*bool MyContainer::eventFilter(QObject *o, QEvent *e)
-{
-	int type = e->type();
-
-	if (type == QEvent::Move || type == QEvent::Resize || type == QEvent::Show || type == QEvent::Hide || type == EVENT_EXPAND)
-	{
-		CWIDGET *ob = CWidget::getReal(o);
-		if (ob && (type == EVENT_EXPAND || !ob->flag.ignore))
-			arrange_now(this);
-	}
-
-	return QObject::eventFilter(o, e);
-}*/
+	void *_object = CWidget::get(this);
+	CALL_FUNCTION(THIS_USERCONTROL, resize_func);
+}
 
 
 /***************************************************************************
@@ -1088,7 +1055,7 @@ BEGIN_PROPERTY(UserContainer_Arrangement)
 
 END_PROPERTY
 
-BEGIN_PROPERTY(UserContainer_Focus)
+BEGIN_PROPERTY(UserControl_Focus)
 
 	if (READ_PROPERTY)
 		GB.ReturnBoolean(CWIDGET_get_allow_focus(THIS));
@@ -1289,6 +1256,8 @@ BEGIN_METHOD(UserControl_new, GB_OBJECT parent)
 			THIS_USERCONTROL->font_func = func.index;
 		if (!GB.GetFunction(&func, THIS, "UserControl_Change", NULL, NULL))
 			THIS_USERCONTROL->change_func = func.index;
+		if (!GB.GetFunction(&func, THIS, "UserControl_Resize", NULL, NULL))
+			THIS_USERCONTROL->resize_func = func.index;
 	}
 	
 	GB.Error(NULL);
@@ -1652,6 +1621,7 @@ GB_DESC UserControlDesc[] =
 	GB_PROPERTY("_Indent", "b", Container_Indent),
 	GB_PROPERTY("_Invert", "b", Container_Invert),
 	GB_PROPERTY("_Centered", "b", Container_Centered),
+	GB_PROPERTY("_Focus", "b", UserControl_Focus),
 
 	USERCONTROL_DESCRIPTION,
 	
