@@ -2,7 +2,7 @@
 
   CContainer.h
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -83,6 +83,7 @@ typedef
 		ushort paint_func;
 		ushort font_func;
 		ushort change_func;
+		ushort resize_func;
 		}
 	CUSERCONTROL;
 	
@@ -150,7 +151,7 @@ class MyFrame : public QWidget
 	Q_OBJECT
 	
 public:
-	MyFrame(QWidget *);
+	explicit MyFrame(QWidget *);
 	
 	int frameStyle() const { return _frame; }
 	void setFrameStyle(int frame);
@@ -188,6 +189,7 @@ protected:
 	virtual void hideEvent(QHideEvent *);
 	virtual void paintEvent(QPaintEvent *);
 	virtual void changeEvent(QEvent *);
+	virtual void resizeEvent(QResizeEvent *);
 };
 
 #endif
