@@ -2,7 +2,7 @@
 
 	CTextBox.h
 
-	(c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+	(c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -35,15 +35,12 @@
 
 extern GB_DESC CTextBoxSelectionDesc[];
 extern GB_DESC CTextBoxDesc[];
-extern GB_DESC CComboBoxDesc[];
-extern GB_DESC CComboBoxItemDesc[];
 
 #else
 
 #define QLINEEDIT(object) ((QLineEdit *)((CWIDGET *)object)->widget)
 
 #define TEXTBOX ((QLineEdit *)((CWIDGET *)_object)->widget)
-#define COMBOBOX ((MyComboBox *)((CWIDGET *)_object)->widget)
 #define THIS ((CTEXTBOX *)_object)
 
 #endif
@@ -57,39 +54,6 @@ typedef
 		}
 	CTEXTBOX;
 
-typedef
-	struct {
-		CWIDGET widget;
-		int start;
-		int length;
-		int locked;
-		int index;
-		bool click;
-		}
-	CCOMBOBOX;
-
-
-class MyComboBox : public QComboBox
-{
-	Q_OBJECT
-
-public:
-
-	MyComboBox(QWidget *parent);
-	virtual void changeEvent(QEvent *e);
-	void calcMinimumHeight();
-	bool isSortingEnabled() const { return _sorted; }
-	void setSortingEnabled(bool v) { _sorted = v; if (v) setDirty(); }
-	void setDirty() { _dirty = true; }
-	void sort();
-
-	virtual void showPopup();
-
-private:
-	bool _sorted;
-	bool _dirty;
-};
-
 
 class CTextBox : public QObject
 {
@@ -103,7 +67,6 @@ public slots:
 
 	void onChange(void);
 	void onActivate(void);
-	void onClick(void);
 	void onCursor(void);
 	//void onSelectionChanged(void);
 
