@@ -2,7 +2,7 @@
 
   CWindow.cpp
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -1624,30 +1624,6 @@ MyMainWindow::~MyMainWindow()
 	//qDebug("~MyMainWindow %p (end)", this);
 }
 
-#if 0
-bool MyMainWindow::event(QEvent *e)
-{
-	if (e->spontaneous() && !CWIDGET_test_flag(THIS, WF_DELETED))
-	{
-		/*if (e->type() == QEvent::WindowActivate)
-		{
-			qDebug("activate: %s %p", GB.GetClassName(THIS), THIS);
-			//CWINDOW_activate((CWIDGET *)THIS);
-			GB.Ref(THIS);
-			GB.Post((void (*)())activate_later, (intptr_t)THIS);
-
-		}
-		else*/ if (e->type() == QEvent::WindowDeactivate)
-		{
-			qDebug("deactivate: %s %p", GB.GetClassName(THIS), THIS);
-			if (THIS == CWINDOW_Active)
-				CWINDOW_activate(NULL);
-		}
-	}
-
-	return QWidget::event(e);
-}
-#endif
 
 void MyMainWindow::showEvent(QShowEvent *e)
 {
@@ -1954,11 +1930,7 @@ void MyMainWindow::doShowModal(bool popup, const QPoint *pos)
 
 		parent = CWINDOW_Current;
 		if (!parent)
-		{
-			parent = CWINDOW_Main;
-			if (!parent)
-				parent = CWINDOW_Active;
-		}
+			parent = CWINDOW_Active;
 
 		if (parent)
 			setParent(CWidget::getTopLevel((CWIDGET *)parent)->widget.widget, Qt::Window | info.flags);
@@ -2908,9 +2880,10 @@ void MyMainWindow::setBetterMask(QPixmap &bg)
 void MyMainWindow::activate(void)
 {
 #ifdef QT5
-	if (!MAIN_platform_is_wayland)
+	PLATFORM.Window.Activate(this);
+#else
+	activateWindow();
 #endif
-		activateWindow();
 }
 
 bool MyMainWindow::focusNextPrevChild(bool next)
@@ -3012,7 +2985,7 @@ void CWINDOW_activate(CWIDGET *ob)
 	if (CWINDOW_Active)
 	{
 		GB.Raise(CWINDOW_Active, EVENT_Deactivate, 0);
-		CWINDOW_Active = 0;
+		CWINDOW_Active = NULL;
 	}
 
 	if (active)
