@@ -2,7 +2,7 @@
 
   c_webview.cpp
 
-  (c) Benoît Minisini <g4mba5@gmail.com>
+  (c) Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -30,6 +30,8 @@
 #include <QWebFrame>
 #include <QWebHistory>*/
 #include <QWebEngineHistory>
+#include <QWebEngineProfile>
+#include <QWebEngineCookieStore>
 #include <QJsonDocument>
 #include <QContextMenuEvent>
 #include <QPointer>
@@ -43,10 +45,10 @@
 
 #define HISTORY (WIDGET->history())
 
-static bool _js_exited = FALSE;
-static volatile bool _js_running = FALSE;
-static bool _js_error = FALSE;
-static char *_js_result = NULL;
+static bool _cb_exited = FALSE;
+static volatile bool _cb_running = FALSE;
+static bool _cb_error = FALSE;
+static char *_cb_result = NULL;
 
 /*typedef
 	struct {
@@ -225,9 +227,40 @@ static void set_link(void *_object, const QString &link)
 	THIS->link = QT.NewString(link);
 }
 
+static bool start_callback()
+{
+	if (_cb_running)
+	{
+		GB.Error("Pending asynchronous method");
+		return true;
+	}
+
+	_cb_running = true;
+	return false;
+}
+
+static void run_callback(const char *error)
+{
+	while(_cb_running)
+		GB.Wait(-1);
+
+	if (_cb_error)
+	{
+		GB.Error(error);
+		GB.FreeString(&_cb_result);
+	}
+	else
+	{
+		GB.ReturnString(GB.FreeStringLater(_cb_result));
+		_cb_result = NULL;
+	}
+
+	_cb_error = FALSE;
+}
+
 static void cb_javascript_finished(const QVariant &result)
 {
-	if (_js_exited)
+	if (_cb_exited)
 		return;
 	
 	QVariantList value;
@@ -236,17 +269,39 @@ static void cb_javascript_finished(const QVariant &result)
 	QByteArray array = QJsonDocument::fromVariant(value).toJson(QJsonDocument::Compact);
 	
 	if (array.size() > 2)
-		_js_result = GB.NewString(array.constData() + 1, array.size() - 2);
+		_cb_result = GB.NewString(array.constData() + 1, array.size() - 2);
 	
-	_js_running = FALSE;
+	_cb_running = FALSE;
+}
+
+static void cb_html_finished(const QString &result)
+{
+	if (_cb_exited)
+		return;
+
+	_cb_result = QT.NewString(result);
+
+	_cb_running = FALSE;
+}
+
+static void update_language(void *_object)
+{
+	QString lang;
+
+	if (THIS->language && *THIS->language)
+		lang = TO_QSTRING(THIS->language);
+	else
+		lang = TO_QSTRING(GB.System.Language()).replace("_", "-");
+
+	WIDGET->page()->profile()->setHttpAcceptLanguage(lang);
 }
 
 //-------------------------------------------------------------------------
 
 BEGIN_METHOD_VOID(WebView_exit)
 
-	_js_exited = TRUE;
-	GB.FreeString(&_js_result);
+	_cb_exited = TRUE;
+	GB.FreeString(&_cb_result);
 
 END_METHOD
 
@@ -287,8 +342,10 @@ BEGIN_METHOD(WebView_new, GB_OBJECT parent)
   QObject::connect(wid, SIGNAL(loadStarted()), &WebViewSignalManager::manager, SLOT(loadStarted()));
   QObject::connect(wid, SIGNAL(loadProgress(int)), &WebViewSignalManager::manager, SLOT(loadProgress(int)));
   QObject::connect(wid, SIGNAL(loadFinished(bool)), &WebViewSignalManager::manager, SLOT(loadFinished(bool)));
+	//QObject::connect(wid, SIGNAL(destroyed()), &WebViewSignalManager::manager, SLOT(destroy()));
 
 	wid->clearPage(false);
+	update_language(THIS);
 
 	//QObject::connect(wid, SIGNAL(linkClicked(const QUrl &)), &CWebView::manager, SLOT(linkClicked(const QUrl &)));
 #if 0
@@ -310,6 +367,7 @@ END_METHOD
 BEGIN_METHOD_VOID(WebView_free)
 
 	GB.FreeString(&THIS->link);
+	GB.FreeString(&THIS->language);
 
 	GB.Unref(POINTER(&THIS->icon));
 	GB.Unref(POINTER(&THIS->new_view));
@@ -443,27 +501,38 @@ BEGIN_METHOD(WebView_ExecJavascript, GB_STRING script)
 	if (LENGTH(script) == 0)
 		return;
 	
-	_js_running = TRUE;
+	if (start_callback())
+		return;
+
 	WIDGET->page()->runJavaScript(QSTRING_ARG(script), cb_javascript_finished);
 	
-	while(_js_running)
-		GB.Wait(0);
-
-	if (_js_error)
-	{
-		GB.Error("Javascript error");
-		GB.FreeString(&_js_result);
-	}
-	else
-	{
-		GB.ReturnString(GB.FreeStringLater(_js_result));
-		_js_result = NULL;
-	}
-	
-	_js_error = FALSE;
+	run_callback("Javascript error");
 
 END_METHOD
 
+BEGIN_METHOD_VOID(WebView_GetHtml)
+
+	if (start_callback())
+		return;
+
+	WIDGET->page()->toHtml(cb_html_finished);
+
+	run_callback("Unable to retrieve HTML contents");
+
+END_METHOD
+
+BEGIN_PROPERTY(WebView_Language)
+
+	if (READ_PROPERTY)
+		GB.ReturnString(THIS->language);
+	else
+	{
+		GB.StoreString(PROP(GB_STRING), &THIS->language);
+		update_language(THIS);
+	}
+
+END_PROPERTY
+
 #if 0
 BEGIN_PROPERTY(WebView_HTML)
 
@@ -590,20 +659,20 @@ BEGIN_PROPERTY(WebView_Cookies)
 	QList<QNetworkCookie> list;
 	GB_ARRAY cookies;
 	int i;
-	
+
 	if (READ_PROPERTY)
 	{
 		list = cookieJar->allCookies();
-		
+
 		GB.Array.New(POINTER(&cookies), GB.FindClass("Cookie"), list.count());
-		
+
 		for (i = 0; i < list.count(); i++)
 		{
 			CCOOKIE *cookie = WEB_create_cookie(list.at(i));
 			*((CCOOKIE **)(GB.Array.Get(cookies, i))) = cookie;
 			GB.Ref(cookie);
 		}
-		
+
 		GB.ReturnObject(cookies);
 	}
 	else
@@ -612,7 +681,7 @@ BEGIN_PROPERTY(WebView_Cookies)
 		cookies = VPROP(GB_OBJECT);
 		if (GB.CheckObject(cookies))
 			return;
-		
+
 		for (i = 0; i < GB.Array.Count(cookies); i++)
 		{
 			CCOOKIE *cookie = *((CCOOKIE **)GB.Array.Get(cookies, i));
@@ -620,7 +689,7 @@ BEGIN_PROPERTY(WebView_Cookies)
 				continue;
 			list.append(*(cookie->cookie));
 		}
-		
+
 		cookieJar->setAllCookies(list);
 	}
 
@@ -727,7 +796,7 @@ static QWebEngineHistoryItem get_item(QWebEngineHistory *history, int index)
 	return history->itemAt(history->currentItemIndex() + index);
 }
 
-BEGIN_PROPERTY(WebViewHistoryItem_Title)
+BEGIN_PROPERTY(WebView_History_Item_Title)
 
 	QWebEngineHistoryItem item = get_item(HISTORY, THIS->history);
 	
@@ -738,7 +807,7 @@ BEGIN_PROPERTY(WebViewHistoryItem_Title)
 
 END_PROPERTY
 
-BEGIN_PROPERTY(WebViewHistoryItem_Url)
+BEGIN_PROPERTY(WebView_History_Item_Url)
 
 	QWebEngineHistoryItem item = get_item(HISTORY, THIS->history);
 	
@@ -749,7 +818,7 @@ BEGIN_PROPERTY(WebViewHistoryItem_Url)
 
 END_PROPERTY
 
-BEGIN_METHOD_VOID(WebViewHistoryItem_GoTo)
+BEGIN_METHOD_VOID(WebView_History_Item_GoTo)
 
 	QWebEngineHistoryItem item = get_item(HISTORY, THIS->history);
 	
@@ -758,13 +827,13 @@ BEGIN_METHOD_VOID(WebViewHistoryItem_GoTo)
 
 END_METHOD
 
-BEGIN_METHOD_VOID(WebViewHistory_Clear)
+BEGIN_METHOD_VOID(WebView_History_Clear)
 
 	HISTORY->clear();
 
 END_METHOD
 
-BEGIN_METHOD(WebViewHistory_get, GB_INTEGER index)
+BEGIN_METHOD(WebView_History_get, GB_INTEGER index)
 
 	int index = VARG(index);
 	
@@ -779,13 +848,13 @@ BEGIN_METHOD(WebViewHistory_get, GB_INTEGER index)
 
 END_METHOD
 
-BEGIN_PROPERTY(WebViewHistory_CanGoBack)
+BEGIN_PROPERTY(WebView_History_CanGoBack)
 
 	GB.ReturnBoolean(HISTORY->canGoBack());
 
 END_PROPERTY
 
-BEGIN_PROPERTY(WebViewHistory_CanGoForward)
+BEGIN_PROPERTY(WebView_History_CanGoForward)
 
 	GB.ReturnBoolean(HISTORY->canGoForward());
 
@@ -793,6 +862,14 @@ END_PROPERTY
 
 //-------------------------------------------------------------------------
 
+BEGIN_METHOD_VOID(WebView_Cookies_Clear)
+
+	WIDGET->page()->profile()->cookieStore()->deleteAllCookies();
+
+END_METHOD
+
+//-------------------------------------------------------------------------
+
 #if 0
 GB_DESC WebViewAuthDesc[] =
 {
@@ -868,14 +945,14 @@ GB_DESC WebViewDesc[] =
 };
 #endif
 
-GB_DESC WebViewHistoryItemDesc[] = 
+GB_DESC WebViewHistoryItemDesc[] =
 {
 	GB_DECLARE_VIRTUAL(".WebView.History.Item"),
-	
-	GB_PROPERTY_READ("Title", "s", WebViewHistoryItem_Title),
-	GB_PROPERTY_READ("Url", "s", WebViewHistoryItem_Url),
-	GB_METHOD("GoTo", NULL, WebViewHistoryItem_GoTo, NULL),
-	
+
+	GB_PROPERTY_READ("Title", "s", WebView_History_Item_Title),
+	GB_PROPERTY_READ("Url", "s", WebView_History_Item_Url),
+	GB_METHOD("GoTo", NULL, WebView_History_Item_GoTo, NULL),
+
 	GB_END_DECLARE
 };
 
@@ -883,10 +960,19 @@ GB_DESC WebViewHistoryDesc[] =
 {
 	GB_DECLARE_VIRTUAL(".WebView.History"),
 
-	GB_METHOD("Clear", NULL, WebViewHistory_Clear, NULL),
-	GB_METHOD("_get", ".WebView.History.Item", WebViewHistory_get, "(Index)i"),
-	GB_PROPERTY_READ("CanGoBack", "b", WebViewHistory_CanGoBack),
-	GB_PROPERTY_READ("CanGoForward", "b", WebViewHistory_CanGoForward),
+	GB_METHOD("Clear", NULL, WebView_History_Clear, NULL),
+	GB_METHOD("_get", ".WebView.History.Item", WebView_History_get, "(Index)i"),
+	GB_PROPERTY_READ("CanGoBack", "b", WebView_History_CanGoBack),
+	GB_PROPERTY_READ("CanGoForward", "b", WebView_History_CanGoForward),
+
+	GB_END_DECLARE
+};
+
+GB_DESC WebViewCookiesDesc[] =
+{
+	GB_DECLARE_VIRTUAL(".WebView.Cookies"),
+
+	GB_METHOD("Clear", NULL, WebView_Cookies_Clear, NULL),
 
 	GB_END_DECLARE
 };
@@ -907,8 +993,11 @@ GB_DESC WebViewDesc[] =
 	GB_PROPERTY_READ("Progress", "f", WebView_Progress),
 	GB_PROPERTY("NewView", "WebView", WebView_NewView),
 	GB_PROPERTY_READ("Link", "s", WebView_Link),
+	GB_PROPERTY("Language", "s", WebView_Language),
 	
 	GB_METHOD("SetHtml", NULL, WebView_SetHtml, "(Html)s[(Root)s]"),
+	GB_METHOD("GetHtml", "s", WebView_GetHtml, NULL),
+
 	GB_METHOD("Clear", NULL, WebView_Clear, NULL),
 	
 	GB_METHOD("Back", NULL, WebView_Back, NULL),
@@ -920,7 +1009,8 @@ GB_DESC WebViewDesc[] =
 	
 	GB_PROPERTY_SELF("History", ".WebView.History"),
 	GB_PROPERTY_SELF("Settings", ".WebView.Settings"),
-	
+	GB_PROPERTY_SELF("Cookies", ".WebView.Cookies"),
+
 	GB_CONSTANT("_Properties", "s", "*,Url,Zoom=1"),
 	GB_CONSTANT("_Group", "s", "View"),
 	
@@ -941,8 +1031,15 @@ GB_DESC WebViewDesc[] =
 
 MyWebEngineView::MyWebEngineView(QWidget *parent) : QWebEngineView(parent)
 {
+	profile = new QWebEngineProfile(this);
 	//settings()->setFontFamily(QWebSettings::FixedFont, "monospace");
-	setPage(new MyWebPage(this));
+	//setPage(new MyWebPage(profile, this));
+}
+
+MyWebEngineView::~MyWebEngineView()
+{
+	delete page();
+	delete profile;
 }
 
 void MyWebEngineView::clearPage(bool destroy)
@@ -951,7 +1048,7 @@ void MyWebEngineView::clearPage(bool destroy)
 	if (destroy)
 		p = page();
 	
-	setPage(new MyWebPage(this));
+	setPage(new MyWebPage(profile, this));
 	QObject::connect(page(), SIGNAL(linkHovered(const QString &)), &WebViewSignalManager::manager, SLOT(linkHovered(const QString &)));
 	
 	if (destroy && p)
@@ -991,7 +1088,7 @@ void MyWebEngineView::contextMenuEvent(QContextMenuEvent *event)
 
 //-------------------------------------------------------------------------
 
-MyWebPage::MyWebPage(QObject *parent) : QWebEnginePage(parent)
+MyWebPage::MyWebPage(QWebEngineProfile *profile, QObject *parent) : QWebEnginePage(profile, parent)
 {
 }
 
