@@ -2,14 +2,11 @@
 
   Ckey.h
 
-  Gambas extension using SDL
-
   (c) 2006 Laurent Carlier <lordheavy@users.sourceforge.net>
-           Benoît Minisini <gambas@users.sourceforge.net>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -19,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -31,14 +29,15 @@
 
 #include "SDL.h"
 #include "SDL_syswm.h"
-#include "X11/keysym.h"
+//#include "X11/keysym.h"
 
 typedef
   struct {
-    bool valid;
-    KeySym code;
-    int state;
-    int cancel;
+    int valid;
+		int code;           // SDL virtual keysym
+		int state;          // modifier state
+    //int cancel;
+		char text[4];       // UTF-8 text
     }
   CKEY_INFO;
 
