@@ -2,7 +2,7 @@
 
   gbc.c
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -798,7 +798,6 @@ int main(int argc, char **argv)
 		
 		if (_convert_form)
 		{
-			COMP_do_not_lock = TRUE;
 			FORM_convert(_convert_form);
 			STR_free(_convert_form);
 		}
@@ -814,6 +813,7 @@ int main(int argc, char **argv)
 			}
 
 			COMPILE_init();
+			COMP_do_not_lock = FALSE;
 
 			// Remove information files if we are compiling everything
 
@@ -841,7 +841,7 @@ int main(int argc, char **argv)
 			if (main_trans)
 				compile_all_lang();
 			
-			COMPILE_exit();
+			COMPILE_exit(_ntask_max == 1);
 			FILE_exit();
 
 			puts("OK");
