@@ -2,7 +2,7 @@
 
   gbc_class.c
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -58,8 +58,8 @@ void CLASS_create(CLASS **result)
 	ARRAY_create(&class->structure);
 	ARRAY_create(&class->names);
 
-	TABLE_create(&class->table, sizeof(CLASS_SYMBOL), TF_IGNORE_CASE);
-	TABLE_create(&class->string, sizeof(SYMBOL), TF_NORMAL);
+	TABLE_create_inc(&class->table, sizeof(CLASS_SYMBOL), TF_IGNORE_CASE, 1024);
+	TABLE_create_inc(&class->string, sizeof(SYMBOL), TF_NORMAL, 1024);
 
 	CLEAR(&func);
 	TYPE_clear(&func.type);
@@ -405,8 +405,8 @@ void CLASS_add_property(CLASS *class, TRANS_PROPERTY *decl)
 	prop = ARRAY_add_void(&class->prop);
 	TYPE_clear(&prop->type);
 	prop->name = NO_SYMBOL;
-	prop->read = TRUE;
-	prop->write = decl->read == 0;
+	prop->read = decl->write_only ? 0 : -1;
+	prop->write = decl->read_only ? 0 : -1;
 	prop->synonymous = -1;
 
 	sym = CLASS_declare(class, decl->index, TK_PROPERTY, TRUE);
@@ -429,8 +429,8 @@ void CLASS_add_property(CLASS *class, TRANS_PROPERTY *decl)
 		prop = ARRAY_add_void(&class->prop);
 		TYPE_clear(&prop->type);
 		prop->name = NO_SYMBOL;
-		prop->read = TRUE;
-		prop->write = decl->read == 0;
+		prop->read = decl->write_only ? 0 : -1;
+		prop->write = decl->read_only ? 0 : -1;
 		prop->synonymous = index;
 
 		sym = CLASS_declare(class, decl->synonymous[i], TK_PROPERTY, TRUE);
@@ -1004,11 +1004,10 @@ void CLASS_check_properties(CLASS *class)
 		}
 		else
 		{
-			prop->read = check_one_property_func(class, prop, FALSE);
-			if (prop->write)
-				prop->write = check_one_property_func(class, prop, TRUE);
-			else
-				prop->write = NO_SYMBOL;
+			// TODO: if COMPILER_version < 0x3180000, and if there is no prop->read, then
+			// replace it by a function that raises an error.
+			prop->read = prop->read ? check_one_property_func(class, prop, FALSE) : NO_SYMBOL;
+			prop->write = prop->write ? check_one_property_func(class, prop, TRUE) : NO_SYMBOL;
 		}
 	}
 }
