@@ -2,7 +2,7 @@
 
   gbc_compile.c
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -64,9 +64,9 @@ char *COMP_info_path;
 char *COMP_lib_path;
 char *COMP_classes = NULL;
 COMPILE COMP_current;
-uint COMPILE_version = GAMBAS_PCODE_VERSION;
+uint COMP_version = GAMBAS_PCODE_VERSION;
 char *COMP_default_namespace = NULL;
-bool COMP_do_not_lock = FALSE;
+bool COMP_do_not_lock = TRUE;
 
 #define STARTUP_MAX_LINE 256
 
@@ -334,6 +334,8 @@ static void startup_print_version(FILE *fs)
 	FILE *fp;
 	char line[256];
 	char *version = NULL;
+	char *branch = NULL;
+	bool add_branch = FALSE;
 
 	fp = open_project_file();
 
@@ -342,18 +344,19 @@ static void startup_print_version(FILE *fs)
 		if (read_line(fp, line, sizeof(line)))
 			break;
 
-		if (line_begins_with(line, "VersionFile=", 12))
+		if (line_begins_with(line, "Version=", 8))
+		{
+			version = STR_copy(&line[8]);
+			branch = index(version, ' ');
+		}
+		else if (line_begins_with(line, "VersionFile=", 12))
 		{
 			if (line[12] == '1')
 			{
 				version = find_version_in_file();
-				break;
+				add_branch = TRUE;
 			}
 		}
-		else if (line_begins_with(line, "Version=", 8))
-		{
-			version = STR_copy(&line[8]);
-		}
 	}
 
 	fclose(fp);
@@ -361,6 +364,8 @@ static void startup_print_version(FILE *fs)
 	if (version)
 	{
 		fputs(version, fs);
+		if (add_branch && branch)
+			fputs(branch, fs);
 		fputc('\n', fs);
 		STR_free(version);
 	}
@@ -457,7 +462,7 @@ static void init_version(void)
 			}
 		}
 
-		COMPILE_version = v;
+		COMP_version = v;
 	}
 }
 
@@ -640,8 +645,11 @@ void COMPILE_end(void)
 }
 
 
-void COMPILE_exit(void)
+void COMPILE_exit(bool can_dump_count)
 {
+	/*if (COMP_verbose && can_dump_count)
+		PCODE_dump_count(stdout);*/
+
 	RESERVED_exit();
 	BUFFER_delete(&COMP_classes);
 	STR_free(COMP_project_name);
