@@ -2,7 +2,7 @@
 
   gbc_compile.h
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -104,17 +104,18 @@ EXTERN char *COMP_project;
 EXTERN char *COMP_project_name;
 EXTERN char *COMP_info_path;
 EXTERN FORM_FAMILY COMP_form_families[];
-EXTERN uint COMPILE_version;
+EXTERN uint COMP_version;
 EXTERN char *COMP_default_namespace;
 EXTERN char *COMP_classes;
 EXTERN bool COMP_do_not_lock;
+
 #endif
 
 #define JOB (&COMP_current)
 
 void COMPILE_init(void);
 void COMPILE_load(void);
-void COMPILE_exit(void);
+void COMPILE_exit(bool can_dump_count);
 void COMPILE_begin(const char *file, bool trans, bool debug);
 void COMPILE_alloc();
 void COMPILE_free();
