@@ -2,7 +2,7 @@
 
   gbc_output.c
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -72,7 +72,7 @@ static char *_const_buffer = NULL;
 
 static const char *get_symbol_name(TABLE *table, int index, int *len)
 {
-	if (UNLIKELY((index < 0) || (index >= ARRAY_count(table->symbol))))
+	if (index < 0 || index >= ARRAY_count(table->symbol))
 	{
 		*len = 1;
 		return "?";
@@ -349,7 +349,7 @@ static void output_header(void)
 	/* magic */
 	write_int(OUTPUT_MAGIC);
 	/* version */
-	write_int(COMPILE_version);
+	write_int(COMP_version);
 	/* endianness */
 	write_int(OUTPUT_ENDIAN);
 	/* flag */
