@@ -2,7 +2,7 @@
 
 	gbc_trans.h
 
-	(c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+	(c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -136,7 +136,7 @@ typedef
 EXTERN short TRANS_in_assignment;
 EXTERN short TRANS_in_left_value;
 EXTERN bool TRANS_in_try;
-EXTERN ushort *TRANS_labels;
+//EXTERN ushort *TRANS_labels;
 #endif
 
 #define TRANS_newline() (PATTERN_is_newline(*JOB->current) ? JOB->line = PATTERN_index(*JOB->current) + 1, JOB->current++, TRUE : FALSE)
@@ -166,14 +166,14 @@ void TRANS_code(void);
 bool TRANS_init_var(TRANS_DECL *decl);
 void TRANS_statement(void);
 void TRANS_init_optional(TRANS_PARAM *param);
-#define TRANS_add_label(_pos) (TRANS_labels ? *ARRAY_add(&TRANS_labels) = (_pos) : 0)
+//#define TRANS_add_label(_pos) (TRANS_labels ? *ARRAY_add(&TRANS_labels) = (_pos) : 0)
 int TRANS_loop_local(bool allow_arg);
 
 // gbc_trans_expr.c
 
 void TRANS_expression(bool check);
 void TRANS_ignore_expression(void);
-bool TRANS_popify_last(void);
+bool TRANS_popify_last(bool no_conv);
 void TRANS_reference(void);
 bool TRANS_affectation(bool dup);
 void TRANS_operation(short op, short nparam, bool output, PATTERN previous);
@@ -181,6 +181,7 @@ void TRANS_new(void);
 TYPE TRANS_variable_get_type(void);
 void TRANS_class(int index);
 bool TRANS_string(PATTERN pattern);
+TYPE TRANS_get_last_type(void);
 
 // gbc_trans_const.c
 
@@ -278,6 +279,7 @@ void TRANS_scan(void);
 void TRANS_randomize(void);
 void TRANS_mid(void);
 void TRANS_use(void);
+void TRANS_poke(void);
 
 #endif
 
