@@ -2,7 +2,7 @@
 
 	gbc_trans_code.c
 
-	(c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+	(c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -34,9 +34,9 @@
 #include "gb_code.h"
 #include "gb_limit.h"
 
-/*#define DEBUG*/
+//#define DEBUG
 
-ushort *TRANS_labels = NULL;
+//ushort *TRANS_labels = NULL;
 
 static FUNCTION *_func;
 
@@ -210,7 +210,8 @@ static bool TRANS_local(void)
 
 				if (TRANS_init_var(&decl))
 				{
-					CODE_pop_local(sym->local.value);
+					CODE_push_local_ref(sym->local.value, TYPE_must_ref(sym->local.type));
+					TRANS_popify_last(TYPE_get_id(TRANS_get_last_type()) == TYPE_get_id(sym->local.type));
 					sym->local_assigned = TRUE;
 				}
 
@@ -341,6 +342,7 @@ void TRANS_statement(void)
 	PATTERN *look = JOB->current;
 	TRANS_STATEMENT *st;
 	COMP_INFO *info;
+	int index;
 
 	if (PATTERN_is_reserved(look[0]))
 	{
@@ -367,11 +369,21 @@ void TRANS_statement(void)
 			return;
 		}
 	}
-	else if (PATTERN_is_subr(look[0]) && (PATTERN_index(look[0]) == SUBR_Mid || PATTERN_index(look[0]) == SUBR_MidS))
+	else if (PATTERN_is_subr(look[0]))
 	{
-		JOB->current++;
-		TRANS_mid();
-		return;
+		index = PATTERN_index(look[0]);
+
+		if (index == SUBR_Mid || index == SUBR_MidS)
+		{
+			JOB->current++;
+			TRANS_mid();
+			return;
+		}
+		else if (COMP_version >= 0x03180000 && COMP_subr_info[index].opcode == (CODE_POKE - CODE_FIRST_SUBR))
+		{
+			TRANS_poke();
+			return;
+		}
 	}
 
 	if (!TRANS_affectation(FALSE))
