@@ -2,7 +2,7 @@
 
   gbc_trans_expr.c
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -184,6 +184,10 @@ static void push_number(int index)
 	{
 		CODE_push_number(number.ival);
 	}
+	else if (number.type == T_FLOAT && COMP_version >= 0x03180000 && number.dval == (double)(int)number.dval && number.dval >= -128 && number.dval <= 127)
+	{
+		CODE_push_float(number.dval);
+	}
 	else
 	{
 		CLEAR(&decl);
@@ -279,7 +283,7 @@ static void trans_identifier(int index, bool point, PATTERN next)
 		if (TYPE_is_static(sym->local.type))
 			CODE_push_global(sym->local.value, TRUE, FALSE);
 		else
-			CODE_push_local(sym->local.value);
+			CODE_push_local_ref(sym->local.value, TYPE_must_ref(sym->local.type));
 		
 		push_type(sym->local.type);
 		sym->local_used = TRUE;
@@ -512,7 +516,7 @@ static void trans_operation(short op, short nparam, PATTERN previous)
 				type = T_FLOAT;
 			ftype = TYPE_make_simple(type);
 			break;
-			
+
 		case RST_AND:
 			type1 = get_type_id(0, nparam);
 			type2 = get_type_id(1, nparam);
@@ -554,6 +558,15 @@ static void trans_operation(short op, short nparam, PATTERN previous)
 			ftype = TYPE_make_simple(Max(type1, type2));
 			break;
 			
+		case RST_DIV:
+			type = Max(get_type_id(0, nparam), get_type_id(1, nparam));
+			if (type <= T_FLOAT)
+				type = T_FLOAT;
+			else
+				type = T_VARIANT;
+			ftype = TYPE_make_simple(type);
+			break;
+
 		case RST_GET:
 			ftype = TYPE_make_simple(T_VARIANT);
 			break;
@@ -720,7 +733,7 @@ static void trans_expr_from_tree(TRANS_TREE *tree, int count)
 		}
 		else if (PATTERN_is(pattern, RS_AT))
 		{
-			if (TRANS_popify_last())
+			if (TRANS_popify_last(FALSE))
 				THROW("This expression cannot be passed by reference");
 		}
 		else if (PATTERN_is(pattern, RS_COMMA))
@@ -981,9 +994,9 @@ TYPE TRANS_variable_get_type()
 }
 
 
-bool TRANS_popify_last()
+bool TRANS_popify_last(bool no_conv)
 {
-	if (!CODE_popify_last())
+	if (!CODE_popify_last(no_conv))
 		return TRUE;
 	
 	if (_last_symbol_used)
@@ -1001,7 +1014,15 @@ bool TRANS_popify_last()
 void TRANS_reference(void)
 {
 	TRANS_expression(FALSE);
-	if (TRANS_popify_last())
+	if (TRANS_popify_last(FALSE))
+		THROW("Invalid assignment");
+}
+
+
+void TRANS_reference_type(TYPE type)
+{
+	TRANS_expression(FALSE);
+	if (TRANS_popify_last(TYPE_get_id(_last_type) == TYPE_get_id(type)))
 		THROW("Invalid assignment");
 }
 
@@ -1119,11 +1140,11 @@ bool TRANS_affectation(bool dup)
 		{
 			push_type(_last_type);
 			push_type(type);
-			/*if (op == RS_AMP && COMPILE_version >= 0x03150000)
+			/*if (op == RS_AMP && COMP_version >= 0x03150000)
 				trans_operation(op, 1, NULL_PATTERN);
 			else*/
 				trans_operation(op, 2, NULL_PATTERN);
-			pop_type();
+			_last_type = pop_type();
 		}
 	}
 
@@ -1132,24 +1153,24 @@ bool TRANS_affectation(bool dup)
 	if (dup)
 		CODE_dup();
 
-	if (COMPILE_version >= 0x03070000)
+	if (COMP_version >= 0x03070000)
 	{
 		if (id == RS_EXEC || id == RS_SHELL)
 			CODE_dup();
 	}
 
 	JOB->current = left;
-	TRANS_reference();
+	TRANS_reference_type(_last_type);
 
 	if (!PATTERN_is_newline(*JOB->current))
 		THROW(E_SYNTAX);
 
-	if (COMPILE_version >= 0x03150000 && op == RS_AMP)
+	if (COMP_version >= 0x03150000 && op == RS_AMP)
 		CODE_check_fast_cat();
 	
 	JOB->current = after;
 
-	if (COMPILE_version >= 0x03070000)
+	if (COMP_version >= 0x03070000)
 	{
 		if (id == RS_EXEC || id == RS_SHELL)
 		{
@@ -1168,3 +1189,8 @@ void TRANS_expression(bool check_statement)
 	trans_expression(check_statement);
 }
 
+
+TYPE TRANS_get_last_type(void)
+{
+	return _last_type;
+}
