@@ -2,7 +2,7 @@
 
   gbc_trans_subr.c
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -1007,6 +1007,25 @@ void TRANS_mid(void)
 }
 
 
+void TRANS_poke(void)
+{
+	int index;
+
+	index = PATTERN_index(*JOB->current);
+	JOB->current++;
+
+	TRANS_want(RS_LBRA, "Left bracket");
+	TRANS_expression(FALSE);
+	TRANS_want(RS_RBRA, "Right bracket");
+
+	TRANS_want(RS_EQUAL, "Equal");
+
+	TRANS_expression(FALSE);
+
+	CODE_op(C_POKE, COMP_subr_info[index].optype, 2, TRUE);
+	CODE_drop();
+}
+
 
 #if 0
 void TRANS_scan(void)
