@@ -2,7 +2,7 @@
 
 	gb_error.c
 
-	(c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+	(c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -536,6 +536,11 @@ void THROW_STACK(void)
 	THROW(E_STACK);
 }
 
+void THROW_BOUND(void)
+{
+	THROW(E_BOUND);
+}
+
 void THROW_SYSTEM(int err, const char *path)
 {
 	char buf[6];
@@ -572,6 +577,16 @@ void THROW_SYSTEM(int err, const char *path)
 	}
 }
 
+void THROW_MATH(bool zero)
+{
+	THROW(zero ? E_ZERO : E_MATH);
+}
+
+void THROW_OVERFLOW_(void)
+{
+	THROW(E_OVERFLOW);
+}
+
 
 void ERROR_fatal(const char *error, ...)
 {
