@@ -2,7 +2,7 @@
 
 	gbx_api.c
 
-	(c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+	(c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -99,7 +99,7 @@ const void *const GAMBAS_Api[] =
 	(void *)EVENT_post,
 	(void *)EVENT_post2,
 	(void *)CTIMER_every,
-	(void *)GB_Raise,
+	(void *)GB_RaiseEvent,
 	(void *)GB_RaiseBegin,
 	(void *)GB_RaiseEnd,
 	(void *)EVENT_post_event,
@@ -118,6 +118,7 @@ const void *const GAMBAS_Api[] =
 
 	(void *)GB_Error,
 	(void *)GB_HasError,
+	(void *)GB_GetErrorMessage,
 	(void *)ERROR_propagate,
 	(void *)GB_Deprecated,
 	(void *)GB_OnErrorBegin,
@@ -126,9 +127,9 @@ const void *const GAMBAS_Api[] =
 	(void *)GB_GetClass,
 	(void *)GB_GetClassName,
 	(void *)GB_ExistClass,
-	(void *)CLASS_find_global,
+	(void *)GB_FindClass,
 	(void *)GB_ExistClassLocal,
-	(void *)CLASS_find,
+	(void *)GB_FindClassLocal,
 	(void *)GB_GetArrayType,
 	(void *)CLASS_get_array_class,
 	(void *)CLASS_find_load_from,
@@ -196,6 +197,7 @@ const void *const GAMBAS_Api[] =
 
 	(void *)STRING_subst,
 	(void *)STRING_subst_add,
+	(void *)STRING_subst_add_unquote,
 	(void *)STRING_make,
 	(void *)GB_ConvString,
 	(void *)STRING_conv_file_name,
@@ -345,7 +347,6 @@ const void *const GAMBAS_JitApi[] =
 	(void *)&SP,
 	(void *)&EXEC_current,
 	(void *)&TEMP,
-	(void *)&EXEC_debug,
 	(void *)&EXEC_super,
 	(void *)JIT_debug,
 	(void *)JIT_get_code,
@@ -372,7 +373,7 @@ const void *const GAMBAS_JitApi[] =
 	(void *)&ERROR_handler,
 	(void *)ERROR_reset,
 	(void *)ERROR_set_last,
-	(void *)&EXEC_got_error,
+	(void *)EXEC_set_got_error,
 	(void *)&EVENT_Last,
 	(void *)EXEC_push_complex,
 	(void *)EXEC_push_vargs,
@@ -388,12 +389,15 @@ const void *const GAMBAS_JitApi[] =
 	(void *)JIT_add_string_local,
 	(void *)JIT_add_string_global,
 	(void *)VALUE_class_write,
+	(void *)SUBR_poke,
+	(void *)OBJECT_get_addr,
 	NULL
 };
 
 
 bool GAMBAS_DoNotRaiseEvent = FALSE;
 bool GAMBAS_StopEvent = FALSE;
+bool GAMBAS_RaiseEventCanPropagate = FALSE;
 
 static bool _event_stopped = FALSE;
 static int _raise_event_level = 0;
@@ -481,6 +485,7 @@ bool GB_LoadComponent(const char *name)
 static void push(int nval, va_list args)
 {
 	TYPE type;
+	GB_VARIANT_VALUE *val;
 
 	STACK_check(nval);
 
@@ -518,6 +523,13 @@ static void push(int nval, va_list args)
 				OBJECT_REF_CHECK(SP->_object.object);
 				break;
 
+			case T_VARIANT:
+				val = va_arg(args, GB_VARIANT_VALUE *);
+				SP->_variant.vtype = val->type;
+				SP->_variant.value.data = val->value.data;
+				BORROW(SP);
+				break;
+
 			default:
 				ERROR_panic("GB.Push: unknown datatype");
 				break;
@@ -600,6 +612,11 @@ void *GB_GetProperty(void *object, const char *name)
 			return NULL;
 		}
 	}
+	else if (type == CD_PROPERTY_WRITE || type == CD_STATIC_PROPERTY_WRITE)
+	{
+		error(E_NREAD, class, name);
+		return NULL;
+	}
 	else
 	{
 		error(E_NPROPERTY, class, name);
@@ -664,7 +681,7 @@ bool GB_SetProperty(void *object, const char *name, GB_VALUE *val)
 
 	type = CLASS_DESC_get_type(desc);
 
-	if (type == CD_PROPERTY || type == CD_VARIABLE)
+	if (type == CD_PROPERTY || type == CD_VARIABLE || type == CD_PROPERTY_WRITE)
 	{
 		if (!object)
 		{
@@ -677,7 +694,7 @@ bool GB_SetProperty(void *object, const char *name, GB_VALUE *val)
 			object = EXEC_auto_create(class, TRUE);
 		}
 	}
-	else if (type == CD_STATIC_PROPERTY || type == CD_STATIC_VARIABLE)
+	else if (type == CD_STATIC_PROPERTY || type == CD_STATIC_VARIABLE || type == CD_STATIC_PROPERTY_WRITE)
 	{
 		if (object)
 		{
@@ -793,7 +810,7 @@ static int get_event_func_id(ushort *event_tab, int event_id)
 	return func_id;
 }
 
-static bool raise_event(OBJECT *observer, void *object, int func_id, int nparam)
+static bool raise_event(OBJECT *observer, void *object, int func_id, int nparam, bool can_propagate)
 {
 	bool stop_event;
 	CLASS *class;
@@ -840,7 +857,11 @@ static bool raise_event(OBJECT *observer, void *object, int func_id, int nparam)
 	}
 	CATCH 
 	{
-		if (ERROR->info.code && ERROR->info.code != E_ABORT) // && !STACK_has_error_handler())
+		// Errors cannot be propagated outside of an event handler, because 
+		// the event may result from a library signal that do not support 
+		// exceptions (like the glib library).
+		
+		if (ERROR->info.code && ERROR->info.code != E_ABORT && !(can_propagate && STACK_has_error_handler()))
 		{
 			ERROR_hook();
 
@@ -912,7 +933,7 @@ void GB_RaiseEnd(GB_RAISE_HANDLER *handler)
 	_GB_Raise_handler = handler->old;
 }
 
-bool GB_Raise(void *object, int event_id, int nparam, ...)
+bool GB_RaiseEvent(void *object, int event_id, int nparam, ...)
 {
 	OBJECT *parent;
 	int func_id;
@@ -920,7 +941,11 @@ bool GB_Raise(void *object, int event_id, int nparam, ...)
 	va_list args;
 	bool arg;
 	COBSERVER *obs;
+	bool can_propagate;
 
+	can_propagate = GAMBAS_RaiseEventCanPropagate;
+	GAMBAS_RaiseEventCanPropagate = FALSE;
+	
 	if (GAMBAS_DoNotRaiseEvent)
 		return FALSE;
 
@@ -968,7 +993,7 @@ bool GB_Raise(void *object, int event_id, int nparam, ...)
 			}
 
 			EXEC_dup(nparam);
-			result = raise_event(parent, object, func_id, nparam);
+			result = raise_event(parent, object, func_id, nparam, can_propagate);
 
 			if (result)
 				goto __RETURN;
@@ -994,7 +1019,7 @@ bool GB_Raise(void *object, int event_id, int nparam, ...)
 				else
 				{
 					EXEC_dup(nparam);
-					result = raise_event(parent, object, func_id, nparam);
+					result = raise_event(parent, object, func_id, nparam, can_propagate);
 					if (result)
 						goto __RETURN;
 				}
@@ -1024,7 +1049,7 @@ bool GB_Raise(void *object, int event_id, int nparam, ...)
 			}
 
 			EXEC_dup(nparam);
-			result = raise_event(parent, object, func_id, nparam);
+			result = raise_event(parent, object, func_id, nparam, can_propagate);
 			if (result)
 				goto __RETURN;
 		}
@@ -1276,11 +1301,19 @@ void GB_Error(const char *error, ...)
 	EXEC_set_native_error(TRUE);
 }
 
+
 bool GB_HasError()
 {
 	return EXEC_has_native_error();
 }
 
+
+char *GB_GetErrorMessage()
+{
+	return ERROR_last.msg;
+}
+
+
 void GB_Deprecated(const char *msg, const char *func, const char *repl)
 {
 	if (EXEC_debug)
@@ -2160,7 +2193,7 @@ int GB_CountArray(void *data)
 
 void *GB_Add(void *pdata)
 {
-	return ARRAY_add_void_size(pdata);
+	return ARRAY_add_void_size((char **)pdata);
 }
 
 char *GB_NewZeroString(char *src)
@@ -2488,6 +2521,16 @@ bool GB_ExistClassLocal(const char *name)
 	return CLASS_look(name, strlen(name)) != NULL;
 }
 
+void *GB_FindClass(const char *name)
+{
+	return CLASS_find_global(name);
+}
+
+void *GB_FindClassLocal(const char *name)
+{
+	return CLASS_find(name);
+}
+
 TYPE GB_GetArrayType(void *klass)
 {
   return ((CLASS *)klass)->array_type;
@@ -2516,8 +2559,6 @@ void GB_Wait(int delay)
 		for(;;)
 		{
 			duration = (int)(wait * 1000 + 0.5);
-			if (duration <= 0)
-				duration = 1;
 			HOOK_DEFAULT(wait, WATCH_wait)(duration);
 
 			if (DATE_timer(&time, FALSE))
