@@ -2,7 +2,7 @@
 
 	gbx_archive.h
 
-	(c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+	(c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@ typedef
 		char *version;
 		char *domain;
 		TABLE *classes;
-		const char *path;
+		char *path;
 		void *current_component;
 		struct _CLASS **exported;
 		void *jit_library;
