@@ -2,7 +2,7 @@
 
   gbx_c_application.c
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -310,6 +310,7 @@ BEGIN_PROPERTY(Application_Startup)
 
 END_PROPERTY
 
+
 BEGIN_PROPERTY(Application_Priority)
 
 	int pr;
@@ -337,6 +338,13 @@ BEGIN_PROPERTY(Application_Priority)
 
 END_PROPERTY
 
+
+BEGIN_PROPERTY(Application_Task)
+
+  GB_ReturnBoolean(EXEC_task);
+
+END_PROPERTY
+
 #endif
 
 //-------------------------------------------------------------------------
@@ -389,6 +397,7 @@ GB_DESC NATIVE_App[] =
   GB_STATIC_PROPERTY("Daemon", "b", Application_Daemon),
   GB_STATIC_PROPERTY_READ("Startup", "Class", Application_Startup),
 	GB_STATIC_PROPERTY("Priority", "i", Application_Priority),
+  GB_STATIC_PROPERTY_READ("Task", "b", Application_Task),
 
   GB_END_DECLARE
 };
