@@ -2,7 +2,7 @@
 
   gbx_c_class.c
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -431,8 +431,10 @@ BEGIN_PROPERTY(Symbol_Kind)
 
 		case CD_PROPERTY:
 		case CD_PROPERTY_READ:
+		case CD_PROPERTY_WRITE:
 		case CD_STATIC_PROPERTY:
 		case CD_STATIC_PROPERTY_READ:
+		case CD_STATIC_PROPERTY_WRITE:
 			GB_ReturnInt(2);
 			return;
 
