@@ -2,7 +2,7 @@
 
   gbx_c_process.c
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -906,6 +906,11 @@ static void callback_child(int signum, intptr_t data)
 	#endif
 }
 
+void CPROCESS_callback_child(void)
+{
+	callback_child(SIGCHLD, 0);
+}
+
 static void init_child(void)
 {
 	if (!_init)
