@@ -2,7 +2,7 @@
 
   gbx_class.c
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -64,8 +64,6 @@ static TABLE _global_table;
 static CLASS *_classes = NULL;
 // First created class (must be 'Class' class)
 static CLASS *_first = NULL;
-// This flag forces CLASS_find() to register in the global table
-static bool _global = FALSE;
 // If set, do not create a new class, but register that class in the global table
 static CLASS *_global_class = NULL;
 
@@ -121,9 +119,6 @@ static void unload_class(CLASS *class)
 		fprintf(stderr, "Unloading class %s...\n", class->name);
 	#endif
 
-	if (class->free_name)
-		FREE(&class->name);
-
 	if (class->is_struct)
 	{
 		FREE(&class->load->dyn);
@@ -204,7 +199,7 @@ static CLASS *class_keep_global(CLASS *class)
 	int len;
 	CLASS swap;
 	char *swap_name;
-	bool swap_free_name;
+	//bool swap_free_name;
 	int nprefix;
 	int i;
 
@@ -238,7 +233,7 @@ static CLASS *class_keep_global(CLASS *class)
 	*old_class = swap;
 
 	SWAP_FIELD(swap_name, class, old_class, name);
-	SWAP_FIELD(swap_free_name, class, old_class, free_name);
+	//SWAP_FIELD(swap_free_name, class, old_class, free_name);
 	SWAP_FIELD(parent, class, old_class, next);
 	SWAP_FIELD(i, class, old_class, count);
 	SWAP_FIELD(i, class, old_class, ref);
@@ -259,7 +254,7 @@ CLASS *class_replace_global(CLASS *class)
 	char *old_name;
 	CLASS *new_class;
 	char *swap_name;
-	bool swap_free_name;
+	//bool swap_free_name;
 	int index;
 	CLASS_SYMBOL *csym;
 	const char *name = class->name;
@@ -294,7 +289,7 @@ CLASS *class_replace_global(CLASS *class)
 	FREE(&old_name);
 	
 	SWAP_FIELD(swap_name, class, new_class, name);
-	SWAP_FIELD(swap_free_name, class, new_class, free_name);
+	//SWAP_FIELD(swap_free_name, class, new_class, free_name);
 
 	CLASS_inheritance(new_class, class);
 	
@@ -466,18 +461,18 @@ void CLASS_exit()
 }
 
 
-CLASS *CLASS_look(const char *name, int len)
+CLASS *CLASS_look_do(const char *name, int len, bool global)
 {
 	CLASS_SYMBOL *csym;
 	ARCHIVE *arch = NULL;
 	int index;
 
 	#if DEBUG_COMP
-	fprintf(stderr, "CLASS_look: %s in %s", name, _global ? "global" : (_export ? "export" : "local"));
+	fprintf(stderr, "CLASS_look: %s in %s", name, global ? "global" : "local");
 	#endif
 
 	//if (CP && CP->component && CP->component->archive)
-	if (!_global && !ARCHIVE_get_current(&arch))
+	if (!global && !ARCHIVE_get_current(&arch))
 	{
 		if (TABLE_find_symbol(arch->classes, name, len, &index))
 		{
@@ -506,14 +501,13 @@ CLASS *CLASS_look(const char *name, int len)
 	}
 }
 
-CLASS *CLASS_find(const char *name)
+CLASS *CLASS_find_do(const char *name, bool global)
 {
 	CLASS_SYMBOL *csym;
 	CLASS *class;
 	int index;
 	int len;
 	ARCHIVE *arch = NULL;
-	bool global;
 
 	if (name == NULL)
 		name = COMMON_buffer;
@@ -521,7 +515,7 @@ CLASS *CLASS_find(const char *name)
 	len = strlen(name);
 
 #if DEBUG_LOAD
-		fprintf(stderr, "CLASS_find: %s (%d)\n", name, _global);
+		fprintf(stderr, "CLASS_find: %s (%d)\n", name, global);
 #endif
 
 	class = CLASS_look(name, len);
@@ -529,9 +523,8 @@ CLASS *CLASS_find(const char *name)
 		return class;
 
 	//if (CP && CP->component && CP->component->archive)
-	if (!_global && !ARCHIVE_get_current(&arch))
+	if (!global && !ARCHIVE_get_current(&arch))
 	{
-		global = FALSE;
 		index = TABLE_add_symbol(arch->classes, name, len);
 		#if DEBUG_LOAD || DEBUG_COMP
 			fprintf(stderr, "Not found -> creating new one in %s\n", arch->name ? arch->name : "main");
@@ -563,7 +556,8 @@ CLASS *CLASS_find(const char *name)
 	ALLOC(&csym->sym.name, len + 1);
 	strcpy(csym->sym.name, name);
 
-	class->name = csym->sym.name;
+	if (!class->name)
+		class->name = csym->sym.name;
 
 	// The first class must be the Class class!
 	if (_first == NULL)
@@ -578,6 +572,7 @@ CLASS *CLASS_find(const char *name)
 	return class;
 }
 
+
 int CLASS_count(void)
 {
 	return TABLE_count(&_global_table);
@@ -595,29 +590,6 @@ CLASS *CLASS_get(const char *name)
 }
 
 
-CLASS *CLASS_look_global(const char *name, int len)
-{
-	CLASS *class;
-
-	_global = TRUE;
-	class = CLASS_look(name, len);
-	_global = FALSE;
-
-	return class;
-}
-
-
-CLASS *CLASS_find_global(const char *name)
-{
-	CLASS *class;
-
-	_global = TRUE;
-	class = CLASS_find(name);
-	_global = FALSE;
-
-	return class;
-}
-
 CLASS *CLASS_find_export(const char *name, const char *global)
 {
 	CLASS *class;
@@ -626,7 +598,7 @@ CLASS *CLASS_find_export(const char *name, const char *global)
 	{
 		class = CLASS_find(name);
 		_global_class = class;
-	CLASS_find_global(global);
+		CLASS_find_global(global);
 		_global_class = NULL;
 	}
 	else
@@ -1049,10 +1021,10 @@ const char *CLASS_DESC_get_type_name(const CLASS_DESC *desc)
 {
 	switch (desc->gambas.val3._int[1])
 	{
-		case CD_PROPERTY_ID: return desc->gambas.val2 < 0 ? "r" : "p";
+		case CD_PROPERTY_ID: return desc->gambas.val2 < 0 ? "r" : desc->gambas.val1 < 0 ? "w" : "p";
 		case CD_VARIABLE_ID: return "v";
 		case CD_METHOD_ID: return "m";
-		case CD_STATIC_PROPERTY_ID: return desc->gambas.val2 < 0 ? "R" : "P";
+		case CD_STATIC_PROPERTY_ID: return desc->gambas.val2 < 0 ? "R" : desc->gambas.val1 < 0 ? "W" : "P";
 		case CD_STATIC_VARIABLE_ID: return "V";
 		case CD_STATIC_METHOD_ID: return "M";
 		case CD_CONSTANT_ID: return "C";
