@@ -2,7 +2,7 @@
 
 	gbx_class_load.c
 
-	(c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+	(c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -172,6 +172,8 @@ static void check_version(CLASS *class, int loaded)
 		THROW_CLASS(class, "Bytecode too recent. Please upgrade Gambas.", "");
 	if (loaded < GAMBAS_PCODE_VERSION_MIN)
 		THROW_CLASS(class, "Bytecode too old. Please recompile the project.", "");
+
+	class->not_3_18 = loaded < 0x3180000;
 }
 
 
@@ -1154,6 +1156,8 @@ static void load_without_inits(CLASS *class)
 			case CD_STATIC_PROPERTY:
 			case CD_PROPERTY_READ:
 			case CD_STATIC_PROPERTY_READ:
+			case CD_PROPERTY_WRITE:
+			case CD_STATIC_PROPERTY_WRITE:
 
 				desc->property.read = (void (*)())desc->gambas.val1;
 				desc->property.write = (void (*)())desc->gambas.val2;
@@ -1331,8 +1335,6 @@ void CLASS_load_real(CLASS *class)
 	char *name = class->name;
 	int len = strlen(name);
 
-	_load_without_init = FALSE;
-
 	if (!CLASS_is_loaded(class))
 	{
 		if (len >= 3 && name[len - 2] == '[' && name[len - 1] == ']' && !class->array_type)
@@ -1347,7 +1349,10 @@ void CLASS_load_real(CLASS *class)
 	class->ready = FALSE;
 
 	if (load_without_init)
+	{
+		_load_without_init = FALSE;
 		return;
+	}
 
 	class->ready = TRUE;
 	CLASS_run_inits(class);
