@@ -1,14 +1,12 @@
 /***************************************************************************
 
-  mathext.h
+  gbx_math.h
 
-  Mathematical extension routines
-
-  (c) 2000-2007 Benoit Minisini <gambas@users.sourceforge.net>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -18,7 +16,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
 
@@ -27,6 +26,14 @@
 
 #include "config.h"
 
+#ifdef OS_BSD
+#undef HAVE_EXP10
+#endif
+
+#ifndef __GBX_MATH_C
+extern const double MATH_pow10_double[];
+#endif
+
 void MATH_init(void);
 
 int lsgn(int x);
@@ -35,15 +42,21 @@ int llsgn(int64_t x);
 
 double frac(double x);
 int fsgn(double x);
-double deg(double x);
-double rad(double x);
+float fixf(float x);
 double fix(double x);
 double frexp10(double x, int *exp);
-double ang(double x, double y);
+
+#define pow10(_n) (((_n) >= 0 && (_n) <= 9) ? MATH_pow10_double[_n] : (((_n) < 0 && (_n) >= -9) ? (1.0 / MATH_pow10_double[-(_n)]) : exp10(_n)))
+//#define mulpow10(_v, _n) (((_n) >= 0 && (_n) <= 9) ? ((_v) * MATH_pow10_double[_n]) : (((_n) < 0 && (_n) >= -9) ? ((_v) / MATH_pow10[-(_n)]) : ((_v) * exp10(_n))))
+
+uint64_t pow10_uint64_p(int n);
 
 void randomize(bool set, uint seed);
 double rnd(void);
 
+#define deg(_x) ((_x) * 180 / M_PI)
+#define rad(_x) ((_x) * M_PI / 180)
+
 #ifndef HAVE_EXP10
 double exp10(double x);
 #endif
@@ -56,20 +69,4 @@ double log2(double x);
 double exp2(double x);
 #endif
 
-#ifndef HAVE_LOG10L
-long double log10l(long double x);
-#endif
-
-#ifndef HAVE_FABSL
-long double fabsl(long double x);
-#endif
-
-#ifndef HAVE_POWL
-long double powl(long double x, long double y);
-#endif
-
-#ifndef HAVE_MODFL
-long double modfl(long double x, long double *iptr);
-#endif
-
 #endif
