@@ -2,7 +2,7 @@
 
   gbx_stream.c
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -1524,6 +1524,10 @@ void STREAM_read_type(STREAM *stream, TYPE type, VALUE *value)
 				
 			break;
 
+		case T_NULL:
+
+			break;
+
 		default:
 
 			THROW_SERIAL();
@@ -1695,6 +1699,10 @@ void STREAM_write_type(STREAM *stream, TYPE type, VALUE *value)
 				}
 				break;
 
+			case T_NULL:
+
+				break;
+
 			default:
 
 				THROW_SERIAL();
@@ -1883,7 +1891,7 @@ void STREAM_write_type(STREAM *stream, TYPE type, VALUE *value)
 				addr = (char *)structure + sizeof(CSTRUCT) + desc->variable.offset;
 
 			VALUE_class_read(desc->variable.class, &temp, (void *)addr, desc->variable.ctype, (void *)structure);
-			BORROW(&temp);
+			//BORROW(&temp);
 			STREAM_write_type(stream, temp.type, &temp);
 			RELEASE(&temp);
 		}
