@@ -2,7 +2,7 @@
 
   gbx_subr_extern.c
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -178,9 +178,10 @@ void SUBR_varptr(ushort code)
 	}
 	else
 	{
-		if ((op & 0xFF00) == C_PUSH_LOCAL || (op & 0xFF00) == C_PUSH_PARAM)
+		if ((op & 0xFF00) == C_PUSH_LOCAL || (op & 0xFF00) == C_PUSH_PARAM
+			  || (op & 0xFF00) == C_PUSH_LOCAL_NOREF || (op & 0xFF00) == C_PUSH_PARAM_NOREF)
 		{
-			if ((op & 0xFF00) == C_PUSH_PARAM)
+			if ((op & 0xFF00) == C_PUSH_PARAM || (op & 0xFF00) == C_PUSH_PARAM_NOREF)
 				val = &PP[(signed char)(op & 0xFF)];
 			else
 				val = &BP[op & 0xFF];
@@ -210,17 +211,16 @@ void SUBR_varptr(ushort code)
 					ptr = &val->_date.date;
 					break;
 
-				case T_STRING:
-				case T_CSTRING:
-					ptr = val->_string.addr + val->_string.start;
-					break;
-
 				case T_POINTER:
 					ptr = &val->_pointer.value;
 					break;
+					
+				case T_VARIANT:
+					ptr = &val->_variant.value.data;
+					break;
 
 				default:
-					THROW(E_TYPE, "Number", TYPE_get_name(val->type));
+					THROW(E_UTYPE);
 			}
 		}
 		else if ((op & 0xF800) == C_PUSH_DYNAMIC)
@@ -231,11 +231,15 @@ void SUBR_varptr(ushort code)
 				THROW_ILLEGAL();
 
 			ptr = &OP[var->pos];
+			if (var->type.id == T_VARIANT)
+				ptr = &((VARIANT *)ptr)->value.data;
 		}
 		else if ((op & 0xF800) == C_PUSH_STATIC)
 		{
 			var = &CP->load->stat[op & 0x7FF];
 			ptr = (char *)CP->stat + var->pos;
+			if (var->type.id == T_VARIANT)
+				ptr = &((VARIANT *)ptr)->value.data;
 		}
 		else
 			THROW_ILLEGAL();
@@ -248,25 +252,46 @@ void SUBR_varptr(ushort code)
 }
 
 
-void SUBR_ptr(ushort code)
+void SUBR_peek(ushort code)
 {
   void *ptr;
-  
+
 	SUBR_ENTER_PARAM(1);
-	
+
   ptr = SUBR_get_pointer_or_string(PARAM);
-	
+
 	CHECK_enter();
-  if (sigsetjmp(CHECK_jump, TRUE) == 0)	
+  if (sigsetjmp(CHECK_jump, TRUE) == 0)
 		VALUE_read(RETURN, ptr, code & 0xF);
 	CHECK_leave();
-	
+
 	if (CHECK_got_error())
 		THROW(E_ARG);
 
 	SUBR_LEAVE();
 }
 
+
+void SUBR_poke(ushort code)
+{
+  void *ptr;
+
+	SUBR_ENTER_PARAM(2);
+
+  ptr = SUBR_get_pointer_or_string(PARAM);
+
+	CHECK_enter();
+  if (sigsetjmp(CHECK_jump, TRUE) == 0)
+		VALUE_write(&PARAM[1], ptr, code & 0xF);
+	CHECK_leave();
+
+	if (CHECK_got_error())
+		THROW(E_ARG);
+
+	SUBR_LEAVE_VOID();
+}
+
+
 void SUBR_make(ushort code)
 {
   static void *jump[] = {
