@@ -2,7 +2,7 @@
 
   gbx_subr_misc.c
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@
 static EVAL_INTERFACE EVAL;
 static CCOLLECTION *eval_env;
 
-static void init_eval()
+static void init_eval(void)
 {
 	static bool init = FALSE;
 	
@@ -72,18 +72,18 @@ void SUBR_wait(ushort code)
 	EXEC_set_native_error(FALSE);
 	
 	if (code == 0)
-		GB_Wait(0);
+		GB_Wait(-1);
 	else if (code == 1)
 	{
 		int delay = (int)(SUBR_get_float(PARAM) * 1000 + 0.5);
-		if (delay < 0)
+		if (delay <= 0)
 			delay = 0;
 		GB_Wait(delay);
 	}
 	else if (code == 2)
 	{
 		NPARAM = 0;
-		GB_Wait(-1);
+		GB_Wait(-2);
 	}
 
 	if (EXEC_has_native_error())
