@@ -2,7 +2,7 @@
 
   gbx_subr_string.c
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -39,8 +39,6 @@
 #include "gbx_local.h"
 #include "gbx_compare.h"
 
-//static int _count = 0;
-
 //---------------------------------------------------------------------------
 
 void SUBR_cat(ushort code)
@@ -935,7 +933,10 @@ static void make_hex_char(uchar c)
 
 void SUBR_quote(ushort code)
 {
-	static void *jump[8] = { &&__QUOTE, &&__SHELL, &&__HTML, &&__BASE64, &&__URL , &&__ILLEGAL, &&__ILLEGAL, &&__ILLEGAL };
+	static void *jump[8] = {
+		&&__QUOTE, &&__SHELL, &&__HTML, &&__BASE64, &&__URL , &&__ILLEGAL, &&__ILLEGAL, &&__ILLEGAL
+	};
+
 	char *str;
 	int lstr;
 	int i;
@@ -949,9 +950,11 @@ void SUBR_quote(ushort code)
 	str = PARAM->_string.addr + PARAM->_string.start;
 	lstr = PARAM->_string.len;
 
+	code &= 0x7;
+
 	STRING_start_len(lstr);
 
-	goto *jump[code & 0x7];
+	goto *jump[code];
 
 __QUOTE:
 
@@ -1107,42 +1110,6 @@ __URL:
 
 	goto __END;
 
-#if 0
-__JAVASCRIPT:
-	{
-		STRING_make_char('\'');
-
-		for (i = 0; i < lstr; i++)
-		{
-			c = str[i];
-			if (c >= ' ' && c <= 126 && c != '\\' && c != '\'')
-				STRING_make_char(c);
-			else
-			{
-				STRING_make_char('\\');
-				if (c == '\n')
-					c = 'n';
-				else if (c == '\r')
-					c = 'r';
-				else if (c == '\t')
-					c = 't';
-				else if (!(c == '\'' || c == '\\'))
-				{
-					snprintf(buf, sizeof(buf), "x%02X", c);
-					STRING_make(buf, 3);
-					continue;
-				}
-				STRING_make_char(c);
-			}
-		}
-
-		STRING_make_char('\'');
-	}
-
-	goto __END;
-
-#endif
-
 __ILLEGAL:
 
 	THROW_ILLEGAL();
