@@ -2,7 +2,7 @@
 
   gbx_value.c
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 
 #include "gb_common.h"
 #include "gb_common_case.h"
+#include "gb_overflow.h"
 
 #include "gbx_math.h"
 #include "gbx_type.h"
@@ -419,20 +420,52 @@ __p2b:
 	value->type = T_BOOLEAN;
 	return;
 
+#if DO_NOT_CHECK_OVERFLOW
+
 __b2c:
 __h2c:
 __i2c:
-
-	value->_integer.value = (unsigned char)value->_integer.value;
+	value->_integer.value = (uchar)value->_integer.value;
 	value->type = T_BYTE;
 	return;
 
 __l2c:
-
-	value->_integer.value = (unsigned char)value->_long.value;
+	value->_integer.value = (uchar)value->_long.value;
 	value->type = T_BYTE;
 	return;
 
+#else
+
+__b2c:
+	value->_integer.value = (uchar)value->_integer.value;
+	value->type = T_BYTE;
+	return;
+
+__h2c:
+__i2c:
+	{
+		uchar result;
+
+		if (__builtin_add_overflow(value->_integer.value, 0, &result))
+			THROW_OVERFLOW();
+		value->_integer.value = result;
+		value->type = T_BYTE;
+		return;
+	}
+
+__l2c:
+	{
+		uchar result;
+
+		if (__builtin_add_overflow(value->_long.value, 0, &result))
+			THROW_OVERFLOW();
+		value->_integer.value = result;
+		value->type = T_BYTE;
+		return;
+	}
+
+#endif
+
 __g2c:
 
 	value->_integer.value = (unsigned char)value->_single.value;
@@ -445,20 +478,50 @@ __f2c:
 	value->type = T_BYTE;
 	return;
 
+#if DO_NOT_CHECK_OVERFLOW
+
 __b2h:
 __c2h:
 __i2h:
-
-	value->_integer.value = (short)value->_integer.value;
 	value->type = T_SHORT;
 	return;
 
 __l2h:
-
 	value->_integer.value = (short)value->_long.value;
 	value->type = T_SHORT;
 	return;
 
+#else
+
+__b2h:
+__c2h:
+	value->type = T_SHORT;
+	return;
+
+__i2h:
+	{
+		short result;
+
+		if (__builtin_add_overflow(value->_integer.value, 0, &result))
+			THROW_OVERFLOW();
+		value->_integer.value = result;
+		value->type = T_SHORT;
+		return;
+	}
+
+__l2h:
+	{
+		short result;
+
+		if (__builtin_add_overflow(value->_long.value, 0, &result))
+			THROW_OVERFLOW();
+		value->_integer.value = result;
+		value->type = T_SHORT;
+		return;
+	}
+
+#endif
+
 __g2h:
 
 	value->_integer.value = (short)value->_single.value;
@@ -471,12 +534,23 @@ __f2h:
 	value->type = T_SHORT;
 	return;
 
-__l2i:
+#if DO_NOT_CHECK_OVERFLOW
 
+__l2i:
 	value->_integer.value = (int)value->_long.value;
 	value->type = T_INTEGER;
 	return;
 
+#else
+
+__l2i:
+	if (__builtin_add_overflow(value->_long.value, 0, &value->_integer.value))
+		THROW_OVERFLOW();
+	value->type = T_INTEGER;
+	return;
+
+#endif
+
 __g2i:
 
 	value->_integer.value = (int)value->_single.value;
@@ -1511,7 +1585,7 @@ void VALUE_from_local_string(VALUE *value, const char *addr, int len)
 
 void VALUE_class_read(CLASS *class, VALUE *value, char *addr, CTYPE ctype, void *ref)
 {
-	VALUE_class_read_inline(class, value, addr, ctype, ref);
+	VALUE_class_read_inline(class, value, addr, ctype, ref, VCR);
 }
 
 void VALUE_class_write(CLASS *class, VALUE *value, char *addr, CTYPE ctype)
@@ -1552,7 +1626,7 @@ void VALUE_class_write(CLASS *class, VALUE *value, char *addr, CTYPE ctype)
 				saddr = (char *)src + sizeof(CSTRUCT);
 		
 			VALUE_class_read(desc->variable.class, &temp, &saddr[offset], ctype, src);
-			BORROW(&temp);
+			//BORROW(&temp);
 			VALUE_class_write(sclass, &temp, &addr[offset], ctype);
 			RELEASE(&temp);
 		}	
@@ -1581,7 +1655,7 @@ void VALUE_class_write(CLASS *class, VALUE *value, char *addr, CTYPE ctype)
 		for (i = 0; i < count; i++)
 		{
 			VALUE_class_read(class, &temp, saddr, ctype, src);
-			BORROW(&temp);
+			//BORROW(&temp);
 			VALUE_class_write(class, &temp, addr, ctype);
 			RELEASE(&temp);
 			saddr += src->size;
@@ -1805,7 +1879,12 @@ __CONV:
 
 __l2i:
 
+#if DO_NOT_CHECK_OVERFLOW
 	value->_integer.value = (int)value->_long.value;
+#else
+	if (__builtin_add_overflow(value->_long.value, 0, &value->_integer.value))
+		THROW_OVERFLOW();
+#endif
 	goto __TYPE;
 
 __g2i:
@@ -2108,136 +2187,6 @@ __NR:
 	THROW(E_NRETURN);
 }
 
-#if 0
-void VALUE_convert_object(VALUE *value, TYPE type)
-{
-	CLASS *class;
-
-__CONV:
-
-	#if 0
-	if (!TYPE_is_object(type))
-	{
-		if (type == T_BOOLEAN)
-		{
-			test = (value->_object.object != NULL);
-			OBJECT_UNREF(value->_object.object);
-			value->_boolean.value = test ? -1 : 0;
-			goto __TYPE;
-		}
-
-		if (type == T_VARIANT)
-			goto __2v;
-
-		goto __N;
-	}
-	#endif
-
-	if (!TYPE_is_object(value->type))
-	{
-		if (value->type == T_NULL)
-		{
-			OBJECT_null(value, (CLASS *)type); /* marche aussi pour type = T_OBJECT */
-			goto __TYPE;
-		}
-
-		if (value->type == T_VARIANT)
-			goto __v2;
-
-		if (value->type == T_FUNCTION)
-			goto __func;
-
-		if (value->type == T_CLASS)
-		{
-			class = value->_class.class;
-
-			if (CLASS_is_virtual(class))
-				THROW(E_VIRTUAL);
-
-			CLASS_load(class);
-
-			if (class->auto_create)
-				value->_object.object = CLASS_auto_create(class, 0);
-			else
-				value->_object.object = class;
-
-			OBJECT_REF(value->_object.object);
-			value->type = T_OBJECT;
-			/* on continue... */
-		}
-		else
-			goto __N;
-
-	}
-
-	if (value->_object.object == NULL)
-		goto __TYPE;
-
-	if (value->type == T_OBJECT)
-	{
-		/*if (value->_object.object == NULL)
-			goto __TYPE;*/
-
-		class = OBJECT_class(value->_object.object);
-		/* on continue */
-	}
-	else
-		class = value->_object.class;
-
-	if (CLASS_is_virtual(class))
-		THROW(E_VIRTUAL);
-
-	if (type == T_OBJECT)
-		goto __TYPE;
-
-__RETRY:
-
-	if ((class == (CLASS *)type) || CLASS_inherits(class, (CLASS *)type))
-		goto __TYPE;
-
-	if (value->type != T_OBJECT && value->_object.object)
-	{
-		class = OBJECT_class(value->_object.object);
-		value->type = T_OBJECT;
-		goto __RETRY;
-	}
-
-	if (class->special[SPEC_CONVERT] != NO_SYMBOL)
-	{
-		void *conv = ((void *(*)())(CLASS_get_desc(class, class->special[SPEC_CONVERT])->constant.value._pointer))(value->_object.object, type);
-		if (conv)
-		{
-			OBJECT_REF(conv);
-			OBJECT_UNREF(value->_object.object);
-			value->_object.object = conv;
-			goto __TYPE;
-		}
-	}
-
-	THROW(E_TYPE, TYPE_get_name(type), TYPE_get_name((TYPE)class));
-
-__v2:
-
-	undo_variant(value);
-	goto __CONV;
-
-__func:
-
-	//if (unknown_function(value))
-	//	goto __CONV;
-	//else
-		goto __N;
-
-__TYPE:
-
-	value->type = type;
-	return;
-
-__N:
-
-	THROW(E_TYPE, TYPE_get_name(type), TYPE_get_name(value->type));
-}
-#endif
 
 void VALUE_undo_variant(VALUE *value)
 {
