@@ -2,13 +2,11 @@
 
   main.c
 
-  Compression Library
-
   (c) 2003-2004 Daniel Campos Fernández <danielcampos@netcourrier.com>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 1, or (at your option)
+  the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
@@ -18,9 +16,11 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+  MA 02110-1301, USA.
 
 ***************************************************************************/
+
 #define MAX_DRIVER 8
 #define __MAIN_C
 
@@ -35,11 +35,6 @@
 #include "main.h"
 
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
-
 
 GB_INTERFACE GB EXPORT;
 
@@ -60,19 +55,20 @@ static void COMPRESS_Register(COMPRESS_DRIVER *driver)
 COMPRESS_DRIVER *COMPRESS_GetDriver(char *type)
 {
   int i;
-  int comp_size = strlen(type) + 14;
-  char comp[comp_size];
+	char *comp;
 
-  if (!type)
+  if (!type || !*type)
   {
     GB.Error("Driver name missing");
     return NULL;
   }
 
-  strcpy(comp, "gb.compress.");
+  comp = alloca(strlen(type) + 14);
+
+	strcpy(comp, "gb.compress.");
   strcat(comp, type);
 
-  if (GB.LoadComponent(comp))
+  if (GB.Component.Load(comp))
   {
     GB.Error("Cannot find driver for : &1", type);
     return NULL;
@@ -96,8 +92,6 @@ void *GB_COMPRESS_1[] EXPORT = {
   NULL
 };
 
-
-
 GB_DESC *GB_CLASSES[] EXPORT =
 {
   CCompressDesc,
@@ -105,22 +99,13 @@ GB_DESC *GB_CLASSES[] EXPORT =
   NULL
 };
 
-
-
 int EXPORT GB_INIT(void)
 {
   return 0;
 }
 
-
-
 void EXPORT GB_EXIT()
 {
 
 }
 
-
-#ifdef _cpluscplus
-}
-#endif
-
