@@ -2,7 +2,7 @@
 
 	CConnection.c
 
-	(c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+	(c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -79,7 +79,11 @@ static SUBCOLLECTION_DESC _tables_desc =
 
 static void open_connection(CCONNECTION *_object)
 {
-	if (DB_Open(&THIS->desc, &THIS->driver, &THIS->db))
+	GB_COLLECTION options;
+	
+	options = GB.GetProperty(THIS, "Options")->_object.value;
+	
+	if (DB_Open(&THIS->desc, &THIS->driver, &THIS->db, options))
 		return;
 
 	THIS->limit = 0;
@@ -147,9 +151,7 @@ static void close_connection(CCONNECTION *_object)
 }
 
 
-BEGIN_METHOD(Connection_new, GB_STRING url)
-
-	char *url, *name, *p;
+BEGIN_METHOD_VOID(Connection_new)
 
 	THIS->db.handle = NULL;
 	THIS->db.ignore_case = FALSE; // Now case is sensitive by default!
@@ -159,51 +161,6 @@ BEGIN_METHOD(Connection_new, GB_STRING url)
 	if (_current == NULL)
 		_current = THIS;
 
-	if (MISSING(url))
-		return;
-
-	url = GB.ToZeroString(ARG(url));
-
-	p = index(url, ':');
-	if (!p || p == url) goto __BAD_URL;
-	*p++ = 0;
-	if (p[0] != '/' || p[1] != '/') goto __BAD_URL;
-	p += 2;
-
-	THIS->desc.type = GB.NewZeroString(url);
-	url = p;
-
-	p = rindex(url, '/');
-	if (!p || p == url) goto __BAD_URL;
-	*p++ = 0;
-
-	name = p;
-
-	p = index(url, '@');
-	if (p)
-	{
-		if (p == url)
-			goto __BAD_URL;
-		*p = 0;
-		THIS->desc.user = GB.NewZeroString(url);
-		url = p + 1;
-	}
-
-	p = index(url, ':');
-	if (p)
-	{
-		*p = 0;
-		THIS->desc.port = GB.NewZeroString(p + 1);
-	}
-
-	THIS->desc.host = GB.NewZeroString(url);
-	THIS->desc.name = GB.NewZeroString(name);
-	return;
-
-__BAD_URL:
-
-	GB.Error("Malformed URL");
-
 END_METHOD
 
 
@@ -458,7 +415,7 @@ static char *make_query(CCONNECTION *_object, char *pattern, int len, int narg,
 	const char *keyword;
 	char buffer[32];
 
-	query = DB_MakeQuery(THIS->driver, pattern, len, narg, arg);
+	query = DB_MakeQuery(THIS->driver, &THIS->db, pattern, len, narg, arg);
 
 	if (query && THIS->limit > 0 && strncasecmp(query, "SELECT ", 7) == 0)
 	{
@@ -788,7 +745,7 @@ GB_DESC CConnectionDesc[] =
 {
 	GB_DECLARE("_Connection", sizeof(CCONNECTION)),
 
-	GB_METHOD("_new", NULL, Connection_new, "[(DatabaseURL)s]"),
+	GB_METHOD("_new", NULL, Connection_new, NULL),
 	GB_METHOD("_free", NULL, Connection_free, NULL),
 
 	GB_PROPERTY("Type", "s", Connection_Type),
