@@ -2,7 +2,7 @@
 
 	debug.c
 
-	(c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+	(c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -441,6 +441,22 @@ static bool unset_breakpoint(CLASS *class, ushort line)
 }
 
 
+static void unset_all_breakpoints()
+{
+	int i;
+	DEBUG_BREAK *brk;
+
+	for (i = 0; i < GB.Count(_breakpoints); i++)
+	{
+		brk = &_breakpoints[i];
+		if (brk->addr)
+			*(brk->addr) = PCODE_BREAKPOINT(0);
+	}
+
+	GB.Remove(&_breakpoints, 0, GB.Count(_breakpoints));
+}
+
+
 void DEBUG_init_breakpoints(CLASS *class)
 {
 	int i;
@@ -610,37 +626,42 @@ static void command_from(char *cmd)
 }
 
 
-static void command_set_breakpoint(char *cmd)
+static void command_breakpoint(char *cmd)
 {
-	char class_name[64];
+	char *comp = NULL;
+	char class_name[256];
 	ushort line;
-	//CLASS *class;
+	bool unset = *cmd == '-';
+	char *p;
 
-	if (sscanf(cmd, "+%64[^.].%hu", class_name, &line) != 2)
-		WARNING("Cannot set breakpoint: syntax error");
-	else
+	cmd++;
+
+	if (unset && cmd[0] == '*' && cmd[1] == 0)
 	{
-		//class = (CLASS *)GB.FindClassLocal(class_name);
-		//CLASS_load_without_init(class);
-		//fprintf(stderr, "command_set_breakpoint: %s %s\n", class->name, class->component ? class->component->name : "?");
-		set_breakpoint((CLASS *)GB_DEBUG.FindClass(class_name), line);
+		unset_all_breakpoints();
+		return;
 	}
-}
 
-
-static void command_unset_breakpoint(char *cmd)
-{
-	char class_name[64];
-	ushort line;
-
-	if (sscanf(cmd, "-%64[^.].%hu", class_name, &line) != 2)
-		WARNING("Cannot remove breakpoint: Syntax error");
-	else
+	if (*cmd == '[')
 	{
-		//class = CLASS_find(class_name);
-		//CLASS_load_without_init(class);
-		unset_breakpoint((CLASS *)GB_DEBUG.FindClass(class_name), line);
+		p = index(cmd, ']');
+		if (p && p[1] == '.')
+		{
+			comp = &cmd[1];
+			*p = 0;
+			cmd = p + 2;
+
+			if (comp[0] == '$' && !comp[1])
+				comp = NULL;
+		}
 	}
+
+	if (sscanf(cmd, "%256[^.].%hu", class_name, &line) != 2)
+		WARNING("Cannot %s breakpoint: syntax error", unset ? "remove" : "add");
+	else if (unset)
+		unset_breakpoint((CLASS *)GB_DEBUG.FindClass(comp, class_name), line);
+	else
+		set_breakpoint((CLASS *)GB_DEBUG.FindClass(comp, class_name), line);
 }
 
 
@@ -648,12 +669,8 @@ void DEBUG_backtrace(FILE *out)
 {
 	int i, n;
 	STACK_CONTEXT *context;
-	ushort line;
 
-	if (CP)
-		fprintf(out, "%s", DEBUG_get_current_position());
-	else
-		fprintf(out, "?");
+	fprintf(out, "%s", DEBUG_get_current_position());
 
 	//for (i = 0; i < (STACK_frame_count - 1); i++)
 	n = 0;
@@ -663,7 +680,9 @@ void DEBUG_backtrace(FILE *out)
 		if (!context)
 			break;
 
-		if (context->pc)
+		n += fprintf(out, " %s", DEBUG_get_position(context->cp, context->fp, context->pc));
+
+		/*if (context->pc)
 		{
 			line = 0;
 			if (DEBUG_calc_line_from_position(context->cp, context->fp, context->pc, &line))
@@ -672,7 +691,7 @@ void DEBUG_backtrace(FILE *out)
 				n += fprintf(out, " %s.%s.%d", context->cp->name, context->fp->debug->name, line);
 		}
 		else if (context->cp)
-			n += fprintf(out, " ?");
+			n += fprintf(out, " ?");*/
 		
 		if (n >= (DEBUG_OUTPUT_MAX_SIZE / 2))
 		{
@@ -683,14 +702,14 @@ void DEBUG_backtrace(FILE *out)
 }
 
 
-static void debug_info()
+static void debug_info(bool frame)
 {
 	const char *p;
 	char c;
 	int i;
 	DEBUG_WATCH *watch;
 	
-	fprintf(_out, "*[%d]\t", getpid());
+	fprintf(_out, "%c[%d]\t", frame ? '@' : '*', getpid());
 	
 	if (_error)
 	{
@@ -779,7 +798,7 @@ static void command_frame(char *cmd)
 	}
 
 	set_info(context);
-	debug_info();
+	debug_info(cmd != NULL);
 }
 
 
@@ -1063,25 +1082,34 @@ void DEBUG_breakpoint(int id)
 
 const char *DEBUG_get_position(CLASS *cp, FUNCTION *fp, PCODE *pc)
 {
-	if (pc)
+	const char *comp_name;
+	const char *class_name;
+	const char *func_name;
+	ushort line = 0;
+
+	if (!cp)
+		return "?";
+
+	class_name = cp->name;
+
+	while (*class_name == '^')
+		class_name++;
+
+	if (cp->component)
+		comp_name = cp->component->name;
+	else
+		comp_name = "$";
+
+	if (fp && fp->debug)
 	{
-		ushort line = 0;
-
-		if (fp != NULL && fp->debug)
+		func_name = fp->debug->name;
+		if (pc)
 			DEBUG_calc_line_from_position(cp, fp, pc, &line);
-
-		snprintf(DEBUG_buffer, sizeof(DEBUG_buffer), "%.64s.%.64s.%d",
-			cp ? cp->name : "?",
-			(fp && fp->debug) ? fp->debug->name : "?",
-			line);
 	}
 	else
-	{
-		snprintf(DEBUG_buffer, sizeof(DEBUG_buffer), "%.64s.%.64s",
-			cp ? cp->name : "?",
-			(fp && fp->debug) ? fp->debug->name : "?");
-	}
+		func_name = "?";
 
+	snprintf(DEBUG_buffer, sizeof(DEBUG_buffer), "[%s].%s.%s.%d", comp_name, class_name, func_name, line);
 	return DEBUG_buffer;
 }
 
@@ -1337,8 +1365,8 @@ void DEBUG_main(bool error)
 		{ "s", TC_STEP, command_step, FALSE },
 		{ "f", TC_FROM, command_from, FALSE },
 		{ "g", TC_GO, command_go, FALSE },
-		{ "+", TC_NONE, command_set_breakpoint, TRUE },
-		{ "-", TC_NONE, command_unset_breakpoint, TRUE },
+		{ "+", TC_NONE, command_breakpoint, TRUE },
+		{ "-", TC_NONE, command_breakpoint, TRUE },
 		{ "&", TC_NONE, command_symbol, TRUE },
 		{ "?", TC_NONE, command_eval, TRUE },
 		{ "!", TC_NONE, command_eval, TRUE },
@@ -1354,9 +1382,10 @@ void DEBUG_main(bool error)
 
 	static bool first = TRUE;
 	char *cmd = NULL;
-	char cmdbuf[64];
+	char cmdbuf[256];
 	int len;
 	DEBUG_COMMAND *tc = NULL;
+	int save_errno = errno;
 	/*static int cpt = 0;*/
 
 	GB.FreeString(&_error);
@@ -1466,6 +1495,8 @@ void DEBUG_main(bool error)
 
 	GB.FreeString(&cmd);
 
+	errno = save_errno;
+
 	#ifdef DEBUG_ME
 	fprintf(stderr, "} DEBUG_main\n");
 	#endif
