@@ -2,7 +2,7 @@
 
 	print.c
 
-	(c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+	(c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -111,9 +111,9 @@ static void print_object(void *object)
 	CLASS *class = OBJECT_class(object);
 	
 	if (*class->name == '$')
-		fprintf(_where, "(Struct %s %p) [%ld]", &class->name[1], object, OBJECT_count(object));
+		fprintf(_where, "(Struct %s %p) [%ld]", &class->name[1], object, (long)OBJECT_count(object));
 	else
-		fprintf(_where, "(%s %p) [%ld]", class->name, object, OBJECT_count(object));
+		fprintf(_where, "(%s %p) [%ld]", class->name, object, (long)OBJECT_count(object));
 
 /*
 	if (GB.Is(object, GB.FindClass("Collection")))
