@@ -2,7 +2,7 @@
 
 	eval_analyze.c
 
-	(c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+	(c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -98,7 +98,7 @@ static int get_type(PATTERN *pattern)
 	return type;
 }
 
-static bool is_me_last_kind(PATTERN pattern)
+static bool is_me_last_error_kind(PATTERN pattern)
 {
 	return PATTERN_is(pattern, RS_ME)
 		|| PATTERN_is(pattern, RS_SUPER)
@@ -107,6 +107,7 @@ static bool is_me_last_kind(PATTERN pattern)
 		|| PATTERN_is(pattern, RS_FALSE)
 		|| PATTERN_is(pattern, RS_PINF)
 		|| PATTERN_is(pattern, RS_MINF)
+		|| PATTERN_is(pattern, RS_ERROR)
 		|| PATTERN_is(pattern, RS_NULL);
 }
 
@@ -400,15 +401,14 @@ static void analyze(EVAL_ANALYZE *result)
 		switch(type)
 		{
 			case RT_RESERVED:
-				//state = Keyword;
-				//if (old_type != RT_OPERATOR)
-				//me = is_me_last(*pattern);
 				
-				if (is_me_last_kind(*pattern))
+				if (is_me_last_error_kind(*pattern))
 				{
 					if (old_type != RT_OPERATOR)
 						space_before = TRUE;
 					next_type = RT_IDENTIFIER;
+					if (PATTERN_is(*pattern, RS_ERROR) && old_type == RT_END)
+						space_after = TRUE;
 				}
 				else if (is_optional_kind(*pattern))
 				{
