@@ -2,7 +2,7 @@
 
 	eval_trans_expr.c
 
-	(c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+	(c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -83,6 +83,10 @@ static void push_number(int index)
 	{
 		CODE_push_number(number.ival);
 	}
+	else if (number.type == T_FLOAT && number.dval == (double)(int)number.dval && number.dval >= -128 && number.dval <= 127)
+	{
+		CODE_push_float(number.dval);
+	}
 	else
 	{
 		cst.type = number.type;
@@ -483,7 +487,7 @@ void TRANS_reference(void)
 {
 	TRANS_expression();
 
-	if (!CODE_popify_last())
+	if (!CODE_popify_last(FALSE))
 		THROW("Invalid assignment");
 
 	EVAL->assign_code = EVAL->code[EVAL->ncode - 1];
