@@ -2,7 +2,7 @@
 
   c_color.c
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -649,7 +649,7 @@ GB_DESC CColorDesc[] =
 {
   GB_DECLARE_STATIC("Color"),
 
-  GB_CONSTANT("Default", "i", COLOR_DEFAULT),
+  GB_CONSTANT("Default", "i", GB_COLOR_DEFAULT),
 
   GB_CONSTANT("Black", "i", 0x000000),
   GB_CONSTANT("White", "i", 0xFFFFFF),
