@@ -2,7 +2,7 @@
 
 	jit_body.c
 
-	(c) 2000-2018 Benoît Minisini <g4mba5@gmail.com>
+	(c) 2000-2018 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -163,16 +163,8 @@ static void enter_function(FUNCTION *func, int index)
 static void print_catch(void)
 {
 	JIT_print("\n  } CATCH {\n\n");
-	JIT_print("  CP = (void *)%p;\n", JIT_class);
-	JIT_print("  FP = (void *)%p;\n", _func);
-	if (_has_catch || _has_finally)
-		JIT_print("  JIT.error_set_last(FALSE); \n");
-	//JIT_print("  fprintf(stderr, \"EP = %%p SP = %%p sp = %%p\\n\", EP, SP, sp);\n");
-	JIT_print("  if (SP > sp) sp = SP; else SP = sp;\n");
-	JIT_print("  LEAVE_SUPER();\n");
-	//JIT_print("  if (sp > bp) { fprintf(stderr, \"sp = %%p bp = %%p release %%d\\n\", sp, bp, sp - bp); JIT.release_many(sp, sp - bp); SP = sp = bp; }\n");
-	JIT_print("  if (sp > ssp) { JIT.release_many(sp, sp - ssp); SP = sp = ssp; }\n");
-	//JIT_print("  PP = SP;\n");
+
+	JIT_print("  sp = _jit_print_catch(psp, sp, ssp, (void *)%p, (void *)%p, %d);\n", JIT_class, _func, _has_catch || _has_finally);
 	JIT_print("  error = TRUE;\n");
 	JIT_print("\n  } END_TRY\n\n");
 	JIT_print("__FINALLY:;\n");
@@ -658,7 +650,7 @@ static char *peek_pop(int n, TYPE conv, const char *fmt, va_list args)
 		{
 			JIT_print("  ");
 			JIT_print(dest, expr);
-			JIT_print(";\n");
+			JIT_print("; ");
 		}
 		else
 		{
@@ -1541,7 +1533,9 @@ static void push_subr_add(ushort code, const char *op, const char *opb, bool all
 	char *expr;
 	char *expr1, *expr2;
 	TYPE type1, type2, type;
-	
+	const char *unsafe;
+	const char *func;
+
 	check_stack(2);
 	
 	type1 = get_type(-2);
@@ -1576,8 +1570,21 @@ static void push_subr_add(ushort code, const char *op, const char *opb, bool all
 	if (type == T_BOOLEAN)
 		op = opb;
 	
-	expr = STR_print("({%s _a = %s; %s _b = %s; _a %s _b;})", JIT_get_ctype(type), expr1, JIT_get_ctype(type), expr2, op);
-	
+	unsafe = (_unsafe || type > T_LONG) ? "_UNSAFE" : "";
+
+	switch (*op)
+	{
+		case '+': func = "_ADD"; break;
+		case '-': func = "_SUB"; break;
+		case '*': func = "_MUL"; break;
+		default: func = NULL;
+	}
+
+	if (func)
+		expr = STR_print("MATH%s%s(%s, %s, %s)", func, unsafe, JIT_get_ctype(type), expr1, expr2);
+	else
+		expr = STR_print("({%s _a = %s; %s _b = %s; _a %s _b;})", JIT_get_ctype(type), expr1, JIT_get_ctype(type), expr2, op);
+
 	pop_stack(2);
 	
 	push(type, "(%s)", expr);
@@ -2108,6 +2115,9 @@ static void push_subr_left_right(ushort code, const char *func)
 	}
 	
 	type = get_type(-1);
+	if (type == T_VARIANT || type == T_UNKNOWN)
+		type = T_STRING;
+
 	expr_str = STR_copy(peek(-1, T_STRING));
 	pop_stack(1);
 	
@@ -2137,6 +2147,9 @@ static void push_subr_mid(ushort code)
 	pop_stack(1);
 	
 	type = get_type(-1);
+	if (type == T_VARIANT || type == T_UNKNOWN)
+		type = T_STRING;
+
 	expr_str = STR_copy(peek(-1, T_STRING));
 	pop_stack(1);
 	
@@ -2337,12 +2350,22 @@ static void push_call(ushort code)
 			}
 			else
 			{
+				TYPE type;
+				char *expr;
+
 				STR_add(&call,"SP = sp;(*(%s (*)())%p)(", JIT_get_ctype(ext->type), JIT.get_extern(ext));
 				
 				for (i = 0; i < ext->n_param; i++)
 				{
 					if (i) STR_add(&call, ",");
-					STR_add(&call, "%s", peek(i - narg, ext->param[i].type));
+					type = ext->param[i].type;
+					expr = peek(i - narg, type);
+					if (type == T_STRING || type == T_CSTRING)
+						STR_add(&call, "GET_STRING_ADDR(%s)", expr);
+					else if (type >= T_OBJECT)
+						STR_add(&call, "JIT.get_object_addr((%s)._object.value)", expr);
+					else
+						STR_add(&call, "%s", expr);
 				}
 				
 				STR_add(&call, ");");
@@ -2511,9 +2534,10 @@ static void push_subr_varptr(ushort code)
 		return;
 	}
 
-	if ((op & 0xFF00) == C_PUSH_LOCAL || (op & 0xFF00) == C_PUSH_PARAM)
+	if ((op & 0xFF00) == C_PUSH_LOCAL || (op & 0xFF00) == C_PUSH_PARAM
+		  || (op & 0xFF00) == C_PUSH_LOCAL_NOREF || (op & 0xFF00) == C_PUSH_PARAM_NOREF)
 	{
-		if ((op & 0xFF00) == C_PUSH_PARAM)
+		if ((op & 0xFF00) == C_PUSH_PARAM || (op & 0xFF00) == C_PUSH_PARAM_NOREF)
 		{
 			index = _func->n_param + (signed char)(op & 0xFF);
 			type = _func->param[index].type;
@@ -2580,13 +2604,13 @@ _ILLEGAL:
 }
 
 
-static void push_subr_ptr(ushort code)
+static void push_subr_peek(ushort code)
 {
 	char *expr;
 	TYPE type;
-	
+
 	check_stack(1);
-	
+
 	if (_unsafe)
 	{
 		type = get_type(-1);
@@ -2598,14 +2622,14 @@ static void push_subr_ptr(ushort code)
 
 				expr = STR_copy(peek(-1, type));
 				pop_stack(1);
-				
+
 				code &= 0xF;
 
 				if (type == T_POINTER)
 					push(code, "*(%s *)(%s)", JIT_get_ctype(code), expr);
 				else
 					push(code, "*(%s *)GET_STRING_ADDR(%s)", JIT_get_ctype(code), expr);
-			
+
 				STR_free(expr);
 				return;
 		}
@@ -2614,6 +2638,67 @@ static void push_subr_ptr(ushort code)
 	push_subr(CALL_SUBR_CODE, code);
 }
 
+
+static void push_subr_poke(ushort code)
+{
+	char *expr = NULL;
+	char *expr1;
+	char *expr2;
+	TYPE type;
+	int i;
+
+	check_stack(2);
+
+	code &= 0xF;
+
+	if (_unsafe)
+	{
+		type = get_type(-2);
+		switch (type)
+		{
+			case T_POINTER:
+			case T_STRING:
+			case T_CSTRING:
+
+				expr1 = peek(-2, type);
+				expr2 = peek(-1, code);
+
+				if (type == T_POINTER)
+					STR_add(&expr, "*(%s *)(%s) = %s;", JIT_get_ctype(code), expr1, expr2);
+				else
+					STR_add(&expr, "*(%s *)GET_STRING_ADDR(%s) = %s;", JIT_get_ctype(code), expr1, expr2);
+
+				pop_stack(2);
+
+				push(T_VOID, "({%s})", expr);
+
+				/*if (check_swap(type, "({%s})", expr))
+					pop(T_VOID, NULL);*/
+
+				STR_free(expr);
+				return;
+		}
+	}
+
+	for (i = _stack_current - 2; i < _stack_current; i++)
+	{
+		STR_add(&expr, "%s;", push_expr(i, get_type(i)));
+		free_stack(i);
+	}
+
+	_stack_current -= 2;
+
+	STR_add(&expr, "CALL_SUBR_CODE(%d, %p, %d); POP_V();", _pc, JIT.subr_poke, code);
+
+	push(T_VOID, "({%s})", expr);
+
+	/*if (check_swap(code & 0xF, "({%s})", expr))
+		pop(T_VOID, NULL);*/
+
+	STR_free(expr);
+}
+
+
 #define GET_XXX()   (((signed short)(code << 4)) >> 4)
 #define GET_UXX()   (code & 0xFFF)
 #define GET_7XX()   (code & 0x7FF)
@@ -2786,7 +2871,7 @@ bool JIT_translate_body(FUNCTION *func, int ind)
 		/* 9C Quote$...       */  &&_SUBR_CODE,
 		/* 9D Unquote$...     */  &&_SUBR_CODE,
 		/* 9E MkInt$...       */  &&_SUBR_CODE,
-		/* 9F Byte@...        */  &&_SUBR_PTR,
+		/* 9F Byte@...        */  &&_SUBR_PEEK,
 		/* A0 ADD QUICK       */  &&_ADD_QUICK,
 		/* A1 ADD QUICK       */  &&_ADD_QUICK,
 		/* A2 ADD QUICK       */  &&_ADD_QUICK,
@@ -2868,18 +2953,18 @@ bool JIT_translate_body(FUNCTION *func, int ind)
 		/* EE PUSH CONST      */  &&_PUSH_CONST,
 		/* EF PUSH CONST      */  &&_PUSH_CONST_EX,
 		/* F0 PUSH QUICK      */  &&_PUSH_QUICK,
-		/* F1 PUSH QUICK      */  &&_PUSH_QUICK,
-		/* F2 PUSH QUICK      */  &&_PUSH_QUICK,
-		/* F3 PUSH QUICK      */  &&_PUSH_QUICK,
-		/* F4 PUSH QUICK      */  &&_PUSH_QUICK,
+		/* F1 PUSH QUICK      */  &&_PUSH_LOCAL_NOREF,
+		/* F2 PUSH QUICK      */  &&_PUSH_PARAM_NOREF,
+		/* F3 PUSH QUICK      */  &&_JUMP_IF_TRUE_FAST,
+		/* F4 PUSH QUICK      */  &&_JUMP_IF_FALSE_FAST,
 		/* F5 PUSH QUICK      */  &&_PUSH_QUICK,
 		/* F6 PUSH QUICK      */  &&_PUSH_QUICK,
-		/* F7 PUSH QUICK      */  &&_PUSH_QUICK,
-		/* F8 PUSH QUICK      */  &&_PUSH_QUICK,
-		/* F9 PUSH QUICK      */  &&_PUSH_QUICK,
-		/* FA PUSH QUICK      */  &&_PUSH_QUICK,
-		/* FB PUSH QUICK      */  &&_PUSH_QUICK,
-		/* FC PUSH QUICK      */  &&_PUSH_QUICK,
+		/* F7 PUSH QUICK      */  &&_PUSH_FLOAT,
+		/* F8 PUSH QUICK      */  &&_SUBR_POKE,
+		/* F9 PUSH QUICK      */  &&_POP_LOCAL_NOREF,
+		/* FA PUSH QUICK      */  &&_POP_PARAM_NOREF,
+		/* F9 PUSH QUICK      */  &&_POP_LOCAL_FAST,
+		/* FA PUSH QUICK      */  &&_POP_PARAM_FAST,
 		/* FD PUSH QUICK      */  &&_PUSH_QUICK,
 		/* FE PUSH QUICK      */  &&_PUSH_QUICK,
 		/* FF PUSH QUICK      */  &&_PUSH_QUICK
@@ -2959,6 +3044,13 @@ _PUSH_LOCAL:
 	
 	goto _MAIN;
 
+_PUSH_LOCAL_NOREF:
+
+	if (class->not_3_18)
+		goto _PUSH_QUICK;
+	else
+		goto _PUSH_LOCAL;
+
 _POP_LOCAL:
 
 	index = GET_XX();
@@ -2976,6 +3068,14 @@ _POP_LOCAL:
 	
 	goto _MAIN;
 
+_POP_LOCAL_NOREF:
+_POP_LOCAL_FAST:
+
+	if (class->not_3_18)
+		goto _PUSH_QUICK;
+	else
+		goto _POP_LOCAL;
+
 _POP_CTRL:
 
 	pop_ctrl(GET_XX() - func->n_local, T_VOID);
@@ -2987,6 +3087,13 @@ _PUSH_PARAM:
 	push(func->param[index].type, "p%d", index);
 	goto _MAIN;
 
+_PUSH_PARAM_NOREF:
+
+	if (class->not_3_18)
+		goto _PUSH_QUICK;
+	else
+		goto _PUSH_PARAM;
+
 _POP_PARAM:
 
 	index = func->n_param + GET_XX();
@@ -2997,11 +3104,27 @@ _POP_PARAM:
 	
 	goto _MAIN;
 
+_POP_PARAM_NOREF:
+_POP_PARAM_FAST:
+
+	if (class->not_3_18)
+		goto _PUSH_QUICK;
+	else
+		goto _POP_PARAM;
+
 _PUSH_QUICK:
 
 	push(T_INTEGER, "%d", GET_XXX());
 	goto _MAIN;
 
+_PUSH_FLOAT:
+
+	if (class->not_3_18)
+		goto _PUSH_QUICK;
+
+	push(T_FLOAT, "%d", GET_XX());
+	goto _MAIN;
+
 _PUSH_SHORT:
 
 	push(T_INTEGER, "%d", (short)PC[0]);
@@ -3275,6 +3398,20 @@ _JUMP_IF_FALSE:
 	p++;
 	goto _MAIN;
 
+_JUMP_IF_TRUE_FAST:
+
+	if (class->not_3_18)
+		goto _PUSH_QUICK;
+	else
+		goto _JUMP_IF_TRUE;
+
+_JUMP_IF_FALSE_FAST:
+
+	if (class->not_3_18)
+		goto _PUSH_QUICK;
+	else
+		goto _JUMP_IF_FALSE;
+
 _JUMP_FIRST:
 
 	index = PC[2] & 0xFF;
@@ -3523,11 +3660,18 @@ _SUBR_VARPTR:
 	push_subr_varptr(code);
 	goto _MAIN;
 	
-_SUBR_PTR:
+_SUBR_PEEK:
 
-	push_subr_ptr(code);
+	push_subr_peek(code);
 	goto _MAIN;
-	
+
+_SUBR_POKE:
+
+	if (class->not_3_18)
+		goto _PUSH_QUICK;
+	push_subr_poke(code);
+	goto _MAIN;
+
 _BREAK:
 
 	goto _MAIN;
@@ -3556,14 +3700,9 @@ _TRY:
 	
 _END_TRY:
 
-	JIT_print("  *JIT.got_error = 0;\n");
+	JIT_print("  JIT.set_got_error(0);\n");
 	JIT_print("  } CATCH {\n");
-	JIT_print("  if (SP > sp) sp = SP; else SP = sp;\n");
-	JIT_print("  LEAVE_SUPER();\n");
-	//JIT_print("  if (sp > EP) { fprintf(stderr, \"release try %%d\\n\", sp - EP); JIT.release_many(sp, sp - EP); SP = sp = EP; }\n");
-	JIT_print("  if (sp > EP) { JIT.release_many(sp, sp - EP); SP = sp = EP; }\n");
-	JIT_print("  *JIT.got_error = 1;\n");
-	JIT_print("  JIT.error_set_last(FALSE);\n");
+	JIT_print("  sp = _jit_end_try(psp, sp);\n");
 	JIT_print("  } END_TRY }\n");
 	JIT_print("  EP = tp;\n");
 	goto _MAIN;
