@@ -2,7 +2,7 @@
 
 	gb_arch_temp.h
 
-	(c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+	(c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -69,7 +69,7 @@ static void load_arch(ARCH *arch, const char *path)
 	
 	_path = path;
 
-	arch->fd = open(path, O_RDONLY);
+	arch->fd = open(path, O_RDONLY | O_CLOEXEC);
 	if (arch->fd < 0)
 		THROW(E_OPEN, path, strerror(errno));
 
