@@ -2,7 +2,7 @@
 
 	gb_code_temp.h
 
-	(c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+	(c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -254,7 +254,7 @@ static ushort *get_last_code2()
 	return &cur_func->code[cur_func->last_code2];
 }
 
-bool CODE_popify_last(void)
+bool CODE_popify_last(bool no_conv)
 {
 	/*
 	#ifdef DEBUG
@@ -272,43 +272,33 @@ bool CODE_popify_last(void)
 
 	op = *last_code & 0xFF00;
 
-	if ((op >= C_PUSH_LOCAL) && (op <= C_PUSH_UNKNOWN))
+	if ((op >= C_PUSH_LOCAL && op <= C_PUSH_UNKNOWN))
 	{
 		*last_code += 0x0800;
-		use_stack(-2);
-		#ifdef DEBUG
-		printf("Popify Last\n");
-		#endif
-		return TRUE;
 	}
-
-	if ((op & 0xF000) == C_PUSH_DYNAMIC)
+	else if (op == C_PUSH_LOCAL_NOREF)
+	{
+		if (no_conv)
+			*last_code = C_POP_LOCAL_FAST | (*last_code & 0xFF);
+		else
+			*last_code = C_POP_LOCAL_NOREF | (*last_code & 0xFF);
+	}
+	else if (op == C_PUSH_PARAM_NOREF)
+	{
+		if (no_conv)
+			*last_code = C_POP_PARAM_FAST | (*last_code & 0xFF);
+		else
+			*last_code = C_POP_PARAM_NOREF | (*last_code & 0xFF);
+	}
+	else if ((op & 0xF000) == C_PUSH_DYNAMIC)
 	{
 		*last_code += 0x1000;
-		use_stack(-2);
-		#ifdef DEBUG
-		printf("Popify Last\n");
-		#endif
-		return TRUE;
 	}
+	else
+		return FALSE;
 
-	/*
-	if (*last_code == (C_PUSH_MISC | CPM_LAST))
-	{
-		*last_code = C_PUSH_MISC | CPM_POP_LAST;
-		use_stack(-2);
-		return TRUE;
-	}
-	*/
-	/*
-	if (op == C_CALL)
-	{
-		*last_code = C_CALL_POP | (*last_code & 0xFF);
-		return TRUE;
-	}
-	*/
-
-	return FALSE;
+	use_stack(-2);
+	return TRUE;
 }
 
 
@@ -345,7 +335,7 @@ bool CODE_check_pop_local_last(short *local)
 	if (!last_code)
 		return FALSE;
 
-	if ((*last_code & 0xFF00) == C_POP_LOCAL)
+	if ((*last_code & 0xFF00) == C_POP_LOCAL || (*last_code & 0xFF00) == C_POP_LOCAL_NOREF)
 	{
 		*local = *last_code & 0xFF;
 		return TRUE;
@@ -382,7 +372,9 @@ bool CODE_check_varptr(void)
 		return TRUE;
 
 	op = *last_code;
-	if (!((op & 0xFF00) == C_PUSH_LOCAL || (op & 0xFF00) == C_PUSH_PARAM || (op & 0xF800) == C_PUSH_STATIC || (op & 0xF800) == C_PUSH_DYNAMIC))
+	if (!((op & 0xFF00) == C_PUSH_LOCAL || (op & 0xFF00) == C_PUSH_LOCAL_NOREF
+			|| (op & 0xFF00) == C_PUSH_PARAM || (op & 0xF800) == C_PUSH_STATIC
+			|| (op & 0xF800) == C_PUSH_DYNAMIC))
 		return TRUE;
 
 	*last_code = C_PUSH_INTEGER;
@@ -419,7 +411,7 @@ bool CODE_check_ismissing(void)
 		return TRUE;
 
 	op = *last_code;
-	if ((op & 0xFF00) != C_PUSH_PARAM)
+	if ((op & 0xFF00) != C_PUSH_PARAM && ((op & 0xFF00) != C_PUSH_PARAM_NOREF))
 		return TRUE;
 
 	*last_code = C_PUSH_QUICK | (op & 0xFF);
@@ -438,7 +430,7 @@ void CODE_push_number(int value)
 
 	use_stack(1);
 
-	if (value >= -2048L && value < 2048L)
+	if (value >= -256 && value < 256)
 	{
 		#ifdef DEBUG
 		printf("PUSH QUICK %d\n", value);
@@ -463,6 +455,15 @@ void CODE_push_number(int value)
 	}
 }
 
+void CODE_push_float(int value)
+{
+	LAST_CODE;
+
+	use_stack(1);
+
+	write_ZZxx(C_PUSH_FLOAT, value);
+}
+
 
 void CODE_push_const(ushort value)
 {
@@ -488,7 +489,7 @@ void CODE_push_const(ushort value)
 }
 
 
-void CODE_push_local(short num)
+void CODE_push_local_ref(short num, bool ref)
 {
 	LAST_CODE;
 
@@ -500,10 +501,20 @@ void CODE_push_local(short num)
 	else
 		printf("PUSH PARAM %d\n", (-1) - num);
 	#endif
-	if (num >= 0)
-		write_ZZxx(C_PUSH_LOCAL, num);
+	if (!ref && COMP_version >= 0x03180000)
+	{
+		if (num >= 0)
+			write_ZZxx(C_PUSH_LOCAL_NOREF, num);
+		else
+			write_ZZxx(C_PUSH_PARAM_NOREF, num);
+	}
 	else
-		write_ZZxx(C_PUSH_PARAM, num);
+	{
+		if (num >= 0)
+			write_ZZxx(C_PUSH_LOCAL, num);
+		else
+			write_ZZxx(C_PUSH_PARAM, num);
+	}
 }
 
 
@@ -527,6 +538,30 @@ void CODE_pop_local(short num)
 		write_ZZxx(C_POP_PARAM, num);
 }
 
+void CODE_pop_local_noref(short num)
+{
+	if (COMP_version < 0x03180000)
+	{
+		CODE_pop_local(num);
+		return;
+	}
+
+	LAST_CODE;
+
+	use_stack(-1);
+
+	#ifdef DEBUG
+	if (num >= 0)
+		printf("POP LOCAL NOREF #%d\n", num);
+	else
+		printf("POP PARAM NOREF #%d\n", (-1) - num);
+	#endif
+	if (num >= 0)
+		write_ZZxx(C_POP_LOCAL_NOREF, num);
+	else
+		write_ZZxx(C_POP_PARAM_NOREF, num);
+}
+
 
 void CODE_pop_ctrl(short num)
 {
@@ -744,60 +779,33 @@ void CODE_on(uchar num)
 }
 
 
-void CODE_jump_if_true()
+void CODE_jump_if(bool test, bool fast)
 {
-	/*
-	ushort *last_code = get_last_code();
 	ushort op;
 
-	if (last_code && PCODE_is(*last_code, C_NOT))
-	{
-		remove_last();
-		op = C_JUMP_IF_FALSE;
-	}
-	else
-		op = C_JUMP_IF_TRUE;
-	*/
-
 	use_stack(-1);
 
 	#ifdef DEBUG
-	printf("JUMP IF TRUE\n");
+	printf("JUMP IF %s\n", test ? "TRUE" : "FALSE");
 	#endif
 
 	LAST_CODE;
 
-	write_short(C_JUMP_IF_TRUE);
-	/**pos = CODE_get_current_pos();*/
-	write_short(0);
-}
-
-
-void CODE_jump_if_false()
-{
-	/*
-	ushort *last_code = get_last_code();
-	ushort op;
-
-	if (last_code && PCODE_is(*last_code, C_NOT))
+	if (test)
 	{
-		remove_last();
-		op = C_JUMP_IF_TRUE;
+		if (fast && COMP_version >= 0x03180000)
+			op = C_JUMP_IF_TRUE_FAST;
+		else
+			op = C_JUMP_IF_TRUE;
 	}
 	else
-		op = C_JUMP_IF_FALSE;
-	*/
-
-	use_stack(-1);
-
-	#ifdef DEBUG
-	printf("JUMP IF FALSE\n");
-	#endif
-
-	LAST_CODE;
-
-	write_short(C_JUMP_IF_FALSE);
-	/**pos = CODE_get_current_pos();*/
+	{
+		if (fast && COMP_version >= 0x03180000)
+			op = C_JUMP_IF_FALSE_FAST;
+		else
+			op = C_JUMP_IF_FALSE;
+	}
+	write_short(op);
 	write_short(0);
 }
 
@@ -893,33 +901,40 @@ void CODE_op(short op, short subcode, short nparam, bool fixed)
 		{
 			value = *last_code & 0xFFF;
 			if (value >= 0x800) value |= 0xF000;
-			if (op == C_SUB) value = (-value);
+			if (op == C_SUB) value = (-value); // prevent -256 to be valid!
 
 			#ifdef DEBUG
 			printf("ADD QUICK %d\n", value);
 			#endif
 
-			*last_code = C_ADD_QUICK | (value & 0x0FFF);
-
-			use_stack(1 - nparam);
-
-			// Now, look if we are PUSH QUICK then ADD QUICK
-
-			last_code = get_last_code2();
-			if (last_code && ((*last_code & 0xF000) == C_PUSH_QUICK))
+			if (COMP_version < 0x03180000 || (value > -256 && value < 256))
 			{
-				value2 = *last_code & 0xFFF;
-				if (value2 >= 0x800) value2 |= 0xF000;
-				value += value2;
+				*last_code = C_ADD_QUICK | (value & 0x0FFF);
 
-				if (value >= -2048L && value < 2048L)
+				use_stack(1 - nparam);
+
+				// Now, look if we are PUSH QUICK then ADD QUICK
+
+				last_code = get_last_code2();
+				if (last_code && ((*last_code & 0xF000) == C_PUSH_QUICK))
 				{
-					*last_code = C_PUSH_QUICK | (value & 0x0FFF);
-					CODE_undo();
-				}
-			}
+					value2 = *last_code & 0xFFF;
+					if (value2 >= 0x800) value2 |= 0xF000;
 
-			return;
+					if (value2 > -256 && value2 < 256)
+					{
+						value += value2;
+
+						if (value >= -256 && value < 256)
+						{
+							*last_code = C_PUSH_QUICK | (value & 0x0FFF);
+							CODE_undo();
+						}
+					}
+				}
+
+				return;
+			}
 		}
 	}
 
