@@ -2,7 +2,7 @@
 
   gb_reserved.h
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -71,7 +71,22 @@ enum {
 	RST_GET,
 	RST_COLLECTION,
 	RST_EXEC,
-	RST_READ
+	RST_READ,
+	RST_DIV
+};
+
+enum {
+	RSJ_OTHER,
+	RSJ_ME,
+	RSJ_CLASS,
+	RSJ_STRUCT,
+	RSJ_SUB,
+	RSJ_CONST,
+	RSJ_READ,
+	RSJ_DATATYPE,
+	RSJ_OPTIONAL,
+	RSJ_BYREF,
+	RSJ_ERROR
 };
 
 #define RES_is_operator(value) (COMP_res_info[value].flag & RSF_OP)
@@ -95,6 +110,8 @@ enum {
 
 #define RES_is_preprocessor(value) (COMP_res_info[value].flag & RSF_PREP)
 
+#define RES_is_identifier(value) (isalpha(*COMP_res_info[value].name))
+
 typedef
 	enum {
 		RS_NONE,
