@@ -2,7 +2,7 @@
 
   gb_reserved_keyword.h
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -28,203 +28,203 @@ COMP_INFO COMP_res_info[] =
 {
 	{ "" },
 
-	{ "Boolean",      RSF_TYPE,                 T_BOOLEAN,  8     },
-	{ "Byte",         RSF_TYPE,                 T_BYTE,     8     },
-	{ "Date",         RSF_TYPE,                 T_DATE,     8     },
-	{ "Single",       RSF_TYPE,                 T_SINGLE,   8     },
-	{ "Float",        RSF_TYPE,                 T_FLOAT,    8     },
-	{ "Integer",      RSF_TYPE,                 T_INTEGER,  8     },
-	{ "Long",         RSF_TYPE,                 T_LONG,     8     },
-	{ "Short",        RSF_TYPE,                 T_SHORT,    8     },
-	{ "String",       RSF_TYPE,                 T_STRING,   8     },
-	{ "Variant",      RSF_TYPE,                 T_VARIANT,  8     },
-	{ "Object",       RSF_TYPE,                 T_OBJECT,   8     },
-	{ "Pointer",      RSF_TYPE,                 T_POINTER,  8     },
-	{ "Class",        RSF_CLASS|RSF_AS,         0,          2     },
-	{ "Function",     RSF_IDENT,                0,          4     },
-	{ "Struct",       RSF_PREV,                 0,          3     },
-	{ "Const",        RSF_IDENT,                0,          5     },
-	{ "Private",      RSF_IDENT|RSF_PUB,                          },
-	{ "Public",       RSF_IDENT|RSF_PUB,                          },
-	{ "Static",       RSF_PUB                                     },
-	{ "Fast",         RSF_PUB                                     },
-	{ "Unsafe",       RSF_PUB                                     },
-	{ "Property",     RSF_IDENT                                   },
-	{ "Event",        RSF_IDENT|RSF_EVENT                         },
-	{ "Inherits",     RSF_CLASS|RSF_AS                            },
-	{ "Implements"                                                },
-	{ "Export"                                                    },
-	{ "As",           RSF_AS                                      },
-	{ "Of"                                                        },
-	{ "Dim",          RSF_IDENT,                                  },
-	{ "New",          RSF_AS ,                  0,          1     },
-	{ "Procedure",    RSF_IDENT,                0,          4     },
-	{ "Sub",          RSF_IDENT,                0,          4     },
-	{ "Return"                                                    },
-	{ "Optional"                                                  },
-	{ "Output"                                                    },
-	{ "Do"                                                        },
-	{ "Loop"                                                      },
-	{ "While"                                                     },
-	{ "Until"                                                     },
-	{ "Repeat"                                                    },
-	{ "Wend"                                                      },
-	{ "If"                                                        },
-	{ "Then"                                                      },
-	{ "Else"                                                      },
-	{ "Endif"                                                     },
-	{ "End"                                                       },
-	{ "For"                                                       },
-	{ "To"                                                        },
-	{ "DownTo"                                                    },
-	{ "From"                                                      },
-	{ "Step"                                                      },
-	{ "Next"                                                      },
-	{ "Select"                                                    },
-	{ "Case"                                                      },
-	{ "Exit"                                                      },
-	{ "Break"                                                     },
-	{ "Continue"                                                  },
-	{ "Goto"                                                      },
-	{ "GoSub"                                                     },
-	{ "On"                                                        },
-	{ "Me",           0,                        0,          1     },
-	{ "Last",         0,                        0,          1     },
-	{ "Try"                                                       },
-	{ "Finally"                                                   },
-	{ "Catch"                                                     },
-	{ "With"                                                      },
-	{ "True"                                                      },
-	{ "False"                                                     },
-	{ "Swap"                                                      },
-	{ "Null"                                                      },
-	{ "Extern",       RSF_IDENT,                0,          5     },
-	{ "Each"                                                      },
-	{ "In"                                                        },
-	{ "Default"                                                   },
-	{ "Stop"                                                      },
-	{ "Quit"                                                      },
-	{ "Raise",        RSF_IDENT|RSF_EVENT                         },
-	{ "Error"                                                     },
-	{ "Super",        0,                        0,          1     },
-	{ "Enum",         0,                        0,          6     },
-	{ "Let"                                                       },
-	{ "+Inf"                                                      },
-	{ "-Inf"                                                      },
-	{ "Use"                                                       },
+	{ "Boolean",      RSF_TYPE,                 T_BOOLEAN,  RSJ_DATATYPE     },
+	{ "Byte",         RSF_TYPE,                 T_BYTE,     RSJ_DATATYPE     },
+	{ "Date",         RSF_TYPE,                 T_DATE,     RSJ_DATATYPE     },
+	{ "Single",       RSF_TYPE,                 T_SINGLE,   RSJ_DATATYPE     },
+	{ "Float",        RSF_TYPE,                 T_FLOAT,    RSJ_DATATYPE     },
+	{ "Integer",      RSF_TYPE,                 T_INTEGER,  RSJ_DATATYPE     },
+	{ "Long",         RSF_TYPE,                 T_LONG,     RSJ_DATATYPE     },
+	{ "Short",        RSF_TYPE,                 T_SHORT,    RSJ_DATATYPE     },
+	{ "String",       RSF_TYPE,                 T_STRING,   RSJ_DATATYPE     },
+	{ "Variant",      RSF_TYPE,                 T_VARIANT,  RSJ_DATATYPE     },
+	{ "Object",       RSF_TYPE,                 T_OBJECT,   RSJ_DATATYPE     },
+	{ "Pointer",      RSF_TYPE,                 T_POINTER,  RSJ_DATATYPE     },
+	{ "Class",        RSF_CLASS|RSF_AS,         0,          RSJ_CLASS        },
+	{ "Function",     RSF_IDENT,                0,          RSJ_SUB          },
+	{ "Struct",       RSF_PREV,                 0,          RSJ_STRUCT       },
+	{ "Const",        RSF_IDENT,                0,          RSJ_CONST        },
+	{ "Private",      RSF_IDENT|RSF_PUB,                                     },
+	{ "Public",       RSF_IDENT|RSF_PUB,                                     },
+	{ "Static",       RSF_PUB                                                },
+	{ "Fast",         RSF_PUB                                                },
+	{ "Unsafe",       RSF_PUB                                                },
+	{ "Property",     RSF_IDENT                                              },
+	{ "Event",        RSF_IDENT|RSF_EVENT                                    },
+	{ "Inherits",     RSF_CLASS|RSF_AS                                       },
+	{ "Implements"                                                           },
+	{ "Export"                                                               },
+	{ "As",           RSF_AS                                                 },
+	{ "Of"                                                                   },
+	{ "Dim",          RSF_IDENT,                                             },
+	{ "New",          RSF_AS ,                  0,          RSJ_ME           },
+	{ "Procedure",    RSF_IDENT,                0,          RSJ_SUB          },
+	{ "Sub",          RSF_IDENT,                0,          RSJ_SUB          },
+	{ "Return"                                                               },
+	{ "Optional",     0,                        0,          RSJ_OPTIONAL     },
+	{ "Output"                                                               },
+	{ "Do"                                                                   },
+	{ "Loop"                                                                 },
+	{ "While"                                                                },
+	{ "Until"                                                                },
+	{ "Repeat"                                                               },
+	{ "Wend"                                                                 },
+	{ "If"                                                                   },
+	{ "Then"                                                                 },
+	{ "Else"                                                                 },
+	{ "Endif"                                                                },
+	{ "End"                                                                  },
+	{ "For"                                                                  },
+	{ "To"                                                                   },
+	{ "DownTo"                                                               },
+	{ "From"                                                                 },
+	{ "Step"                                                                 },
+	{ "Next"                                                                 },
+	{ "Select"                                                               },
+	{ "Case"                                                                 },
+	{ "Exit"                                                                 },
+	{ "Break"                                                                },
+	{ "Continue"                                                             },
+	{ "Goto"                                                                 },
+	{ "GoSub"                                                                },
+	{ "On"                                                                   },
+	{ "Me",           0,                        0,          RSJ_ME           },
+	{ "Last",         0,                        0,          RSJ_ME           },
+	{ "Try"                                                                  },
+	{ "Finally"                                                              },
+	{ "Catch"                                                                },
+	{ "With"                                                                 },
+	{ "True",         0,                        0,          RSJ_ME           },
+	{ "False",        0,                        0,          RSJ_ME           },
+	{ "Swap"                                                                 },
+	{ "Null",         0,                        0,          RSJ_ME           },
+	{ "Extern",       RSF_IDENT,                0,          RSJ_CONST        },
+	{ "Each"                                                                 },
+	{ "In"                                                                   },
+	{ "Default"                                                              },
+	{ "Stop"                                                                 },
+	{ "Quit"                                                                 },
+	{ "Raise",        RSF_IDENT|RSF_EVENT                                    },
+	{ "Error",        0,                        0,          RSJ_ERROR        },
+	{ "Super",        0,                        0,          RSJ_ME           },
+	{ "Enum",         0,                        0,          RSJ_CONST        },
+	{ "Let"                                                                  },
+	{ "+Inf"                                                                 },
+	{ "-Inf"                                                                 },
+	{ "Use"                                                                  },
 
-	{ "Print"                                                     },
-	{ "Input"                                                     },
-	{ "Read",         RSF_PREV,                 0,          7     },
-	{ "Peek",                                                     },
-	{ "Write"                                                     },
-	{ "Open"                                                      },
-	{ "Close"                                                     },
-	{ "Seek"                                                      },
-	{ "Append"                                                    },
-	{ "Create"                                                    },
-	{ "Binary"                                                    },
-	{ "Line"                                                      },
-	{ "Flush"                                                     },
-	{ "Exec"                                                      },
-	{ "Shell"                                                     },
-	{ "Wait"                                                      },
-	{ "Sleep"                                                     },
-	{ "Kill"                                                      },
-	{ "Move"                                                      },
-	{ "Copy"                                                      },
-	{ "Inc"                                                       },
-	{ "Dec"                                                       },
-	{ "Mkdir"                                                     },
-	{ "Rmdir"                                                     },
-	{ "Watch"                                                     },
-	{ "Link"                                                      },
-	{ "Lock"                                                      },
-	{ "Unlock"                                                    },
-	{ "Library"                                                   },
-	{ "Debug"                                                     },
-	{ "Assert"                                                    },
-	{ "Pipe"                                                      },
-	{ "Randomize"                                                 },
-	{ "ByRef"                                                     },
-	{ "Memory"                                                    },
-	{ "Chmod"                                                     },
-	{ "Chown"                                                     },
-	{ "Chgrp"                                                     },
+	{ "Print"                                                                },
+	{ "Input"                                                                },
+	{ "Read",         RSF_PREV,                 0,          RSJ_READ         },
+	{ "Peek",                                                                },
+	{ "Write",        RSF_PREV,                 0,          RSJ_READ         },
+	{ "Open"                                                                 },
+	{ "Close"                                                                },
+	{ "Seek"                                                                 },
+	{ "Append"                                                               },
+	{ "Create"                                                               },
+	{ "Binary"                                                               },
+	{ "Line"                                                                 },
+	{ "Flush"                                                                },
+	{ "Exec"                                                                 },
+	{ "Shell"                                                                },
+	{ "Wait"                                                                 },
+	{ "Sleep"                                                                },
+	{ "Kill"                                                                 },
+	{ "Move"                                                                 },
+	{ "Copy"                                                                 },
+	{ "Inc"                                                                  },
+	{ "Dec"                                                                  },
+	{ "Mkdir"                                                                },
+	{ "Rmdir"                                                                },
+	{ "Watch"                                                                },
+	{ "Link"                                                                 },
+	{ "Lock"                                                                 },
+	{ "Unlock"                                                               },
+	{ "Library"                                                              },
+	{ "Debug"                                                                },
+	{ "Assert"                                                               },
+	{ "Pipe"                                                                 },
+	{ "Randomize"                                                            },
+	{ "ByRef",        0,                        0,          RSJ_BYREF        },
+	{ "Memory"                                                               },
+	{ "Chmod"                                                                },
+	{ "Chown"                                                                },
+	{ "Chgrp"                                                                },
 	
-	{ "#If",          RSF_PREP,                                   },
-	{ "#Else",        RSF_PREP,                                   },
-	{ "#Endif",       RSF_PREP,                                   },
-	{ "#Const",       RSF_PREP,                                   },
-	{ "#Line",        RSF_PREP,                                   },
-	{ "#Include",     RSF_PREP,                                   },
-	{ "#Script",      RSF_PREP,                                   },
+	{ "#If",          RSF_PREP,                                              },
+	{ "#Else",        RSF_PREP,                                              },
+	{ "#Endif",       RSF_PREP,                                              },
+	{ "#Const",       RSF_PREP,                                              },
+	{ "#Line",        RSF_PREP,                                              },
+	{ "#Include",     RSF_PREP,                                              },
+	{ "#Script",      RSF_PREP,                                              },
 
-	{ ":",            RSF_NONE,                 OP_COLON,     0,  0,    T_OBJECT                      },  // Use for the immediate collection syntax
-	{ ";"                                                                                             },
-	{ ","                                                                                             },
-	{ "..."                                                                                           },
-	{ "#"                                                                                             },
-	{ "@"                                                                                             },
-	{ "?"                                                                                             },
-	{ "{"                                                                                             },
-	{ "}"                                                                                             },
-	{ "=",            RSF_OP2S,                 OP_EQUAL,     0,   4,   T_BOOLEAN,      C_EQ          },
-	{ "==",           RSF_OP2S,                 OP_NEAR,      0,   4,   T_BOOLEAN,      C_NEAR        },
-	{ "(",            RSF_OPP,                  OP_LBRA,      0,   12                                 },
-	{ ")",                                                                                            },
-	{ ".",            RSF_OP2|RSF_POINT,        OP_PT,        0,   20,  T_VARIANT                     },
-	{ "!",            RSF_OP2|RSF_POINT,        OP_EXCL,      0,   20,  T_VARIANT                     },
-	{ "+",            RSF_OP2,                  OP_PLUS,      0,   5,   RST_ADD,        C_ADD         },
-	{ "-",            RSF_OP2,                  OP_MINUS,     0,   5,   RST_ADD,        C_SUB         },
-	{ "*",            RSF_OP2,                  OP_STAR,      0,   6,   RST_ADD,        C_MUL         },
-	{ "/",            RSF_OP2,                  OP_SLASH,     0,   6,   T_FLOAT,        C_DIV         },
-	{ "^",            RSF_OP2S,                 OP_FLEX,      0,   7,   T_FLOAT,        C_POW         },
-	{ "&",            RSF_OPN,                  OP_AMP,       0,   9,   T_STRING,       C_CAT         },
-	{ "&/",           RSF_OPN,                  OP_FILE,      0,   8,   T_STRING,       C_FILE        },
-	{ ">",            RSF_OP2S,                 OP_GT,        0,   4,   T_BOOLEAN,      C_GT          },
-	{ "<",            RSF_OP2S,                 OP_LT,        0,   4,   T_BOOLEAN,      C_LT          },
-	{ ">=",           RSF_OP2S,                 OP_GE,        0,   4,   T_BOOLEAN,      C_GE          },
-	{ "<=",           RSF_OP2S,                 OP_LE,        0,   4,   T_BOOLEAN,      C_LE          },
-	{ "<>",           RSF_OP2S,                 OP_NE,        0,   4,   T_BOOLEAN,      C_NE          },
-	{ "[",            RSF_OPP,                  OP_LSQR,      0,   12,  RST_GET                       },
-	{ "]",            RSF_NONE,                 OP_RSQR,      0,   0,   T_OBJECT                      },  // Use for the immediate array syntax
-	{ "And",          RSF_OP2SM,                OP_AND,       0,   2,   RST_AND,        C_AND         },
-	{ "Or",           RSF_OP2SM,                OP_OR,        0,   2,   RST_AND,        C_OR          },
-	{ "Not",          RSF_OP1,                  OP_NOT,       0,   10,  RST_NOT,        C_NOT         },
-	{ "Xor",          RSF_OP2SM,                OP_XOR,       0,   2,   RST_AND,        C_XOR         },
-	{ "Shl",          RSF_OP2,                  OP_SHL,       0,   2,   RST_BCLR,       C_BCLR, 5     }, 
-	{ "Asl",          RSF_OP2,                  OP_SHL,       0,   2,   RST_BCLR,       C_BCLR, 5     }, 
-	{ "Shr",          RSF_OP2,                  OP_SHL,       0,   2,   RST_BCLR,       C_BCLR, 6     }, 
-	{ "Asr",          RSF_OP2,                  OP_SHL,       0,   2,   RST_BCLR,       C_BCLR, 6     }, 
-	{ "Rol",          RSF_OP2,                  OP_SHL,       0,   2,   RST_BCLR,       C_BCLR, 7     }, 
-	{ "Ror",          RSF_OP2,                  OP_SHL,       0,   2,   RST_BCLR,       C_BCLR, 8     }, 
-	{ "Lsl",          RSF_OP2,                  OP_SHL,       0,   2,   RST_BCLR,       C_BCLR, 9     }, 
-	{ "Lsr",          RSF_OP2,                  OP_SHL,       0,   2,   RST_BCLR,       C_BCLR, 10    }, 
-	{ "\\",           RSF_OP2S,                 OP_DIV,       0,   6,   T_INTEGER,      C_QUO         },
-	{ "Div",          RSF_OP2S,                 OP_DIV,       0,   6,   T_INTEGER,      C_QUO         },
-	{ "%",            RSF_OP2S,                 OP_MOD,       0,   6,   RST_MOD,        C_REM         },
-	{ "Mod",          RSF_OP2S,                 OP_MOD,       0,   6,   RST_MOD,        C_REM         },
-	{ "Is",           RSF_OP2|RSF_AS|RSF_NOT,   OP_IS,        0,   11,  T_BOOLEAN,      C_IS,   0     },
-	{ "",             RSF_OP2|RSF_AS,           OP_IS,        0,   11,  T_BOOLEAN,      C_IS,   1     },
-	{ "Like",         RSF_OP2S|RSF_NOT,         OP_LIKE,      0,   4,   T_BOOLEAN,      C_LIKE, 0     },
-	{ "",             RSF_OP2S,                 OP_LIKE,      0,   4,   T_BOOLEAN,      C_LIKE, 4     }, // NOT LIKE
-	{ "Begins",       RSF_OP2S|RSF_NOT,         OP_LIKE,      0,   4,   T_BOOLEAN,      C_LIKE, 1     },
-	{ "",             RSF_OP2S,                 OP_LIKE,      0,   4,   T_BOOLEAN,      C_LIKE, 5     }, // NOT BEGINS
-	{ "Ends",         RSF_OP2S|RSF_NOT,         OP_LIKE,      0,   4,   T_BOOLEAN,      C_LIKE, 2     },
-	{ "",             RSF_OP2S,                 OP_LIKE,      0,   4,   T_BOOLEAN,      C_LIKE, 6     }, // NOT ENDS
-	{ "Match",        RSF_OP2S|RSF_NOT,         OP_LIKE,      0,   4,   T_BOOLEAN,      C_LIKE, 3     },
-	{ "",             RSF_OP2S,                 OP_LIKE,      0,   4,   T_BOOLEAN,      C_LIKE, 7     }, // NOT MATCH
+	{ ":",            RSF_NONE,                 OP_COLON,   0,   0,    T_OBJECT                     },  // Use for the immediate collection syntax
+	{ ";"                                                                                           },
+	{ ","                                                                                           },
+	{ "..."                                                                                         },
+	{ "#"                                                                                           },
+	{ "@"                                                                                           },
+	{ "?"                                                                                           },
+	{ "{"                                                                                           },
+	{ "}"                                                                                           },
+	{ "=",            RSF_OP2S,                 OP_EQUAL,   0,   4,   T_BOOLEAN,      C_EQ          },
+	{ "==",           RSF_OP2S,                 OP_NEAR,    0,   4,   T_BOOLEAN,      C_NEAR        },
+	{ "(",            RSF_OPP,                  OP_LBRA,    0,   12                                 },
+	{ ")",                                                                                          },
+	{ ".",            RSF_OP2|RSF_POINT,        OP_PT,      0,   20,  T_VARIANT                     },
+	{ "!",            RSF_OP2|RSF_POINT,        OP_EXCL,    0,   20,  T_VARIANT                     },
+	{ "+",            RSF_OP2,                  OP_PLUS,    0,   5,   RST_ADD,        C_ADD         },
+	{ "-",            RSF_OP2,                  OP_MINUS,   0,   5,   RST_ADD,        C_SUB         },
+	{ "*",            RSF_OP2,                  OP_STAR,    0,   6,   RST_ADD,        C_MUL         },
+	{ "/",            RSF_OP2,                  OP_SLASH,   0,   6,   RST_DIV,        C_DIV         },
+	{ "^",            RSF_OP2S,                 OP_FLEX,    0,   7,   RST_DIV,        C_POW         },
+	{ "&",            RSF_OPN,                  OP_AMP,     0,   9,   T_STRING,       C_CAT         },
+	{ "&/",           RSF_OPN,                  OP_FILE,    0,   8,   T_STRING,       C_FILE        },
+	{ ">",            RSF_OP2S,                 OP_GT,      0,   4,   T_BOOLEAN,      C_GT          },
+	{ "<",            RSF_OP2S,                 OP_LT,      0,   4,   T_BOOLEAN,      C_LT          },
+	{ ">=",           RSF_OP2S,                 OP_GE,      0,   4,   T_BOOLEAN,      C_GE          },
+	{ "<=",           RSF_OP2S,                 OP_LE,      0,   4,   T_BOOLEAN,      C_LE          },
+	{ "<>",           RSF_OP2S,                 OP_NE,      0,   4,   T_BOOLEAN,      C_NE          },
+	{ "[",            RSF_OPP,                  OP_LSQR,    0,   12,  RST_GET                       },
+	{ "]",            RSF_NONE,                 OP_RSQR,    0,   0,   T_OBJECT                      },  // Use for the immediate array syntax
+	{ "And",          RSF_OP2SM,                OP_AND,     0,   2,   RST_AND,        C_AND         },
+	{ "Or",           RSF_OP2SM,                OP_OR,      0,   2,   RST_AND,        C_OR          },
+	{ "Not",          RSF_OP1,                  OP_NOT,     0,   10,  RST_NOT,        C_NOT         },
+	{ "Xor",          RSF_OP2SM,                OP_XOR,     0,   2,   RST_AND,        C_XOR         },
+	{ "Shl",          RSF_OP2,                  OP_SHL,     0,   2,   RST_BCLR,       C_BCLR, 5     },
+	{ "Asl",          RSF_OP2,                  OP_SHL,     0,   2,   RST_BCLR,       C_BCLR, 5     },
+	{ "Shr",          RSF_OP2,                  OP_SHL,     0,   2,   RST_BCLR,       C_BCLR, 6     },
+	{ "Asr",          RSF_OP2,                  OP_SHL,     0,   2,   RST_BCLR,       C_BCLR, 6     },
+	{ "Rol",          RSF_OP2,                  OP_SHL,     0,   2,   RST_BCLR,       C_BCLR, 7     },
+	{ "Ror",          RSF_OP2,                  OP_SHL,     0,   2,   RST_BCLR,       C_BCLR, 8     },
+	{ "Lsl",          RSF_OP2,                  OP_SHL,     0,   2,   RST_BCLR,       C_BCLR, 9     },
+	{ "Lsr",          RSF_OP2,                  OP_SHL,     0,   2,   RST_BCLR,       C_BCLR, 10    },
+	{ "\\",           RSF_OP2S,                 OP_DIV,     0,   6,   T_INTEGER,      C_QUO         },
+	{ "Div",          RSF_OP2S,                 OP_DIV,     0,   6,   T_INTEGER,      C_QUO         },
+	{ "%",            RSF_OP2S,                 OP_MOD,     0,   6,   RST_MOD,        C_REM         },
+	{ "Mod",          RSF_OP2S,                 OP_MOD,     0,   6,   RST_MOD,        C_REM         },
+	{ "Is",           RSF_OP2|RSF_AS|RSF_NOT,   OP_IS,      0,   11,  T_BOOLEAN,      C_IS,   0     },
+	{ "",             RSF_OP2|RSF_AS,           OP_IS,      0,   11,  T_BOOLEAN,      C_IS,   1     },
+	{ "Like",         RSF_OP2S|RSF_NOT,         OP_LIKE,    0,   4,   T_BOOLEAN,      C_LIKE, 0     },
+	{ "",             RSF_OP2S,                 OP_LIKE,    0,   4,   T_BOOLEAN,      C_LIKE, 4     }, // NOT LIKE
+	{ "Begins",       RSF_OP2S|RSF_NOT,         OP_LIKE,    0,   4,   T_BOOLEAN,      C_LIKE, 1     },
+	{ "",             RSF_OP2S,                 OP_LIKE,    0,   4,   T_BOOLEAN,      C_LIKE, 5     }, // NOT BEGINS
+	{ "Ends",         RSF_OP2S|RSF_NOT,         OP_LIKE,    0,   4,   T_BOOLEAN,      C_LIKE, 2     },
+	{ "",             RSF_OP2S,                 OP_LIKE,    0,   4,   T_BOOLEAN,      C_LIKE, 6     }, // NOT ENDS
+	{ "Match",        RSF_OP2S|RSF_NOT,         OP_LIKE,    0,   4,   T_BOOLEAN,      C_LIKE, 3     },
+	{ "",             RSF_OP2S,                 OP_LIKE,    0,   4,   T_BOOLEAN,      C_LIKE, 7     }, // NOT MATCH
 
-	{ "+=",           RSF_ASGN,                 RS_PLUS                                               },
-	{ "-=",           RSF_ASGN,                 RS_MINUS                                              },
-	{ "*=",           RSF_ASGN,                 RS_STAR                                               },
-	{ "/=",           RSF_ASGN,                 RS_SLASH                                              },
-	{ "\\=",          RSF_ASGN,                 RS_BSLASH                                             },
-	{ "%=",           RSF_ASGN,                 RS_PERCENT                                            },
-	{ "&=",           RSF_ASGN,                 RS_AMP                                                },
-	{ "&/=",          RSF_ASGN,                 RS_FILE                                               },
-	{ "^=",           RSF_ASGN,                 RS_FLEX                                               },
+	{ "+=",           RSF_ASGN,                 RS_PLUS                                             },
+	{ "-=",           RSF_ASGN,                 RS_MINUS                                            },
+	{ "*=",           RSF_ASGN,                 RS_STAR                                             },
+	{ "/=",           RSF_ASGN,                 RS_SLASH                                            },
+	{ "\\=",          RSF_ASGN,                 RS_BSLASH                                           },
+	{ "%=",           RSF_ASGN,                 RS_PERCENT                                          },
+	{ "&=",           RSF_ASGN,                 RS_AMP                                              },
+	{ "&/=",          RSF_ASGN,                 RS_FILE                                             },
+	{ "^=",           RSF_ASGN,                 RS_FLEX                                             },
 
 	{ NULL }
 };
