@@ -2,7 +2,7 @@
 
   gb_table_temp.h
 
-  (c) 2000-2017 Benoît Minisini <g4mba5@gmail.com>
+  (c) 2000-2017 Benoît Minisini <benoit.minisini@gambas-basic.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
 
 ***************************************************************************/
 
-#define __TABLE_C
+#define __GB_TABLE_C
 
 #include <stdlib.h>
 #include <string.h>
@@ -38,6 +38,7 @@
 #define SYM(table, ind) (TABLE_get_symbol(table, ind))
 #define SSYM(_symbol, _pos, _size) ((SYMBOL *)((char *)(_symbol) + (_pos) * (_size)))
 
+bool TABLE_new_symbol = FALSE;
 static char _buffer[MAX_SYMBOL_LEN + 1];
 
 #if TABLE_USE_KEY
@@ -86,13 +87,13 @@ char TABLE_compare(const char *s1, int len1, const char *s2, int len2)
 		c1 = s1[i];
 		c2 = s2[i];
 
-		if (LIKELY(c1 > c2)) return 1;
-		if (LIKELY(c1 < c2)) return -1;
+		if (c1 > c2) return 1;
+		if (c1 < c2) return -1;
 	}
 
-	if (LIKELY(len1 < len2))
+	if (len1 < len2)
 		return -1;
-	else if (LIKELY(len1 > len2))
+	else if (len1 > len2)
 		return 1;
 	else
 		return 0;
@@ -107,14 +108,14 @@ char TABLE_compare_ignore_case(const char *s1, int len1, const char *s2, int len
 	for (i = 0; len > 0; i++)
 	{
 		result = toupper(s1[i]) - toupper(s2[i]);
-		if (LIKELY(result))
+		if (result)
 			return result; // < 0 ? -1 : 1;
 		len--;
 	}
 
-	if (LIKELY(len1 < len2))
+	if (len1 < len2)
 		return -1;
-	else if (LIKELY(len1 > len2))
+	else if (len1 > len2)
 		return 1;
 	else
 		return 0;
@@ -124,15 +125,15 @@ char TABLE_compare_ignore_case_len(const char *s1, int len1, const char *s2, int
 {
 	int result;
 
-	if (LIKELY(len1 < len2))
+	if (len1 < len2)
 		return -1;
-	else if (LIKELY(len1 > len2))
+	else if (len1 > len2)
 		return 1;
 
 	while (len1)
 	{
 		result = tolower(*s1++) - tolower(*s2++);
-		if (LIKELY(result))
+		if (result)
 			return result; // < 0 ? -1 : 1;
 		--len1;
 	}
@@ -140,7 +141,7 @@ char TABLE_compare_ignore_case_len(const char *s1, int len1, const char *s2, int
 	return 0;
 }
 
-static inline bool search(void *symbol, ushort *sort, int n_symbol, size_t size, const char *name, int len, int *index)
+static inline int search(void *symbol, ushort *sort, int n_symbol, size_t size, const char *name, int len)
 {
 	int pos, deb, fin;
 	SYMBOL *sym;
@@ -158,19 +159,16 @@ static inline bool search(void *symbol, ushort *sort, int n_symbol, size_t size,
 
 	for(;;)
 	{
-		if (UNLIKELY(deb >= fin))
-		{
-			*index = deb;
-			return FALSE;
-		}
+		if (deb >= fin)
+			return (-deb) - 1;
 
 		pos = (deb + fin) >> 1;
 
 		sym = SSYM(symbol, sort[pos], size);
 		
-		if (LIKELY(len < sym->len))
+		if (len < sym->len)
 			goto __B_LOWER;
-		else if (LIKELY(len > sym->len))
+		else if (len > sym->len)
 			goto __B_GREATER;
 		
 #if TABLE_USE_KEY
@@ -194,12 +192,12 @@ static inline bool search(void *symbol, ushort *sort, int n_symbol, size_t size,
 			{
 				result = *s1 - *s2;
 				
-				if (LIKELY(result < 0))
+				if (result < 0)
 					goto __B_LOWER;
-				else if (LIKELY(result > 0))
+				else if (result > 0)
 					goto __B_GREATER;
 				
-				if (UNLIKELY(--l == 0))
+				if (--l == 0)
 					break;
 				
 				s1++;
@@ -207,8 +205,7 @@ static inline bool search(void *symbol, ushort *sort, int n_symbol, size_t size,
 			}
 		}
 
-		*index = pos;
-		return TRUE;
+		return pos;
 		
 		__B_LOWER: fin = pos; continue;
 		__B_GREATER: deb = pos + 1; continue;
@@ -216,7 +213,7 @@ static inline bool search(void *symbol, ushort *sort, int n_symbol, size_t size,
 }
 
 
-static inline bool search_ignore_case(void *symbol, ushort *sort, int n_symbol, size_t size, const char *name, int len, int *index)
+static inline int search_ignore_case(void *symbol, ushort *sort, int n_symbol, size_t size, const char *name, int len)
 {
 	int pos, deb, fin;
 	SYMBOL *sym;
@@ -234,19 +231,20 @@ static inline bool search_ignore_case(void *symbol, ushort *sort, int n_symbol,
 
 	for(;;)
 	{
-		if (UNLIKELY(deb >= fin))
+		if (deb >= fin)
 		{
-			*index = deb;
-			return FALSE;
+			return -deb - 1;
+			/**index = deb;
+			return FALSE;*/
 		}
 
 		pos = (deb + fin) >> 1;
 
 		sym = SSYM(symbol, sort[pos], size);
 
-		if (LIKELY(len < sym->len))
+		if (len < sym->len)
 			goto __T_LOWER;
-		else if (LIKELY(len > sym->len))
+		else if (len > sym->len)
 			goto __T_GREATER;
 
 #if TABLE_USE_KEY
@@ -270,12 +268,12 @@ static inline bool search_ignore_case(void *symbol, ushort *sort, int n_symbol,
 			{
 				result = tolower(*s1) - tolower(*s2);
 				
-				if (LIKELY(result < 0))
+				if (result < 0)
 					goto __T_LOWER;
-				else if (LIKELY(result > 0))
+				else if (result > 0)
 					goto __T_GREATER;
 				
-				if (UNLIKELY(--l == 0))
+				if (--l == 0)
 					break;
 				
 				s1++;
@@ -283,8 +281,9 @@ static inline bool search_ignore_case(void *symbol, ushort *sort, int n_symbol,
 			}
 		}
 
-		*index = pos;
-		return TRUE;
+		return pos;
+		/**index = pos;
+		return TRUE;*/
 		
 		__T_LOWER: fin = pos; continue;
 		__T_GREATER: deb = pos + 1; continue;
@@ -294,7 +293,7 @@ static inline bool search_ignore_case(void *symbol, ushort *sort, int n_symbol,
 
 const char *TABLE_get_symbol_name(TABLE *table, int index)
 {
-	if (UNLIKELY((index < 0) || (index >= ARRAY_count(table->symbol))))
+	if (index < 0 || index >= ARRAY_count(table->symbol))
 		strcpy(_buffer, "?");
 	else
 		SYMBOL_get_name(SYM(table, index));
@@ -307,7 +306,7 @@ const char *TABLE_get_symbol_name_suffix(TABLE *table, int index, const char* su
 {
 	SYMBOL *sym;
 	
-	if (UNLIKELY((index < 0) || (index >= ARRAY_count(table->symbol))))
+	if (index < 0 || index >= ARRAY_count(table->symbol))
 		return "?";
 	
 	sym = SYM(table, index);
@@ -339,16 +338,23 @@ void TABLE_create(TABLE **result, size_t size, TABLE_FLAG flag)
 }
 
 
+void TABLE_create_inc(TABLE **result, size_t size, TABLE_FLAG flag, uint inc)
+{
+	TABLE_create(result, size, flag);
+	ARRAY_set_inc((*result)->symbol, inc);
+	ARRAY_set_inc((*result)->sort, inc);
+}
+
+
 void TABLE_create_from(TABLE **result, size_t size, const char *sym_list[], TABLE_FLAG flag)
 {
 	TABLE *table;
-	int index;
 
 	TABLE_create(&table, size, flag);
 
 	while (*sym_list)
 	{
-		TABLE_add_symbol(table, *sym_list, strlen(*sym_list), &index);
+		TABLE_add_symbol(table, *sym_list, strlen(*sym_list));
 		sym_list++;
 	}
 
@@ -376,7 +382,6 @@ void TABLE_delete(TABLE **p_table)
 bool TABLE_find_symbol(TABLE *table, const char *name, int len, int *index)
 {
 	int ind;
-	bool result;
 	SYMBOL *tsym;
 	int count;
 	size_t size;
@@ -386,14 +391,17 @@ bool TABLE_find_symbol(TABLE *table, const char *name, int len, int *index)
 	size = ARRAY_size(tsym);
 
 	if (table->flag)
-		result = search_ignore_case(tsym, table->sort, count, size, name, len, &ind);
+		ind = search_ignore_case(tsym, table->sort, count, size, name, len);
 	else
-		result = search(tsym, table->sort, count, size, name, len, &ind);
+		ind = search(tsym, table->sort, count, size, name, len);
 	
-	if (result)
+	if (ind >= 0)
+	{
 		*index = table->sort[ind];
-
-	return result;
+		return TRUE;
+	}
+	else
+		return FALSE;
 }
 
 #if 0
@@ -418,10 +426,9 @@ void TABLE_add_new_symbol_without_sort(TABLE *table, const char *name, int len,
 }
 #endif
 
-bool TABLE_add_symbol(TABLE *table, const char *name, int len, int *index)
+int TABLE_add_symbol(TABLE *table, const char *name, int len)
 {
 	int ind;
-	bool result;
 	SYMBOL *sym;
 	int count;
 	size_t size;
@@ -433,12 +440,13 @@ bool TABLE_add_symbol(TABLE *table, const char *name, int len, int *index)
 	size = ARRAY_size(table->symbol);
 	
 	if (table->flag)
-		result = search_ignore_case(table->symbol, table->sort, count, size, name, len, &ind);
+		ind = search_ignore_case(table->symbol, table->sort, count, size, name, len);
 	else
-		result = search(table->symbol, table->sort, count, size, name, len, &ind);
+		ind = search(table->symbol, table->sort, count, size, name, len);
 
-	if (!result)
+	if (ind < 0)
 	{
+		ind = -ind - 1;
 		sym = (SYMBOL *)ARRAY_add_void_size(&table->symbol);
 
 		sym->name = (char *)name;
@@ -459,17 +467,14 @@ bool TABLE_add_symbol(TABLE *table, const char *name, int len, int *index)
 
 		table->sort[ind] = (ushort)count;
 		ind = count;
+		TABLE_new_symbol = TRUE;
 	}
 	else
 		ind = table->sort[ind]; //SYM(table, ind)->sort; /*table->symbol[ind].sort;*/
 
-	*index = ind;
-
-	return result;
+	return ind;
 }
 
-
-
 void TABLE_print(TABLE *table, bool sort)
 {
 	int i;
@@ -508,7 +513,7 @@ void TABLE_print(TABLE *table, bool sort)
 }
 
 
-void TABLE_copy_symbol_with_prefix(TABLE *table, int ind_src, char prefix, int *index)
+int TABLE_copy_symbol_with_prefix(TABLE *table, int ind_src, char prefix)
 {
 	SYMBOL *sym;
 	char *ptr;
@@ -517,12 +522,12 @@ void TABLE_copy_symbol_with_prefix(TABLE *table, int ind_src, char prefix, int *
 
 	ptr = (char *)sym->name - 1;
 
-	if (UNLIKELY(!isspace((unsigned char)*ptr)))
+	if (!isspace((unsigned char)*ptr))
 		ERROR_panic("Cannot add prefix to symbol");
 
 	*ptr = prefix;
 
-	TABLE_add_symbol(table, ptr, sym->len + 1, index);
+	return TABLE_add_symbol(table, ptr, sym->len + 1);
 }
 
 
@@ -538,11 +543,11 @@ int SYMBOL_find(void *symbol, ushort *sort, int n_symbol, size_t s_symbol, int f
 	int index;
 	int len_prefix;
 
-	if (UNLIKELY(prefix != NULL))
+	if (prefix != NULL)
 	{
 		len_prefix = strlen(prefix);
 
-		if (UNLIKELY((len + len_prefix) > MAX_SYMBOL_LEN))
+		if ((len + len_prefix) > MAX_SYMBOL_LEN)
 			ERROR_panic("SYMBOL_find: prefixed symbol too long");
 
 		strcpy(_buffer, prefix);
@@ -552,17 +557,14 @@ int SYMBOL_find(void *symbol, ushort *sort, int n_symbol, size_t s_symbol, int f
 	}
 
 	if (flag)
-	{
-		if (search_ignore_case(symbol, sort, n_symbol, s_symbol, name, len, &index))
-			return sort[index];
-	}
+		index = search_ignore_case(symbol, sort, n_symbol, s_symbol, name, len);
 	else
-	{
-		if (search(symbol, sort, n_symbol, s_symbol, name, len, &index))
-			return sort[index];
-	}
+		index = search(symbol, sort, n_symbol, s_symbol, name, len);
 
-	return NO_SYMBOL;
+	if (index >= 0)
+		return sort[index];
+	else
+		return NO_SYMBOL;
 }
 
 
